@@ -30,10 +30,10 @@ import numpy as np
 
 from art.attacks.evasion.adversarial_patch.adversarial_patch_numpy import AdversarialPatchNumpy
 from art.attacks.evasion.adversarial_patch.adversarial_patch_tensorflow import AdversarialPatchTensorFlowV2
+from art.attacks.evasion.adversarial_patch.adversarial_patch_pytorch import AdversarialPatchPyTorch
 from art.estimators.estimator import BaseEstimator, NeuralNetworkMixin
 from art.estimators.classification.classifier import ClassifierMixin
-
-from art.estimators.classification import TensorFlowV2Classifier
+from art.estimators.classification import TensorFlowV2Classifier, PyTorchClassifier
 from art.attacks.attack import EvasionAttack
 
 if TYPE_CHECKING:
@@ -92,10 +92,10 @@ class AdversarialPatch(EvasionAttack):
         :param verbose: Show progress bars.
         """
         super().__init__(estimator=classifier)
-        if self.estimator.clip_values is None:
+        if self.estimator.clip_values is None:  # pragma: no cover
             raise ValueError("Adversarial Patch attack requires a classifier with clip_values.")
 
-        self._attack: Union[AdversarialPatchTensorFlowV2, AdversarialPatchNumpy]
+        self._attack: Union[AdversarialPatchTensorFlowV2, AdversarialPatchPyTorch, AdversarialPatchNumpy]
         if isinstance(self.estimator, TensorFlowV2Classifier):
             self._attack = AdversarialPatchTensorFlowV2(
                 classifier=classifier,
@@ -108,6 +108,20 @@ class AdversarialPatch(EvasionAttack):
                 patch_shape=patch_shape,
                 verbose=verbose,
             )
+        elif isinstance(self.estimator, PyTorchClassifier):
+            self._attack = AdversarialPatchPyTorch(
+                classifier=classifier,
+                rotation_max=rotation_max,
+                scale_min=scale_min,
+                scale_max=scale_max,
+                distortion_scale_max=0.0,
+                learning_rate=learning_rate,
+                max_iter=max_iter,
+                batch_size=batch_size,
+                patch_shape=patch_shape,
+                patch_type="circle",
+                verbose=verbose,
+            )
         else:
             self._attack = AdversarialPatchNumpy(
                 classifier=classifier,
@@ -127,18 +141,22 @@ class AdversarialPatch(EvasionAttack):
 
         :param x: An array with the original input images of shape NHWC or NCHW or input videos of shape NFHWC or NFCHW.
         :param y: An array with the original true labels.
-        :param mask: An boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
+        :param mask: A boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
                      (N, H, W) without their channel dimensions. Any features for which the mask is True can be the
                      center location of the patch during sampling.
         :type mask: `np.ndarray`
+        :param reset_patch: If `True` reset patch to initial values of mean of minimal and maximal clip value, else if
+                            `False` (default) restart from previous patch values created by previous call to `generate`
+                            or mean of minimal and maximal clip value if first call to `generate`.
+        :type reset_patch: bool
         :return: An array with adversarial patch and an array of the patch mask.
         """
         logger.info("Creating adversarial patch.")
 
-        if y is None:
+        if y is None:  # pragma: no cover
             raise ValueError("Adversarial Patch attack requires target values `y`.")
 
-        if len(x.shape) == 2:
+        if len(x.shape) == 2:  # pragma: no cover
             raise ValueError(
                 "Feature vectors detected. The adversarial patch can only be applied to data with spatial "
                 "dimensions."
@@ -146,7 +164,9 @@ class AdversarialPatch(EvasionAttack):
 
         return self._attack.generate(x=x, y=y, **kwargs)
 
-    def apply_patch(self, x: np.ndarray, scale: float, patch_external: Optional[np.ndarray] = None) -> np.ndarray:
+    def apply_patch(
+        self, x: np.ndarray, scale: float, patch_external: Optional[np.ndarray] = None, **kwargs
+    ) -> np.ndarray:
         """
         A function to apply the learned adversarial patch to images or videos.
 
@@ -155,7 +175,28 @@ class AdversarialPatch(EvasionAttack):
         :param patch_external: External patch to apply to images `x`.
         :return: The patched instances.
         """
-        return self._attack.apply_patch(x, scale, patch_external=patch_external)
+        return self._attack.apply_patch(x, scale, patch_external=patch_external, **kwargs)
+
+    def reset_patch(self, initial_patch_value: Optional[Union[float, np.ndarray]]) -> None:
+        """
+        Reset the adversarial patch.
+
+        :param initial_patch_value: Patch value to use for resetting the patch.
+        """
+        self._attack.reset_patch(initial_patch_value=initial_patch_value)
+
+    def insert_transformed_patch(self, x: np.ndarray, patch: np.ndarray, image_coords: np.ndarray):
+        """
+        Insert patch to image based on given or selected coordinates.
+
+        :param x: The image to insert the patch.
+        :param patch: The patch to be transformed and inserted.
+        :param image_coords: The coordinates of the 4 corners of the transformed, inserted patch of shape
+            [[x1, y1], [x2, y2], [x3, y3], [x4, y4]] in pixel units going in clockwise direction, starting with upper
+            left corner.
+        :return: The input `x` with the patch inserted.
+        """
+        return self._attack.insert_transformed_patch(x, patch, image_coords)
 
     def set_params(self, **kwargs) -> None:
         super().set_params(**kwargs)
@@ -181,6 +222,7 @@ class AdversarialPatch(EvasionAttack):
 
         if not isinstance(self._attack.learning_rate, float):
             raise ValueError("The learning rate must be of type float.")
+
         if not self._attack.learning_rate > 0.0:
             raise ValueError("The learning rate must be greater than 0.0.")
 
