@@ -28,14 +28,17 @@ import math
 from typing import Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.attacks.attack import EvasionAttack
+from art.attacks.evasion.adversarial_patch.utils import insert_transformed_patch
 from art.estimators.estimator import BaseEstimator, NeuralNetworkMixin
 from art.estimators.classification.classifier import ClassifierMixin
-from art.utils import check_and_transform_label_format
+from art.utils import check_and_transform_label_format, is_probability, to_categorical
+from art.summary_writer import SummaryWriter
 
 if TYPE_CHECKING:
+    # pylint: disable=C0412
     import tensorflow as tf
 
     from art.utils import CLASSIFIER_NEURALNETWORK_TYPE
@@ -58,6 +61,7 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         "max_iter",
         "batch_size",
         "patch_shape",
+        "summary_writer",
         "verbose",
     ]
 
@@ -73,6 +77,7 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         max_iter: int = 500,
         batch_size: int = 16,
         patch_shape: Optional[Tuple[int, int, int]] = None,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ):
         """
@@ -89,11 +94,18 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         :param max_iter: The number of optimization steps.
         :param batch_size: The size of the training batch.
         :param patch_shape: The shape of the adversarial patch as a tuple of shape HWC (width, height, nb_channels).
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         :param verbose: Show progress bars.
         """
         import tensorflow as tf  # lgtm [py/repeated-import]
 
-        super().__init__(estimator=classifier)
+        super().__init__(estimator=classifier, summary_writer=summary_writer)
         self.rotation_max = rotation_max
         self.scale_min = scale_min
         self.scale_max = scale_max
@@ -108,9 +120,11 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         self.verbose = verbose
         self._check_params()
 
-        if self.estimator.channels_first:
+        if self.estimator.channels_first:  # pragma: no cover
             raise ValueError("Color channel needs to be in last dimension.")
 
+        self.use_logits: Optional[bool] = None
+
         self.i_h_patch = 0
         self.i_w_patch = 1
 
@@ -122,10 +136,12 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
             self.i_h = 1
             self.i_w = 2
 
-        if self.patch_shape[0] != self.patch_shape[1]:
+        if self.patch_shape[0] != self.patch_shape[1]:  # pragma: no cover
             raise ValueError("Patch height and width need to be the same.")
 
-        if not (self.estimator.postprocessing_defences is None or self.estimator.postprocessing_defences == []):
+        if not (  # pragma: no cover
+            self.estimator.postprocessing_defences is None or self.estimator.postprocessing_defences == []
+        ):
             raise ValueError(
                 "Framework-specific implementation of Adversarial Patch attack does not yet support "
                 + "postprocessing defences."
@@ -134,16 +150,16 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         mean_value = (self.estimator.clip_values[1] - self.estimator.clip_values[0]) / 2.0 + self.estimator.clip_values[
             0
         ]
-        initial_value = np.ones(self.patch_shape) * mean_value
+        self._initial_value = np.ones(self.patch_shape) * mean_value
         self._patch = tf.Variable(
-            initial_value=initial_value,
+            initial_value=self._initial_value,
             shape=self.patch_shape,
             dtype=tf.float32,
             constraint=lambda x: tf.clip_by_value(x, self.estimator.clip_values[0], self.estimator.clip_values[1]),
         )
 
-        self._train_op = tf.keras.optimizers.SGD(
-            learning_rate=self.learning_rate, momentum=0.0, nesterov=False, name="SGD"
+        self._train_op = tf.keras.optimizers.Adam(
+            learning_rate=self.learning_rate, beta_1=0.9, beta_2=0.999, epsilon=1e-07, amsgrad=False, name="Adam"
         )
 
     def _train_step(
@@ -170,26 +186,28 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
 
         return loss
 
-    def _probabilities(self, images: "tf.Tensor", mask: Optional["tf.Tensor"]) -> "tf.Tensor":
+    def _predictions(self, images: "tf.Tensor", mask: Optional["tf.Tensor"]) -> "tf.Tensor":
         import tensorflow as tf  # lgtm [py/repeated-import]
 
         patched_input = self._random_overlay(images, self._patch, mask=mask)
 
         patched_input = tf.clip_by_value(
-            patched_input, clip_value_min=self.estimator.clip_values[0], clip_value_max=self.estimator.clip_values[1],
+            patched_input,
+            clip_value_min=self.estimator.clip_values[0],
+            clip_value_max=self.estimator.clip_values[1],
         )
 
-        probabilities = self.estimator._predict_framework(patched_input)
+        predictions = self.estimator._predict_framework(patched_input)  # pylint: disable=W0212
 
-        return probabilities
+        return predictions
 
     def _loss(self, images: "tf.Tensor", target: "tf.Tensor", mask: Optional["tf.Tensor"]) -> "tf.Tensor":
         import tensorflow as tf  # lgtm [py/repeated-import]
 
-        probabilities = self._probabilities(images, mask)
+        predictions = self._predictions(images, mask)
 
         self._loss_per_example = tf.keras.losses.categorical_crossentropy(
-            y_true=target, y_pred=probabilities, from_logits=False, label_smoothing=0
+            y_true=target, y_pred=predictions, from_logits=self.use_logits, label_smoothing=0
         )
 
         loss = tf.reduce_mean(self._loss_per_example)
@@ -247,7 +265,7 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         pad_w_before = int((self.image_shape[self.i_w] - image_mask.shape[self.i_w_patch + 1]) / 2)
         pad_w_after = int(self.image_shape[self.i_w] - pad_w_before - image_mask.shape[self.i_w_patch + 1])
 
-        image_mask = tf.pad(
+        image_mask = tf.pad(  # pylint: disable=E1123
             image_mask,
             paddings=tf.constant([[0, 0], [pad_h_before, pad_h_after], [pad_w_before, pad_w_after], [0, 0]]),
             mode="CONSTANT",
@@ -269,7 +287,7 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
             name=None,
         )
 
-        padded_patch = tf.pad(
+        padded_patch = tf.pad(  # pylint: disable=E1123
             padded_patch,
             paddings=tf.constant([[0, 0], [pad_h_before, pad_h_after], [pad_w_before, pad_w_after], [0, 0]]),
             mode="CONSTANT",
@@ -322,7 +340,10 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
 
             # Rotation
             rotation_matrix = np.array(
-                [[math.cos(-phi_rotate), -math.sin(-phi_rotate)], [math.sin(-phi_rotate), math.cos(-phi_rotate)],]
+                [
+                    [math.cos(-phi_rotate), -math.sin(-phi_rotate)],
+                    [math.sin(-phi_rotate), math.cos(-phi_rotate)],
+                ]
             )
 
             # Scale
@@ -342,11 +363,27 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
             transform_vectors.append([a_0, a_1, x_origin_delta, b_0, b_1, y_origin_delta, 0, 0])
             translation_vectors.append([1, 0, -x_shift, 0, 1, -y_shift, 0, 0])
 
-        image_mask = tfa.image.transform(image_mask, transform_vectors, "BILINEAR",)
-        padded_patch = tfa.image.transform(padded_patch, transform_vectors, "BILINEAR",)
+        image_mask = tfa.image.transform(
+            image_mask,
+            transform_vectors,
+            "BILINEAR",
+        )
+        padded_patch = tfa.image.transform(
+            padded_patch,
+            transform_vectors,
+            "BILINEAR",
+        )
 
-        image_mask = tfa.image.transform(image_mask, translation_vectors, "BILINEAR",)
-        padded_patch = tfa.image.transform(padded_patch, translation_vectors, "BILINEAR",)
+        image_mask = tfa.image.transform(
+            image_mask,
+            translation_vectors,
+            "BILINEAR",
+        )
+        padded_patch = tfa.image.transform(
+            padded_patch,
+            translation_vectors,
+            "BILINEAR",
+        )
 
         if self.nb_dims == 4:
             image_mask = tf.stack([image_mask] * images.shape[1], axis=1)
@@ -365,10 +402,14 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
 
         :param x: An array with the original input images of shape NHWC or input videos of shape NFHWC.
         :param y: An array with the original true labels.
-        :param mask: An boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
+        :param mask: A boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
                      (N, H, W) without their channel dimensions. Any features for which the mask is True can be the
                      center location of the patch during sampling.
         :type mask: `np.ndarray`
+        :param reset_patch: If `True` reset patch to initial values of mean of minimal and maximal clip value, else if
+                            `False` (default) restart from previous patch values created by previous call to `generate`
+                            or mean of minimal and maximal clip value if first call to `generate`.
+        :type reset_patch: bool
         :return: An array with adversarial patch and an array of the patch mask.
         """
         import tensorflow as tf  # lgtm [py/repeated-import]
@@ -377,13 +418,77 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         mask = kwargs.get("mask")
         if mask is not None:
             mask = mask.copy()
-        if mask is not None and (
-            (mask.dtype != np.bool)
+        mask = self._check_mask(mask=mask, x=x)
+
+        if y is None:  # pragma: no cover
+            logger.info("Setting labels to estimator predictions and running untargeted attack because `y=None`.")
+            y = to_categorical(np.argmax(self.estimator.predict(x=x), axis=1), nb_classes=self.estimator.nb_classes)
+            self.targeted = False
+        else:
+            self.targeted = True
+
+        if kwargs.get("reset_patch"):
+            self.reset_patch(initial_patch_value=self._initial_value)
+
+        y = check_and_transform_label_format(labels=y, nb_classes=self.estimator.nb_classes)
+
+        # check if logits or probabilities
+        y_pred = self.estimator.predict(x=x[[0]])
+
+        if is_probability(y_pred):
+            self.use_logits = False
+        else:
+            self.use_logits = True
+
+        if mask is None:
+            if shuffle:
+                dataset = tf.data.Dataset.from_tensor_slices((x, y)).shuffle(10000).batch(self.batch_size)
+            else:
+                dataset = tf.data.Dataset.from_tensor_slices((x, y)).batch(self.batch_size)
+        else:
+            if shuffle:
+                dataset = tf.data.Dataset.from_tensor_slices((x, y, mask)).shuffle(10000).batch(self.batch_size)
+            else:
+                dataset = tf.data.Dataset.from_tensor_slices((x, y, mask)).batch(self.batch_size)
+
+        for i_iter in trange(self.max_iter, desc="Adversarial Patch TensorFlow v2", disable=not self.verbose):
+            if mask is None:
+                counter = 0
+                for images, target in dataset:
+                    counter += 1
+                    _ = self._train_step(images=images, target=target, mask=None)
+            else:
+                for images, target, mask_i in dataset:
+                    _ = self._train_step(images=images, target=target, mask=mask_i)
+
+            # Write summary
+            if self.summary_writer is not None:  # pragma: no cover
+                x_patched = self._random_overlay(images=x, patch=self._patch, mask=mask)
+
+                self.summary_writer.update(
+                    batch_id=0,
+                    global_step=i_iter,
+                    grad=None,
+                    patch=self._patch.numpy().transpose((2, 0, 1)),
+                    estimator=self.estimator,
+                    x=x_patched,
+                    y=y,
+                    targeted=self.targeted,
+                )
+
+        if self.summary_writer is not None:
+            self.summary_writer.reset()
+
+        return (
+            self._patch.numpy(),
+            self._get_circular_patch_mask(nb_samples=1).numpy()[0],
+        )
+
+    def _check_mask(self, mask: np.ndarray, x: np.ndarray) -> np.ndarray:
+        if mask is not None and (  # pragma: no cover
+            (mask.dtype != bool)
             or not (mask.shape[0] == 1 or mask.shape[0] == x.shape[0])
-            or not (
-                (mask.shape[1] == x.shape[1] and mask.shape[2] == x.shape[2])
-                or (mask.shape[1] == x.shape[2] and mask.shape[2] == x.shape[3])
-            )
+            or not (mask.shape[1] == x.shape[self.i_h + 1] and mask.shape[2] == x.shape[self.i_w + 1])
         ):
             raise ValueError(
                 "The shape of `mask` has to be equal to the shape of a single samples (1, H, W) or the"
@@ -393,49 +498,7 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         if mask is not None and mask.shape[0] == 1:
             mask = np.repeat(mask, repeats=x.shape[0], axis=0)
 
-        y = check_and_transform_label_format(labels=y, nb_classes=self.estimator.nb_classes)
-
-        if mask is None:
-            if shuffle:
-                dataset = (
-                    tf.data.Dataset.from_tensor_slices((x, y))
-                    .shuffle(10000)
-                    .batch(self.batch_size)
-                    .repeat(math.ceil(x.shape[0] / self.batch_size))
-                )
-            else:
-                dataset = (
-                    tf.data.Dataset.from_tensor_slices((x, y))
-                    .batch(self.batch_size)
-                    .repeat(math.ceil(x.shape[0] / self.batch_size))
-                )
-        else:
-            if shuffle:
-                dataset = (
-                    tf.data.Dataset.from_tensor_slices((x, y, mask))
-                    .shuffle(10000)
-                    .batch(self.batch_size)
-                    .repeat(math.ceil(x.shape[0] / self.batch_size))
-                )
-            else:
-                dataset = (
-                    tf.data.Dataset.from_tensor_slices((x, y, mask))
-                    .batch(self.batch_size)
-                    .repeat(math.ceil(x.shape[0] / self.batch_size))
-                )
-
-        for _ in trange(self.max_iter, desc="Adversarial Patch TensorFlow v2", disable=not self.verbose):
-            if mask is None:
-                for images, target in dataset:
-                    _ = self._train_step(images=images, target=target, mask=None)
-            else:
-                for images, target, mask_i in dataset:
-                    _ = self._train_step(images=images, target=target, mask=mask_i)
-
-        return (
-            self._patch.numpy(),
-            self._get_circular_patch_mask(nb_samples=1).numpy()[0],
-        )
+        return mask
 
     def apply_patch(
         self,
@@ -457,14 +520,36 @@ class AdversarialPatchTensorFlowV2(EvasionAttack):
         """
         if mask is not None:
             mask = mask.copy()
+        mask = self._check_mask(mask=mask, x=x)
         patch = patch_external if patch_external is not None else self._patch
         return self._random_overlay(images=x, patch=patch, scale=scale, mask=mask).numpy()
 
-    def reset_patch(self, initial_patch_value: np.ndarray) -> None:
+    def reset_patch(self, initial_patch_value: Optional[Union[float, np.ndarray]] = None) -> None:
         """
         Reset the adversarial patch.
 
         :param initial_patch_value: Patch value to use for resetting the patch.
         """
-        initial_value = np.ones(self.patch_shape) * initial_patch_value
-        self._patch.assign(np.ones(shape=self.patch_shape) * initial_value)
+        if initial_patch_value is None:
+            self._patch.assign(self._initial_value)
+        elif isinstance(initial_patch_value, float):
+            initial_value = np.ones(self.patch_shape) * initial_patch_value
+            self._patch.assign(initial_value)
+        elif self._patch.shape == initial_patch_value.shape:
+            self._patch.assign(initial_patch_value)
+        else:  # pragma: no cover
+            raise ValueError("Unexpected value for initial_patch_value.")
+
+    @staticmethod
+    def insert_transformed_patch(x: np.ndarray, patch: np.ndarray, image_coords: np.ndarray):
+        """
+        Insert patch to image based on given or selected coordinates.
+
+        :param x: The image to insert the patch.
+        :param patch: The patch to be transformed and inserted.
+        :param image_coords: The coordinates of the 4 corners of the transformed, inserted patch of shape
+            [[x1, y1], [x2, y2], [x3, y3], [x4, y4]] in pixel units going in clockwise direction, starting with upper
+            left corner.
+        :return: The input `x` with the patch inserted.
+        """
+        return insert_transformed_patch(x, patch, image_coords)
