@@ -27,7 +27,7 @@ import logging
 from typing import Optional, Tuple, TYPE_CHECKING
 
 import numpy as np
-from tqdm import tqdm, trange
+from tqdm.auto import tqdm, trange
 
 from art.attacks.attack import EvasionAttack
 from art.config import ART_NUMPY_DTYPE
@@ -67,6 +67,7 @@ class BoundaryAttack(EvasionAttack):
     def __init__(
         self,
         estimator: "CLASSIFIER_TYPE",
+        batch_size: int = 64,
         targeted: bool = True,
         delta: float = 0.01,
         epsilon: float = 0.01,
@@ -75,13 +76,14 @@ class BoundaryAttack(EvasionAttack):
         num_trial: int = 25,
         sample_size: int = 20,
         init_size: int = 100,
-        min_epsilon: Optional[float] = None,
+        min_epsilon: float = 0.0,
         verbose: bool = True,
     ) -> None:
         """
         Create a boundary attack instance.
 
         :param estimator: A trained classifier.
+        :param batch_size: The size of the batch used by the estimator during inference.
         :param targeted: Should the attack target one specific class.
         :param delta: Initial step size for the orthogonal step.
         :param epsilon: Initial step size for the step towards the target.
@@ -104,7 +106,7 @@ class BoundaryAttack(EvasionAttack):
         self.sample_size = sample_size
         self.init_size = init_size
         self.min_epsilon = min_epsilon
-        self.batch_size = 1
+        self.batch_size = batch_size
         self.verbose = verbose
         self._check_params()
 
@@ -123,6 +125,11 @@ class BoundaryAttack(EvasionAttack):
         """
         y = check_and_transform_label_format(y, self.estimator.nb_classes, return_one_hot=False)
 
+        if y is not None and self.estimator.nb_classes == 2 and y.shape[1] == 1:
+            raise ValueError(  # pragma: no cover
+                "This attack has not yet been tested for binary classification with a single output classifier."
+            )
+
         # Get clip_min and clip_max from the classifier or infer them from data
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
@@ -142,7 +149,7 @@ class BoundaryAttack(EvasionAttack):
             x_adv_init = [None] * len(x)
 
         # Assert that, if attack is targeted, y is provided
-        if self.targeted and y is None:
+        if self.targeted and y is None:  # pragma: no cover
             raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
         # Some initial setups
@@ -182,7 +189,14 @@ class BoundaryAttack(EvasionAttack):
         return x_adv
 
     def _perturb(
-        self, x: np.ndarray, y: int, y_p: int, init_pred: int, adv_init: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        x: np.ndarray,
+        y: int,
+        y_p: int,
+        init_pred: int,
+        adv_init: np.ndarray,
+        clip_min: float,
+        clip_max: float,
     ) -> np.ndarray:
         """
         Internal attack function for one example.
@@ -204,7 +218,16 @@ class BoundaryAttack(EvasionAttack):
             return x
 
         # If an initial adversarial example found, then go with boundary attack
-        x_adv = self._attack(initial_sample[0], x, initial_sample[1], self.delta, self.epsilon, clip_min, clip_max,)
+        x_adv = self._attack(
+            initial_sample[0],
+            x,
+            y_p,
+            initial_sample[1],
+            self.delta,
+            self.epsilon,
+            clip_min,
+            clip_max,
+        )
 
         return x_adv
 
@@ -212,6 +235,7 @@ class BoundaryAttack(EvasionAttack):
         self,
         initial_sample: np.ndarray,
         original_sample: np.ndarray,
+        y_p: int,
         target: int,
         initial_delta: float,
         initial_epsilon: float,
@@ -223,6 +247,7 @@ class BoundaryAttack(EvasionAttack):
 
         :param initial_sample: An initial adversarial example.
         :param original_sample: The original input.
+        :param y_p: The predicted label of the original input.
         :param target: The target label.
         :param initial_delta: Initial step size for the orthogonal step.
         :param initial_epsilon: Initial step size for the step towards the target.
@@ -247,8 +272,16 @@ class BoundaryAttack(EvasionAttack):
                     potential_adv = np.clip(potential_adv, clip_min, clip_max)
                     potential_advs.append(potential_adv)
 
-                preds = np.argmax(self.estimator.predict(np.array(potential_advs), batch_size=self.batch_size), axis=1,)
-                satisfied = preds == target
+                preds = np.argmax(
+                    self.estimator.predict(np.array(potential_advs), batch_size=self.batch_size),
+                    axis=1,
+                )
+
+                if self.targeted:
+                    satisfied = preds == target
+                else:
+                    satisfied = preds != y_p
+
                 delta_ratio = np.mean(satisfied)
 
                 if delta_ratio < 0.2:
@@ -259,7 +292,7 @@ class BoundaryAttack(EvasionAttack):
                 if delta_ratio > 0:
                     x_advs = np.array(potential_advs)[np.where(satisfied)[0]]
                     break
-            else:
+            else:  # pragma: no cover
                 logger.warning("Adversarial example found but not optimal.")
                 return x_adv
 
@@ -269,8 +302,16 @@ class BoundaryAttack(EvasionAttack):
                 perturb *= self.curr_epsilon
                 potential_advs = x_advs + perturb
                 potential_advs = np.clip(potential_advs, clip_min, clip_max)
-                preds = np.argmax(self.estimator.predict(potential_advs, batch_size=self.batch_size), axis=1,)
-                satisfied = preds == target
+                preds = np.argmax(
+                    self.estimator.predict(potential_advs, batch_size=self.batch_size),
+                    axis=1,
+                )
+
+                if self.targeted:
+                    satisfied = preds == target
+                else:
+                    satisfied = preds != y_p
+
                 epsilon_ratio = np.mean(satisfied)
 
                 if epsilon_ratio < 0.2:
@@ -279,14 +320,14 @@ class BoundaryAttack(EvasionAttack):
                     self.curr_epsilon /= self.step_adapt
 
                 if epsilon_ratio > 0:
-                    x_adv = potential_advs[np.where(satisfied)[0][0]]
+                    x_adv = self._best_adv(original_sample, potential_advs[np.where(satisfied)[0]])
                     self.curr_adv = x_adv
                     break
-            else:
+            else:  # pragma: no cover
                 logger.warning("Adversarial example found but not optimal.")
-                return x_advs[0]
+                return self._best_adv(original_sample, x_advs)
 
-            if self.min_epsilon is not None and self.curr_epsilon < self.min_epsilon:
+            if self.curr_epsilon < self.min_epsilon:
                 return x_adv
 
         return x_adv
@@ -322,7 +363,14 @@ class BoundaryAttack(EvasionAttack):
         return perturb
 
     def _init_sample(
-        self, x: np.ndarray, y: int, y_p: int, init_pred: int, adv_init: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        x: np.ndarray,
+        y: int,
+        y_p: int,
+        init_pred: int,
+        adv_init: np.ndarray,
+        clip_min: float,
+        clip_max: float,
     ) -> Optional[Tuple[np.ndarray, int]]:
         """
         Find initial adversarial example for the attack.
@@ -352,7 +400,8 @@ class BoundaryAttack(EvasionAttack):
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class == y:
@@ -372,7 +421,8 @@ class BoundaryAttack(EvasionAttack):
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class != y_p:
@@ -380,22 +430,35 @@ class BoundaryAttack(EvasionAttack):
 
                     logger.info("Found initial adversarial image for untargeted attack.")
                     break
-            else:
+            else:  # pragma: no cover
                 logger.warning("Failed to draw a random image that is adversarial, attack failed.")
 
         return initial_sample
 
+    @staticmethod
+    def _best_adv(original_sample: np.ndarray, potential_advs: np.ndarray) -> np.ndarray:
+        """
+        From the potential adversarial examples, find the one that has the minimum L2 distance from the original sample
+
+        :param original_sample: The original input.
+        :param potential_advs: Array containing the potential adversarial examples
+        :return: The adversarial example that has the minimum L2 distance from the original input
+        """
+        shape = potential_advs.shape
+        min_idx = np.linalg.norm(original_sample.flatten() - potential_advs.reshape(shape[0], -1), axis=1).argmin()
+        return potential_advs[min_idx]
+
     def _check_params(self) -> None:
-        if not isinstance(self.max_iter, (int, np.int)) or self.max_iter < 0:
+        if not isinstance(self.max_iter, int) or self.max_iter < 0:
             raise ValueError("The number of iterations must be a non-negative integer.")
 
-        if not isinstance(self.num_trial, (int, np.int)) or self.num_trial < 0:
+        if not isinstance(self.num_trial, int) or self.num_trial < 0:
             raise ValueError("The number of trials must be a non-negative integer.")
 
-        if not isinstance(self.sample_size, (int, np.int)) or self.sample_size <= 0:
+        if not isinstance(self.sample_size, int) or self.sample_size <= 0:
             raise ValueError("The number of samples must be a positive integer.")
 
-        if not isinstance(self.init_size, (int, np.int)) or self.init_size <= 0:
+        if not isinstance(self.init_size, int) or self.init_size <= 0:
             raise ValueError("The number of initial trials must be a positive integer.")
 
         if self.epsilon <= 0:
@@ -407,8 +470,8 @@ class BoundaryAttack(EvasionAttack):
         if self.step_adapt <= 0 or self.step_adapt >= 1:
             raise ValueError("The adaptation factor must be in the range (0, 1).")
 
-        if self.min_epsilon is not None and (isinstance(self.min_epsilon, float) or self.min_epsilon <= 0):
-            raise ValueError("The minimum epsilon must be a positive float.")
+        if not isinstance(self.min_epsilon, (float, int)) or self.min_epsilon < 0:
+            raise ValueError("The minimum epsilon must be non-negative.")
 
         if not isinstance(self.verbose, bool):
             raise ValueError("The argument `verbose` has to be of type bool.")
