@@ -39,10 +39,15 @@
 # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+"""
+This module implements Brendel and Bethge attack.
+"""
+# pylint: disable=C0103,R0201,C0115,C0116,C0144,C0302,W0612,W0613,E1120,R1716,R1705,R1723,R1720
 from typing import Union, Optional, Tuple, TYPE_CHECKING
+import logging
 
 import numpy as np
-import logging
+from numba.experimental import jitclass
 
 from art import config
 from art.utils import get_labels_np_array, check_and_transform_label_format, is_probability
@@ -58,7 +63,8 @@ EPS = 1e-10
 logger = logging.getLogger(__name__)
 
 
-class BFGSB(object):
+@jitclass(spec=[])
+class BFGSB:  # pragma: no cover
     def __init__(self):
         pass
 
@@ -68,7 +74,7 @@ class BFGSB(object):
         if maxiter is None:
             maxiter = N * 200
 
-        l = bounds[:, 0]  # noqa: E741
+        var_l = bounds[:, 0]
         u = bounds[:, 1]
 
         func_calls = 0
@@ -96,7 +102,7 @@ class BFGSB(object):
                 if _gfk[v] < 0:
                     gv = max(qk[v] - u[v], _gfk[v])
                 else:
-                    gv = min(qk[v] - l[v], _gfk[v])
+                    gv = min(qk[v] - var_l[v], _gfk[v])
 
                 if pg_norm < np.abs(gv):
                     pg_norm = np.abs(gv)
@@ -105,13 +111,19 @@ class BFGSB(object):
                 break
 
             # get cauchy point
-            x_cp = self._cauchy_point(qk, l, u, _gfk.copy(), Hk)
-            qk1 = self._subspace_min(qk, l, u, x_cp, _gfk.copy(), Hk)
+            x_cp = self._cauchy_point(qk, var_l, u, _gfk.copy(), Hk)
+            qk1 = self._subspace_min(qk, var_l, u, x_cp, _gfk.copy(), Hk)
             pk = qk1 - qk
 
-            (alpha_k, fc, gc, old_fval, old_old_fval, gfkp1, fnev,) = self._line_search_wolfe(
-                fun_and_jac, qk, pk, _gfk, old_fval, old_old_fval, l, u, args
-            )
+            (
+                alpha_k,
+                fc,
+                gc,
+                old_fval,
+                old_old_fval,
+                gfkp1,
+                fnev,
+            ) = self._line_search_wolfe(fun_and_jac, qk, pk, _gfk, old_fval, old_old_fval, var_l, u, args)
             func_calls += fnev
 
             if alpha_k is None:
@@ -120,7 +132,7 @@ class BFGSB(object):
             if np.abs(old_fval - old_old_fval) <= (ftol + ftol * np.abs(old_fval)):
                 break
 
-            qkp1 = self._project(qk + alpha_k * pk, l, u)
+            qkp1 = self._project(qk + alpha_k * pk, var_l, u)
 
             if gfkp1 is None:
                 _, gfkp1 = fun_and_jac(qkp1, *args)
@@ -164,7 +176,7 @@ class BFGSB(object):
 
         return qk
 
-    def _cauchy_point(self, x, l, u, g, B):  # noqa: E741
+    def _cauchy_point(self, x, var_l, u, g, B):
         # finds the cauchy point for q(x)=x'Gx+x'd s$t. l<=x<=u
         # g=G*x+d #gradient of q(x)
         # converted from r-code: https://github.com/andrewhooker/PopED/blob/master/R/cauchy_point.R
@@ -176,7 +188,7 @@ class BFGSB(object):
             if g[i] < 0:
                 t[i] = (x[i] - u[i]) / g[i]
             elif g[i] > 0:
-                t[i] = (x[i] - l[i]) / g[i]
+                t[i] = (x[i] - var_l[i]) / g[i]
             elif g[i] == 0:
                 t[i] = np.inf
 
@@ -218,11 +230,11 @@ class BFGSB(object):
 
         return x_cp
 
-    def _subspace_min(self, x, l, u, x_cp, d, G):  # noqa: E741
+    def _subspace_min(self, x, var_l, u, x_cp, d, G):
         # converted from r-code: https://github.com/andrewhooker/PopED/blob/master/R/subspace_min.R
         n = x.shape[0]
         Z = np.eye(n)
-        fixed = (x_cp <= l + 1e-8) + (x_cp >= u - 1e8)
+        fixed = (x_cp <= var_l + 1e-8) + (x_cp >= u - 1e8)
 
         if np.all(fixed):
             x = x_cp
@@ -239,7 +251,7 @@ class BFGSB(object):
         for i in np.arange(n)[~fixed]:
             dk = d[i]
             if dk < 0:
-                temp2 = l[i] - x_cp[i]
+                temp2 = var_l[i] - x_cp[i]
                 if temp2 >= 0:
                     temp1 = 0
                 else:
@@ -259,18 +271,33 @@ class BFGSB(object):
 
         return x_cp + alpha * Z.dot(d[~fixed])
 
-    def _project(self, q, l, u):  # noqa: E741
+    def _project(self, q, var_l, u):
         N = q.shape[0]
         for k in range(N):
-            if q[k] < l[k]:
-                q[k] = l[k]
+            if q[k] < var_l[k]:
+                q[k] = var_l[k]
             elif q[k] > u[k]:
                 q[k] = u[k]
 
         return q
 
     def _line_search_armijo(
-        self, fun_and_jac, pt, dpt, func_calls, m, gk, l, u, x0, x, b, min_, max_, c, r,  # noqa: E741
+        self,
+        fun_and_jac,
+        pt,
+        dpt,
+        func_calls,
+        m,
+        gk,
+        var_l,
+        u,
+        x0,
+        x,
+        b,
+        min_,
+        max_,
+        c,
+        r,
     ):
         ls_rho = 0.6
         ls_c = 1e-4
@@ -279,7 +306,7 @@ class BFGSB(object):
         t = m * ls_c
 
         for k2 in range(100):
-            ls_pt = self._project(pt + ls_alpha * dpt, l, u)
+            ls_pt = self._project(pt + ls_alpha * dpt, var_l, u)
 
             gkp1, dgkp1 = fun_and_jac(ls_pt, x0, x, b, min_, max_, c, r)
             func_calls += 1
@@ -291,8 +318,17 @@ class BFGSB(object):
 
         return ls_alpha, ls_pt, gkp1, dgkp1, func_calls
 
-    def _line_search_wolfe(  # noqa: C901
-        self, fun_and_jac, xk, pk, gfk, old_fval, old_old_fval, l, u, args,  # noqa: #E741
+    def _line_search_wolfe(
+        self,
+        fun_and_jac,
+        xk,
+        pk,
+        gfk,
+        old_fval,
+        old_old_fval,
+        var_l,
+        u,
+        args,
     ):
         """Find alpha that satisfies strong Wolfe conditions.
         Uses the line search algorithm to enforce strong Wolfe conditions
@@ -330,7 +366,7 @@ class BFGSB(object):
             old_fval = old_old_fval
             fprime_star = None
 
-        _xkp1 = self._project(xk + alpha1 * pk, l, u)
+        _xkp1 = self._project(xk + alpha1 * pk, var_l, u)
         phi_a1, _ls_ingfk = fun_and_jac(_xkp1, *args)
         _ls_fc += 1
         # derphi_a1 = phiprime(alpha1)  evaluated below
@@ -388,7 +424,7 @@ class BFGSB(object):
                             a_j = a_lo + 0.5 * dalpha
 
                     # Check new value of a_j
-                    _xkp1 = self._project(xk + a_j * pk, l, u)
+                    _xkp1 = self._project(xk + a_j * pk, var_l, u)
                     # if _xkp1[1] < 0:
                     #     _xkp1[1] = 0
                     phi_aj, _ls_ingfk = fun_and_jac(_xkp1, *args)
@@ -439,7 +475,7 @@ class BFGSB(object):
             if i > maxiter:
                 break
 
-            _xkp1 = self._project(xk + alpha1 * pk, l, u)
+            _xkp1 = self._project(xk + alpha1 * pk, var_l, u)
             _, _ls_ingfk = fun_and_jac(_xkp1, *args)
             derphi_a1 = 0
             for v in range(N):
@@ -497,7 +533,7 @@ class BFGSB(object):
                             a_j = a_lo + 0.5 * dalpha
 
                     # Check new value of a_j
-                    _xkp1 = self._project(xk + a_j * pk, l, u)
+                    _xkp1 = self._project(xk + a_j * pk, var_l, u)
                     phi_aj, _ls_ingfk = fun_and_jac(_xkp1, *args)
                     derphi_aj = 0
                     for v in range(N):
@@ -545,7 +581,7 @@ class BFGSB(object):
             alpha0 = alpha1
             alpha1 = alpha2
             phi_a0 = phi_a1
-            _xkp1 = self._project(xk + alpha1 * pk, l, u)
+            _xkp1 = self._project(xk + alpha1 * pk, var_l, u)
             phi_a1, _ls_ingfk = fun_and_jac(_xkp1, *args)
             _ls_fc += 1
             derphi_a0 = derphi_a1
@@ -602,7 +638,7 @@ class BFGSB(object):
         return xmin
 
 
-class Optimizer(object):
+class Optimizer:  # pragma: no cover
     """
     Base class for the trust-region optimization. If feasible, this optimizer solves the problem
 
@@ -839,8 +875,12 @@ class Optimizer(object):
         return _delta
 
 
-class L2Optimizer(Optimizer):
-    def optimize_distance_s_t_boundary_and_trustregion(self, x0, x, b, min_, max_, c, r):  # noqa: C901
+spec = [("bfgsb", BFGSB.class_type.instance_type)]  # type: ignore
+
+
+@jitclass(spec=spec)
+class L2Optimizer(Optimizer):  # pragma: no cover
+    def optimize_distance_s_t_boundary_and_trustregion(self, x0, x, b, min_, max_, c, r):
         """
         Solves the L2 trust region problem
 
@@ -1029,7 +1069,8 @@ class L2Optimizer(Optimizer):
         return np.linalg.norm(x0 - x) ** 2
 
 
-class L1Optimizer(Optimizer):
+@jitclass(spec=spec)
+class L1Optimizer(Optimizer):  # pragma: no cover
     def fun_and_jac(self, params, x0, x, b, min_, max_, c, r):
         lam, mu = params
         # arg min_delta ||delta - dx||_1 + lam * b^T delta + mu * ||delta||_2^2  s.t.  min <= delta + x <= max
@@ -1172,7 +1213,8 @@ class L1Optimizer(Optimizer):
         return np.abs(x0 - x).sum()
 
 
-class LinfOptimizer(Optimizer):
+@jitclass(spec=spec)
+class LinfOptimizer(Optimizer):  # pragma: no cover
     def optimize_distance_s_t_boundary_and_trustregion(self, x0, x, b, min_, max_, c, r):
         """
         Find the solution to the optimization problem
@@ -1356,7 +1398,8 @@ class LinfOptimizer(Optimizer):
         return np.abs(x0 - x).max()
 
 
-class L0Optimizer(Optimizer):
+@jitclass(spec=spec)
+class L0Optimizer(Optimizer):  # pragma: no cover
     def optimize_distance_s_t_boundary_and_trustregion(self, x0, x, b, min_, max_, c, r):
         """
         Find the solution to the optimization problem
@@ -1369,7 +1412,19 @@ class L0Optimizer(Optimizer):
         return self.minimize(params0, bounds, x0, x, b, min_, max_, c, r)
 
     def minimize(
-        self, q0, bounds, x0, x, b, min_, max_, c, r, ftol=1e-9, xtol=-1e-5, maxiter=1000,
+        self,
+        q0,
+        bounds,
+        x0,
+        x,
+        b,
+        min_,
+        max_,
+        c,
+        r,
+        ftol=1e-9,
+        xtol=-1e-5,
+        maxiter=1000,
     ):
         # First check whether solution can be computed without trust region
         delta, delta_norm = self.minimize_without_trustregion(x0, x, b, c, r, min_, max_)
@@ -1423,7 +1478,17 @@ class L0Optimizer(Optimizer):
         return delta, delta_norm
 
     def _nelder_mead_algorithm(
-        self, q0, bounds, args=(), ρ=1.0, χ=2.0, γ=0.5, σ=0.5, tol_f=1e-8, tol_x=1e-8, max_iter=1000,
+        self,
+        q0,
+        bounds,
+        args=(),
+        ρ=1.0,
+        χ=2.0,
+        γ=0.5,
+        σ=0.5,
+        tol_f=1e-8,
+        tol_x=1e-8,
+        max_iter=1000,
     ):
         """
         Implements the Nelder-Mead algorithm described in Lagarias et al. (1998)
@@ -1878,6 +1943,26 @@ class L0Optimizer(Optimizer):
 
 
 class BrendelBethgeAttack(EvasionAttack):
+    """
+    Base class for the Brendel & Bethge adversarial attack [#Bren19]_, a powerful gradient-based adversarial attack that
+    follows the adversarial boundary (the boundary between the space of adversarial and non-adversarial images as
+    defined by the adversarial criterion) to find the minimum distance to the clean image.
+
+    This is implementation of the Brendel & Bethge attack follows the reference implementation at
+    https://github.com/bethgelab/foolbox/blob/master/foolbox/attacks/brendel_bethge.py.
+
+    Implementation differs from the attack used in the paper in two ways:
+
+    * The initial binary search is always using the full 10 steps (for ease of implementation).
+    * The adaptation of the trust region over the course of optimisation is less
+      greedy but is more robust, reliable and simpler (decay every K steps)
+
+    References:
+        .. [#Bren19] Wieland Brendel, Jonas Rauber, Matthias Kümmerer, Ivan Ustyuzhaninov, Matthias Bethge,
+            "Accurate, reliable and fast robustness evaluation",
+            33rd Conference on Neural Information Processing Systems (2019)
+            https://arxiv.org/abs/1907.01003
+    """
 
     attack_params = EvasionAttack.attack_params + [
         "norm",
@@ -1892,54 +1977,9 @@ class BrendelBethgeAttack(EvasionAttack):
         "binary_search_steps",
         "init_size",
     ]
+
     _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)
 
-    """
-    Base class for the Brendel & Bethge adversarial attack [#Bren19]_, a powerful gradient-based adversarial attack that
-    follows the adversarial boundary (the boundary between the space of adversarial and non-adversarial images as
-    defined by the adversarial criterion) to find the minimum distance to the clean image.
-
-    This is implementation of the Brendel & Bethge attack follows the reference implementation at
-    https://github.com/bethgelab/foolbox/blob/master/foolbox/attacks/brendel_bethge.py.
-
-    Implementation differs from the attack used in the paper in two ways:
-    * The initial binary search is always using the full 10 steps (for ease of implementation).
-    * The adaptation of the trust region over the course of optimisation is less
-      greedy but is more robust, reliable and simpler (decay every K steps)
-
-    Args:
-        estimator : A trained ART classifier providing loss gradients.
-        norm : The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
-        targeted : Flag determining if attack is targeted.
-        overshoot : If 1 the attack tries to return exactly to the adversarial boundary
-            in each iteration. For higher values the attack tries to overshoot
-            over the boundary to ensure that the perturbed sample in each iteration
-            is adversarial.
-        steps : Maximum number of iterations to run. Might converge and stop
-            before that.
-        lr : Trust region radius, behaves similar to a learning rate. Smaller values
-            decrease the step size in each iteration and ensure that the attack
-            follows the boundary more faithfully.
-        lr_decay : The trust region lr is multiplied with lr_decay in regular intervals (see
-            lr_num_decay).
-        lr_num_decay : Number of learning rate decays in regular intervals of
-            length steps / lr_num_decay.
-        momentum : Averaging of the boundary estimation over multiple steps. A momentum of
-            zero would always take the current estimate while values closer to one
-            average over a larger number of iterations.
-        binary_search_steps : Number of binary search steps used to find the adversarial boundary
-            between the starting point and the clean image.
-        batch_size : Batch size for evaluating the model for predictions and gradients.
-        init_size : Maximum number of random search steps to find initial adversarial example.
-
-    References:
-        .. [#Bren19] Wieland Brendel, Jonas Rauber, Matthias Kümmerer,
-            Ivan Ustyuzhaninov, Matthias Bethge,
-            "Accurate, reliable and fast robustness evaluation",
-            33rd Conference on Neural Information Processing Systems (2019)
-            https://arxiv.org/abs/1907.01003
-    """
-
     def __init__(
         self,
         estimator: "CLASSIFIER_LOSS_GRADIENTS_TYPE",
@@ -1955,13 +1995,32 @@ class BrendelBethgeAttack(EvasionAttack):
         init_size: int = 100,
         batch_size: int = 32,
     ):
+        """
+        :param estimator: A trained ART classifier providing loss gradients.
+        :param norm: The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
+        :param targeted: Flag determining if attack is targeted.
+        :param overshoot: If 1 the attack tries to return exactly to the adversarial boundary in each iteration. For
+                          higher values the attack tries to overshoot over the boundary to ensure that the perturbed
+                          sample in each iteration is adversarial.
+        :param steps: Maximum number of iterations to run. Might converge and stop before that.
+        :param lr: Trust region radius, behaves similar to a learning rate. Smaller values decrease the step size in
+                   each iteration and ensure that the attack follows the boundary more faithfully.
+        :param lr_decay: The trust region lr is multiplied with lr_decay in regular intervals (see lr_num_decay).
+        :param lr_num_decay: Number of learning rate decays in regular intervals of length steps / lr_num_decay.
+        :param momentum: Averaging of the boundary estimation over multiple steps. A momentum of zero would always take
+                         the current estimate while values closer to one average over a larger number of iterations.
+        :param binary_search_steps: Number of binary search steps used to find the adversarial boundary between the
+                                    starting point and the clean image.
+        :param init_size: Maximum number of random search steps to find initial adversarial example.
+        :param batch_size: Batch size for evaluating the model for predictions and gradients.
+        """
         from art.estimators.classification import TensorFlowV2Classifier, PyTorchClassifier
 
         if isinstance(estimator, TensorFlowV2Classifier):
             import tensorflow as tf
 
             if is_probability(estimator.predict(x=np.ones(shape=(1, *estimator.input_shape)))):
-                raise ValueError(
+                raise ValueError(  # pragma: no cover
                     "The provided estimator seems to predict probabilities. If loss_type='difference_logits_ratio' "
                     "the estimator has to to predict logits."
                 )
@@ -1993,7 +2052,7 @@ class BrendelBethgeAttack(EvasionAttack):
                 self._loss_fn = logits_difference
                 self._loss_object = logits_difference
 
-            estimator_bb = TensorFlowV2Classifier(
+            estimator_bb: "CLASSIFIER_LOSS_GRADIENTS_TYPE" = TensorFlowV2Classifier(
                 model=estimator.model,
                 nb_classes=estimator.nb_classes,
                 input_shape=estimator.input_shape,
@@ -2012,7 +2071,7 @@ class BrendelBethgeAttack(EvasionAttack):
             if is_probability(
                 estimator.predict(x=np.ones(shape=(1, *estimator.input_shape), dtype=config.ART_NUMPY_DTYPE))
             ):
-                raise ValueError(
+                raise ValueError(  # pragma: no cover
                     "The provided estimator seems to predict probabilities. If loss_type='difference_logits_ratio' "
                     "the estimator has to to predict logits."
                 )
@@ -2020,9 +2079,9 @@ class BrendelBethgeAttack(EvasionAttack):
 
                 # def difference_logits_ratio(y_true, y_pred):
                 def logits_difference(y_pred, y_true):  # type: ignore
-                    if isinstance(y_true, np.ndarray):
+                    if isinstance(y_true, np.ndarray):  # pragma: no cover
                         y_true = torch.from_numpy(y_true)
-                    if isinstance(y_pred, np.ndarray):
+                    if isinstance(y_pred, np.ndarray):  # pragma: no cover
                         y_pred = torch.from_numpy(y_pred)
 
                     y_true = y_true.float()
@@ -2054,7 +2113,7 @@ class BrendelBethgeAttack(EvasionAttack):
 
             estimator_bb = PyTorchClassifier(
                 model=estimator.model,
-                loss=self._loss_object,
+                loss=self._loss_object,  # type: ignore
                 input_shape=estimator.input_shape,
                 nb_classes=estimator.nb_classes,
                 optimizer=None,
@@ -2063,13 +2122,19 @@ class BrendelBethgeAttack(EvasionAttack):
                 preprocessing_defences=estimator.preprocessing_defences,
                 postprocessing_defences=estimator.postprocessing_defences,
                 preprocessing=estimator.preprocessing,
-                device_type=estimator._device,
+                device_type=str(estimator._device),
             )
 
         else:
-            estimator_bb = None
+            logger.warning(
+                "The type of the provided estimator is not yet support for automated setting of logits "
+                "difference loss. Therefore, this attack is defaulting to attacking the loss provided by "
+                "the model in the provided estimator."
+            )
+            estimator_bb = estimator
 
         super().__init__(estimator=estimator_bb)
+
         self.norm = norm
         self._targeted = targeted
         self.overshoot = overshoot
@@ -2093,12 +2158,16 @@ class BrendelBethgeAttack(EvasionAttack):
         elif norm in ["inf", np.inf]:
             self._optimizer = LinfOptimizer()
 
+        # Set binary search threshold
+        if norm == 2:
+            self.theta = 0.01 / np.sqrt(np.prod(self.estimator.input_shape))
+        else:
+            self.theta = 0.01 / np.prod(self.estimator.input_shape)
+
     def generate(
         self,
         x: np.ndarray,
         y: Optional[np.ndarray] = None,
-        starting_points: Optional[np.ndarray] = None,
-        early_stop: Optional[float] = None,
         **kwargs,
     ) -> np.ndarray:
         """
@@ -2106,9 +2175,14 @@ class BrendelBethgeAttack(EvasionAttack):
 
         :param x: The original clean inputs.
         :param y: The labels for inputs `x`.
-        :param starting_points: Adversarial inputs to use as a starting points, in particular for targeted attacks.
-        :param early_stop: Early-stopping criteria.
+
+        :Keyword Arguments:
+            * *starting_points* (``np.ndarray``)
+                Optional. Adversarial inputs to use as a starting points, in particular for targeted attacks.
         """
+        starting_points = kwargs.get("starting_points")
+        # early_stop = kwargs.get("early_stop")
+
         originals = x.copy()
 
         y = check_and_transform_label_format(y, self.estimator.nb_classes)
@@ -2120,8 +2194,11 @@ class BrendelBethgeAttack(EvasionAttack):
 
             # Use model predictions as correct outputs
             logger.info("Using model predictions as correct labels for FGM.")
-            y = get_labels_np_array(
-                self.estimator.predict(x, batch_size=self.batch_size)  # type: ignore
+            y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))  # type: ignore
+
+        if self.estimator.nb_classes == 2 and y.shape[1] == 1:
+            raise ValueError(
+                "This attack has not yet been tested for binary classification with a single output classifier."
             )
 
         # Prediction from the initial adversarial examples if not None
@@ -2197,7 +2274,7 @@ class BrendelBethgeAttack(EvasionAttack):
         x = starting_points.astype(config.ART_NUMPY_DTYPE)
         lrs = self.lr * np.ones(N)
         lr_reduction_interval = max(1, int(self.steps / self.lr_num_decay))
-        converged = np.zeros(N, dtype=np.bool)
+        converged = np.zeros(N, dtype=bool)
         rate_normalization = np.prod(x.shape) * (max_ - min_)
         original_shape = x.shape
         _best_advs = best_advs.copy()
@@ -2269,7 +2346,9 @@ class BrendelBethgeAttack(EvasionAttack):
 
             # we aim to slight overshoot over the boundary to stay within the adversarial region
             corr_logits_diffs = np.where(
-                -logits_diffs < 0, -self.overshoot * logits_diffs, -(2 - self.overshoot) * logits_diffs,
+                -logits_diffs < 0,
+                -self.overshoot * logits_diffs,
+                -(2 - self.overshoot) * logits_diffs,
             )
 
             # employ solver to find optimal step within trust region for each sample
@@ -2286,18 +2365,15 @@ class BrendelBethgeAttack(EvasionAttack):
                     _c = corr_logits_diffs[k]
                     r = region[sample]
 
-                    delta = self._optimizer.solve(  # type: ignore
-                        _x0, _x, _b, bounds[0], bounds[1], _c, r
-                    )
+                    delta = self._optimizer.solve(_x0, _x, _b, bounds[0], bounds[1], _c, r)  # type: ignore
                     deltas.append(delta)
 
                     k += 1  # idx of masked sample
 
-            deltas = np.stack(deltas)
-            deltas = deltas.astype(np.float32)
+            deltas_array = np.stack(deltas).astype(np.float32)
 
             # add step to current perturbation
-            x = (x + deltas).reshape(original_shape)
+            x = (x + deltas_array).reshape(original_shape)
 
         return best_advs.astype(config.ART_NUMPY_DTYPE)
 
@@ -2307,7 +2383,11 @@ class BrendelBethgeAttack(EvasionAttack):
         return norm
 
     def mid_points(
-        self, x0: np.ndarray, x1: np.ndarray, epsilons: np.ndarray, bounds: Tuple[float, float],
+        self,
+        x0: np.ndarray,
+        x1: np.ndarray,
+        epsilons: np.ndarray,
+        bounds: Tuple[float, float],
     ) -> np.ndarray:
         """
         returns a point between x0 and x1 where epsilon = 0 returns x0 and epsilon = 1 returns x1
@@ -2344,7 +2424,14 @@ class BrendelBethgeAttack(EvasionAttack):
         return new_x.astype(config.ART_NUMPY_DTYPE)
 
     def _init_sample(
-        self, x: np.ndarray, y: int, y_p: int, init_pred: int, adv_init: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        x: np.ndarray,
+        y: int,
+        y_p: int,
+        init_pred: int,
+        adv_init: np.ndarray,
+        clip_min: float,
+        clip_max: float,
     ) -> Optional[Union[np.ndarray, Tuple[np.ndarray, int]]]:
         """
         Find initial adversarial example for the attack.
@@ -2374,7 +2461,8 @@ class BrendelBethgeAttack(EvasionAttack):
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class == y:
@@ -2404,7 +2492,8 @@ class BrendelBethgeAttack(EvasionAttack):
             for _ in range(self.init_size):
                 random_img = nprd.uniform(clip_min, clip_max, size=x.shape).astype(x.dtype)
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class != y_p:
@@ -2422,7 +2511,7 @@ class BrendelBethgeAttack(EvasionAttack):
 
                     logger.info("Found initial adversarial image for untargeted attack.")
                     break
-            else:
+            else:  # pragma: no cover
                 logger.warning("Failed to draw a random image that is adversarial, attack failed.")
 
         return initial_sample
@@ -2470,18 +2559,27 @@ class BrendelBethgeAttack(EvasionAttack):
             # Interpolation point
             alpha = (upper_bound + lower_bound) / 2.0
             interpolated_sample = self._interpolate(
-                current_sample=current_sample, original_sample=original_sample, alpha=alpha, norm=norm,
+                current_sample=current_sample,
+                original_sample=original_sample,
+                alpha=alpha,
+                norm=norm,
             )
 
             # Update upper_bound and lower_bound
             satisfied = self._adversarial_satisfactory(
-                samples=interpolated_sample[None], target=target, clip_min=clip_min, clip_max=clip_max,
+                samples=interpolated_sample[None],
+                target=target,
+                clip_min=clip_min,
+                clip_max=clip_max,
             )[0]
             lower_bound = np.where(satisfied == 0, alpha, lower_bound)
             upper_bound = np.where(satisfied == 1, alpha, upper_bound)
 
         result = self._interpolate(
-            current_sample=current_sample, original_sample=original_sample, alpha=upper_bound, norm=norm,
+            current_sample=current_sample,
+            original_sample=original_sample,
+            alpha=upper_bound,
+            norm=norm,
         )
 
         return result
@@ -2530,7 +2628,7 @@ class BrendelBethgeAttack(EvasionAttack):
 
     def _check_params(self) -> None:
 
-        if self.norm not in [1, 2, np.inf, "inf"]:
+        if self.norm not in [0, 1, 2, np.inf, "inf"]:
             raise ValueError('The argument norm has to be either 1, 2, np.inf, or "inf".')
 
         if not isinstance(self.targeted, bool):
