@@ -26,12 +26,11 @@ import random
 from typing import Dict, List, Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.attacks.attack import EvasionAttack
 from art.estimators.estimator import BaseEstimator, LossGradientsMixin
 from art.estimators.object_detection.object_detector import ObjectDetectorMixin
-from art.utils import Deprecated, deprecated_keyword_arg
 from art import config
 
 if TYPE_CHECKING:
@@ -82,6 +81,9 @@ class DPatch(EvasionAttack):
         self.learning_rate = learning_rate
         self.max_iter = max_iter
         self.batch_size = batch_size
+        self.verbose = verbose
+        self._check_params()
+
         if self.estimator.clip_values is None:
             self._patch = np.zeros(shape=patch_shape, dtype=config.ART_NUMPY_DTYPE)
         else:
@@ -91,12 +93,10 @@ class DPatch(EvasionAttack):
                 * (self.estimator.clip_values[1] - self.estimator.clip_values[0])
                 + self.estimator.clip_values[0]
             ).astype(config.ART_NUMPY_DTYPE)
-        self.verbose = verbose
-        self._check_params()
 
-        self.target_label = []
+        self.target_label: Optional[Union[int, np.ndarray, List[int]]] = list()
 
-    def generate(
+    def generate(  # pylint: disable=W0221
         self,
         x: np.ndarray,
         y: Optional[np.ndarray] = None,
@@ -118,9 +118,8 @@ class DPatch(EvasionAttack):
         mask = kwargs.get("mask")
         if mask is not None:
             mask = mask.copy()
-        if (
-            mask is not None
-            and (mask.dtype != np.bool)
+        if mask is not None and (  # pragma: no cover
+            mask.dtype != bool
             or not (mask.shape[0] == 1 or mask.shape[0] == x.shape[0])
             or not (
                 (mask.shape[1] == x.shape[1] and mask.shape[2] == x.shape[2])
@@ -137,28 +136,40 @@ class DPatch(EvasionAttack):
             raise ValueError("The color channel index of the images and the patch have to be identical.")
         if y is not None:
             raise ValueError("The DPatch attack does not use target labels.")
-        if x.ndim != 4:
+        if x.ndim != 4:  # pragma: no cover
             raise ValueError("The adversarial patch can only be applied to images.")
         if target_label is not None:
             if isinstance(target_label, int):
                 self.target_label = [target_label] * x.shape[0]
             elif isinstance(target_label, np.ndarray):
-                if not (target_label.shape == (x.shape[0], 1) or target_label.shape == (x.shape[0],)):
+                if not (  # pragma: no cover
+                    target_label.shape == (x.shape[0], 1) or target_label.shape == (x.shape[0],)
+                ):
                     raise ValueError("The target_label has to be a 1-dimensional array.")
                 self.target_label = target_label.tolist()
             else:
-                if not len(target_label) == x.shape[0] or not isinstance(target_label, list):
+                if not len(target_label) == x.shape[0] or not isinstance(target_label, list):  # pragma: no cover
                     raise ValueError("The target_label as list of integers needs to of length number of images in `x`.")
                 self.target_label = target_label
 
         patched_images, transforms = self._augment_images_with_patch(
-            x, self._patch, random_location=True, channels_first=self.estimator.channels_first, mask=mask
+            x,
+            self._patch,
+            random_location=True,
+            channels_first=self.estimator.channels_first,
+            mask=mask,
+            transforms=None,
         )
         patch_target: List[Dict[str, np.ndarray]] = list()
 
         if self.target_label:
 
             for i_image in range(patched_images.shape[0]):
+                if isinstance(self.target_label, int):
+                    t_l = self.target_label
+                else:
+                    t_l = self.target_label[i_image]
+
                 i_x_1 = transforms[i_image]["i_x_1"]
                 i_x_2 = transforms[i_image]["i_x_2"]
                 i_y_1 = transforms[i_image]["i_y_1"]
@@ -166,14 +177,22 @@ class DPatch(EvasionAttack):
 
                 target_dict = dict()
                 target_dict["boxes"] = np.asarray([[i_x_1, i_y_1, i_x_2, i_y_2]])
-                target_dict["labels"] = np.asarray([self.target_label[i_image],])
-                target_dict["scores"] = np.asarray([1.0,])
+                target_dict["labels"] = np.asarray(
+                    [
+                        t_l,
+                    ]
+                )
+                target_dict["scores"] = np.asarray(
+                    [
+                        1.0,
+                    ]
+                )
 
                 patch_target.append(target_dict)
 
         else:
 
-            predictions = self.estimator.predict(x=patched_images)
+            predictions = self.estimator.predict(x=patched_images, standardise_output=True)
 
             for i_image in range(patched_images.shape[0]):
                 target_dict = dict()
@@ -195,7 +214,9 @@ class DPatch(EvasionAttack):
                 i_batch_end = min((i_batch + 1) * self.batch_size, patched_images.shape[0])
 
                 gradients = self.estimator.loss_gradient(
-                    x=patched_images[i_batch_start:i_batch_end], y=patch_target[i_batch_start:i_batch_end],
+                    x=patched_images[i_batch_start:i_batch_end],
+                    y=patch_target[i_batch_start:i_batch_end],
+                    standardise_output=True,
                 )
 
                 for i_image in range(gradients.shape[0]):
@@ -219,20 +240,30 @@ class DPatch(EvasionAttack):
 
             if self.estimator.clip_values is not None:
                 self._patch = np.clip(
-                    self._patch, a_min=self.estimator.clip_values[0], a_max=self.estimator.clip_values[1],
+                    self._patch,
+                    a_min=self.estimator.clip_values[0],
+                    a_max=self.estimator.clip_values[1],
                 )
 
+            patched_images, _ = self._augment_images_with_patch(
+                x,
+                self._patch,
+                random_location=False,
+                channels_first=self.estimator.channels_first,
+                mask=None,
+                transforms=transforms,
+            )
+
         return self._patch
 
     @staticmethod
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
     def _augment_images_with_patch(
         x: np.ndarray,
         patch: np.ndarray,
         random_location: bool,
         channels_first: bool,
-        channel_index=Deprecated,
         mask: Optional[np.ndarray] = None,
+        transforms: List[Dict[str, int]] = None,
     ) -> Tuple[np.ndarray, List[Dict[str, int]]]:
         """
         Augment images with patch.
@@ -242,22 +273,19 @@ class DPatch(EvasionAttack):
         :param random_location: If True apply patch at randomly shifted locations, otherwise place patch at origin
                                 (top-left corner).
         :param channels_first: Set channels first or last.
-        :param channel_index: Index of the color channel.
-        :type channel_index: `int`
         :param mask: An boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
                      (N, H, W) without their channel dimensions. Any features for which the mask is True can be the
                      center location of the patch during sampling.
+        :param transforms: Patch transforms, requires `random_location=False`, and `mask=None`.
         :type mask: `np.ndarray`
         """
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
+        if transforms is not None:
+            if random_location or mask is not None:
+                raise ValueError(
+                    "Definition of patch locations in `locations` requires `random_location=False`, and `mask=None`."
+                )
 
-        transformations = list()
+        random_transformations = list()
         x_copy = x.copy()
         patch_copy = patch.copy()
 
@@ -267,48 +295,56 @@ class DPatch(EvasionAttack):
 
         for i_image in range(x.shape[0]):
 
-            if random_location:
-                if mask is None:
-                    i_x_1 = random.randint(0, x_copy.shape[1] - 1 - patch_copy.shape[0])
-                    i_y_1 = random.randint(0, x_copy.shape[2] - 1 - patch_copy.shape[1])
-                else:
+            if transforms is None:
 
-                    if mask.shape[0] == 1:
-                        mask_2d = mask[0, :, :]
+                if random_location:
+                    if mask is None:
+                        i_x_1 = random.randint(0, x_copy.shape[1] - 1 - patch_copy.shape[0])
+                        i_y_1 = random.randint(0, x_copy.shape[2] - 1 - patch_copy.shape[1])
                     else:
-                        mask_2d = mask[i_image, :, :]
 
-                    edge_x_0 = patch_copy.shape[0] // 2
-                    edge_x_1 = patch_copy.shape[0] - edge_x_0
-                    edge_y_0 = patch_copy.shape[1] // 2
-                    edge_y_1 = patch_copy.shape[1] - edge_y_0
+                        if mask.shape[0] == 1:
+                            mask_2d = mask[0, :, :]
+                        else:
+                            mask_2d = mask[i_image, :, :]
 
-                    mask_2d[0:edge_x_0, :] = False
-                    mask_2d[-edge_x_1:, :] = False
-                    mask_2d[:, 0:edge_y_0] = False
-                    mask_2d[:, -edge_y_1:] = False
+                        edge_x_0 = patch_copy.shape[0] // 2
+                        edge_x_1 = patch_copy.shape[0] - edge_x_0
+                        edge_y_0 = patch_copy.shape[1] // 2
+                        edge_y_1 = patch_copy.shape[1] - edge_y_0
 
-                    num_pos = np.argwhere(mask_2d).shape[0]
-                    pos_id = np.random.choice(num_pos, size=1)
-                    pos = np.argwhere(mask_2d > 0)[pos_id[0]]
-                    i_x_1 = pos[0] - edge_x_0
-                    i_y_1 = pos[1] - edge_y_0
+                        mask_2d[0:edge_x_0, :] = False
+                        mask_2d[-edge_x_1:, :] = False
+                        mask_2d[:, 0:edge_y_0] = False
+                        mask_2d[:, -edge_y_1:] = False
+
+                        num_pos = np.argwhere(mask_2d).shape[0]
+                        pos_id = np.random.choice(num_pos, size=1)
+                        pos = np.argwhere(mask_2d > 0)[pos_id[0]]
+                        i_x_1 = pos[0] - edge_x_0
+                        i_y_1 = pos[1] - edge_y_0
+
+                else:
+                    i_x_1 = 0
+                    i_y_1 = 0
+
+                i_x_2 = i_x_1 + patch_copy.shape[0]
+                i_y_2 = i_y_1 + patch_copy.shape[1]
+
+                random_transformations.append({"i_x_1": i_x_1, "i_y_1": i_y_1, "i_x_2": i_x_2, "i_y_2": i_y_2})
 
             else:
-                i_x_1 = 0
-                i_y_1 = 0
-
-            i_x_2 = i_x_1 + patch_copy.shape[0]
-            i_y_2 = i_y_1 + patch_copy.shape[1]
-
-            transformations.append({"i_x_1": i_x_1, "i_y_1": i_y_1, "i_x_2": i_x_2, "i_y_2": i_y_2})
+                i_x_1 = transforms[i_image]["i_x_1"]
+                i_x_2 = transforms[i_image]["i_x_2"]
+                i_y_1 = transforms[i_image]["i_y_1"]
+                i_y_2 = transforms[i_image]["i_y_2"]
 
             x_copy[i_image, i_x_1:i_x_2, i_y_1:i_y_2, :] = patch_copy
 
         if channels_first:
             x_copy = np.transpose(x_copy, (0, 3, 1, 2))
 
-        return x_copy, transformations
+        return x_copy, random_transformations
 
     def apply_patch(
         self,
