@@ -27,7 +27,7 @@ from typing import Optional, Tuple, TYPE_CHECKING
 
 import numpy as np
 import six
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
@@ -130,7 +130,11 @@ class ElasticNet(EvasionAttack):
         return np.argmax(predictions, axis=1), l1dist, l2dist, endist
 
     def _gradient_of_loss(
-        self, target: np.ndarray, x: np.ndarray, x_adv: np.ndarray, c_weight: np.ndarray,
+        self,
+        target: np.ndarray,
+        x: np.ndarray,
+        x_adv: np.ndarray,
+        c_weight: np.ndarray,
     ) -> np.ndarray:
         """
         Compute the gradient of the loss function.
@@ -147,12 +151,14 @@ class ElasticNet(EvasionAttack):
         if self.targeted:
             i_sub = np.argmax(target, axis=1)
             i_add = np.argmax(
-                predictions * (1 - target) + (np.min(predictions, axis=1) - 1)[:, np.newaxis] * target, axis=1,
+                predictions * (1 - target) + (np.min(predictions, axis=1) - 1)[:, np.newaxis] * target,
+                axis=1,
             )
         else:
             i_add = np.argmax(target, axis=1)
             i_sub = np.argmax(
-                predictions * (1 - target) + (np.min(predictions, axis=1) - 1)[:, np.newaxis] * target, axis=1,
+                predictions * (1 - target) + (np.min(predictions, axis=1) - 1)[:, np.newaxis] * target,
+                axis=1,
             )
 
         loss_gradient = self.estimator.class_gradient(x_adv, label=i_add)
@@ -166,6 +172,12 @@ class ElasticNet(EvasionAttack):
         loss_gradient *= c_mult
         loss_gradient += 2 * (x_adv - x)
 
+        # Set gradients where loss is constant to zero
+        cond = (
+            predictions[np.arange(x.shape[0]), i_add] - predictions[np.arange(x.shape[0]), i_sub] + self.confidence
+        ) < 0
+        loss_gradient[cond] = 0.0
+
         return loss_gradient
 
     def _decay_learning_rate(self, global_step: int, end_learning_rate: float, decay_steps: int) -> float:
@@ -196,13 +208,18 @@ class ElasticNet(EvasionAttack):
         x_adv = x.astype(ART_NUMPY_DTYPE)
 
         # Assert that, if attack is targeted, y is provided:
-        if self.targeted and y is None:
+        if self.targeted and y is None:  # pragma: no cover
             raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
         # No labels provided, use model prediction as correct class
         if y is None:
             y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))
 
+        if self.estimator.nb_classes == 2 and y.shape[1] == 1:  # pragma: no cover
+            raise ValueError(
+                "This attack has not yet been tested for binary classification with a single output classifier."
+            )
+
         # Compute adversarial examples with implicit batching
         nb_batches = int(np.ceil(x_adv.shape[0] / float(self.batch_size)))
         for batch_id in trange(nb_batches, desc="EAD", disable=not self.verbose):
@@ -243,7 +260,10 @@ class ElasticNet(EvasionAttack):
         # Start with a binary search
         for bss in range(self.binary_search_steps):
             logger.debug(
-                "Binary search step %i out of %i (c_mean==%f)", bss, self.binary_search_steps, np.mean(c_current),
+                "Binary search step %i out of %i (c_mean==%f)",
+                bss,
+                self.binary_search_steps,
+                np.mean(c_current),
             )
 
             # Run with 1 specific binary search step
@@ -347,7 +367,7 @@ class ElasticNet(EvasionAttack):
                 zip_set = zip(l1dist, logits)
             elif self.decision_rule == "L2":
                 zip_set = zip(l2dist, logits)
-            else:
+            else:  # pragma: no cover
                 raise ValueError("The decision rule only supports `EN`, `L1`, `L2`.")
 
             for j, (distance, label) in enumerate(zip_set):
