@@ -39,6 +39,7 @@ from art.utils import (
     projection,
     check_and_transform_label_format,
 )
+from art.summary_writer import SummaryWriter
 
 if TYPE_CHECKING:
     from art.utils import CLASSIFIER_LOSS_GRADIENTS_TYPE
@@ -63,6 +64,7 @@ class FastGradientMethod(EvasionAttack):
         "num_random_init",
         "batch_size",
         "minimal",
+        "summary_writer",
     ]
     _estimator_requirements = (BaseEstimator, LossGradientsMixin)
 
@@ -76,6 +78,7 @@ class FastGradientMethod(EvasionAttack):
         num_random_init: int = 0,
         batch_size: int = 32,
         minimal: bool = False,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
     ) -> None:
         """
         Create a :class:`.FastGradientMethod` instance.
@@ -90,8 +93,15 @@ class FastGradientMethod(EvasionAttack):
         :param batch_size: Size of the batch on which adversarial samples are generated.
         :param minimal: Indicates if computing the minimal perturbation (True). If True, also define `eps_step` for
                         the step size and eps for the maximum perturbation.
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         """
-        super().__init__(estimator=estimator)
+        super().__init__(estimator=estimator, summary_writer=summary_writer)
         self.norm = norm
         self.eps = eps
         self.eps_step = eps_step
@@ -102,6 +112,9 @@ class FastGradientMethod(EvasionAttack):
         self._project = True
         FastGradientMethod._check_params(self)
 
+        self._batch_id = 0
+        self._i_max_iter = 0
+
     def _check_compatibility_input_and_eps(self, x: np.ndarray):
         """
         Check the compatibility of the input with `eps` and `eps_step` which are of the same shape.
@@ -110,7 +123,7 @@ class FastGradientMethod(EvasionAttack):
         """
         if isinstance(self.eps, np.ndarray):
             # Ensure the eps array is broadcastable
-            if self.eps.ndim > x.ndim:
+            if self.eps.ndim > x.ndim:  # pragma: no cover
                 raise ValueError("The `eps` shape must be broadcastable to input shape.")
 
     def _minimal_perturbation(self, x: np.ndarray, y: np.ndarray, mask: np.ndarray) -> np.ndarray:
@@ -146,7 +159,7 @@ class FastGradientMethod(EvasionAttack):
             # Get current predictions
             active_indices = np.arange(len(batch))
 
-            if isinstance(self.eps, np.ndarray):
+            if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
                 if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
                     current_eps = self.eps_step[batch_index_1:batch_index_2]
                     partial_stop_condition = (current_eps <= self.eps[batch_index_1:batch_index_2]).all()
@@ -174,7 +187,7 @@ class FastGradientMethod(EvasionAttack):
                     active_indices = np.where(np.argmax(batch_labels, axis=1) == np.argmax(adv_preds, axis=1))[0]
 
                 # Update current eps and check the stop condition
-                if isinstance(self.eps, np.ndarray):
+                if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
                     if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
                         current_eps = current_eps + self.eps_step[batch_index_1:batch_index_2]
                         partial_stop_condition = (current_eps <= self.eps[batch_index_1:batch_index_2]).all()
@@ -215,13 +228,15 @@ class FastGradientMethod(EvasionAttack):
 
             if y is None:
                 # Throw error if attack is targeted, but no targets are provided
-                if self.targeted:
+                if self.targeted:  # pragma: no cover
                     raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
                 # Use model predictions as correct outputs
                 logger.info("Using model predictions as correct labels for FGM.")
                 y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))  # type: ignore
-            y = y / np.sum(y, axis=1, keepdims=True)
+
+            if self.estimator.nb_classes > 2:
+                y = y / np.sum(y, axis=1, keepdims=True)
 
             # Return adversarial examples computed with minimal perturbation if option is active
             rate_best: Optional[float]
@@ -229,18 +244,37 @@ class FastGradientMethod(EvasionAttack):
                 logger.info("Performing minimal perturbation FGM.")
                 adv_x_best = self._minimal_perturbation(x, y, mask)
                 rate_best = 100 * compute_success(
-                    self.estimator, x, y, adv_x_best, self.targeted, batch_size=self.batch_size,  # type: ignore
+                    self.estimator,  # type: ignore
+                    x,
+                    y,
+                    adv_x_best,
+                    self.targeted,
+                    batch_size=self.batch_size,  # type: ignore
                 )
             else:
                 adv_x_best = None
                 rate_best = None
 
                 for _ in range(max(1, self.num_random_init)):
-                    adv_x = self._compute(x, x, y, mask, self.eps, self.eps, self._project, self.num_random_init > 0,)
+                    adv_x = self._compute(
+                        x,
+                        x,
+                        y,
+                        mask,
+                        self.eps,
+                        self.eps,
+                        self._project,
+                        self.num_random_init > 0,
+                    )
 
                     if self.num_random_init > 1:
                         rate = 100 * compute_success(
-                            self.estimator, x, y, adv_x, self.targeted, batch_size=self.batch_size,  # type: ignore
+                            self.estimator,  # type: ignore
+                            x,
+                            y,
+                            adv_x,
+                            self.targeted,
+                            batch_size=self.batch_size,  # type: ignore
                         )
                         if rate_best is None or rate > rate_best or adv_x_best is None:
                             rate_best = rate
@@ -264,42 +298,56 @@ class FastGradientMethod(EvasionAttack):
             )
 
         else:
-            if self.minimal:
+            if self.minimal:  # pragma: no cover
                 raise ValueError("Minimal perturbation is only supported for classification.")
 
             if y is None:
                 # Throw error if attack is targeted, but no targets are provided
-                if self.targeted:
+                if self.targeted:  # pragma: no cover
                     raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
                 # Use model predictions as correct outputs
                 logger.info("Using model predictions as correct labels for FGM.")
                 y = self.estimator.predict(x, batch_size=self.batch_size)
 
-            adv_x_best = self._compute(x, x, y, None, self.eps, self.eps, self._project, self.num_random_init > 0,)
+            adv_x_best = self._compute(
+                x,
+                x,
+                y,
+                None,
+                self.eps,
+                self.eps,
+                self._project,
+                self.num_random_init > 0,
+            )
+
+        if self.summary_writer is not None:
+            self.summary_writer.reset()
 
         return adv_x_best
 
     def _check_params(self) -> None:
-        # Check if order of the norm is acceptable given current implementation
+
         if self.norm not in [1, 2, np.inf, "inf"]:
             raise ValueError('Norm order must be either 1, 2, `np.inf` or "inf".')
 
-        if (not (isinstance(self.eps, (int, float, np.ndarray)) and isinstance(self.eps_step, (int, float)))) and (
-            hasattr(self, "minimal")
-            and self.minimal
-            and not (isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray))
+        if not (
+            isinstance(self.eps, (int, float))
+            and isinstance(self.eps_step, (int, float))
+            or isinstance(self.eps, np.ndarray)
+            and isinstance(self.eps_step, np.ndarray)
         ):
             raise TypeError(
-                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same type."
+                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same type of `int`"
+                ", `float`, or `np.ndarray`."
             )
 
         if isinstance(self.eps, (int, float)):
-            if self.eps <= 0:
-                raise ValueError("The perturbation size `eps` has to be positive.")
+            if self.eps < 0:
+                raise ValueError("The perturbation size `eps` has to be nonnegative.")
         else:
-            if (self.eps <= 0).any():
-                raise ValueError("The perturbation size `eps` has to be positive.")
+            if (self.eps < 0).any():
+                raise ValueError("The perturbation size `eps` has to be nonnegative.")
 
         if isinstance(self.eps_step, (int, float)):
             if self.eps_step <= 0:
@@ -308,19 +356,16 @@ class FastGradientMethod(EvasionAttack):
             if (self.eps_step <= 0).any():
                 raise ValueError("The perturbation step-size `eps_step` has to be positive.")
 
-        if (
-            isinstance(self.eps, np.ndarray)
-            and isinstance(self.eps_step, np.ndarray)
-            and self.eps.shape != self.eps_step.shape
-        ):
-            raise ValueError(
-                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same shape."
-            )
+        if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
+            if self.eps.shape != self.eps_step.shape:
+                raise ValueError(
+                    "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same shape."
+                )
 
         if not isinstance(self.targeted, bool):
             raise ValueError("The flag `targeted` has to be of type bool.")
 
-        if not isinstance(self.num_random_init, (int, np.int)):
+        if not isinstance(self.num_random_init, int):
             raise TypeError("The number of random initialisations has to be of type integer")
 
         if self.num_random_init < 0:
@@ -341,12 +386,40 @@ class FastGradientMethod(EvasionAttack):
         # Get gradient wrt loss; invert it if attack is targeted
         grad = self.estimator.loss_gradient(batch, batch_labels) * (1 - 2 * int(self.targeted))
 
+        # Write summary
+        if self.summary_writer is not None:  # pragma: no cover
+            self.summary_writer.update(
+                batch_id=self._batch_id,
+                global_step=self._i_max_iter,
+                grad=grad,
+                patch=None,
+                estimator=self.estimator,
+                x=batch,
+                y=batch_labels,
+                targeted=self.targeted,
+            )
+
+        # Check for NaN before normalisation an replace with 0
+        if grad.dtype != object and np.isnan(grad).any():  # pragma: no cover
+            logger.warning("Elements of the loss gradient are NaN and have been replaced with 0.0.")
+            grad = np.where(np.isnan(grad), 0.0, grad)
+        else:
+            for i, _ in enumerate(grad):
+                grad_i_array = grad[i].astype(np.float32)
+                if np.isnan(grad_i_array).any():
+                    grad[i] = np.where(np.isnan(grad_i_array), 0.0, grad_i_array).astype(object)
+
         # Apply mask
         if mask is not None:
             grad = np.where(mask == 0.0, 0.0, grad)
 
         # Apply norm bound
         def _apply_norm(grad, object_type=False):
+            if (grad.dtype != object and np.isinf(grad).any()) or np.isnan(  # pragma: no cover
+                grad.astype(np.float32)
+            ).any():
+                logger.info("The loss gradient array contains at least one positive or negative infinity.")
+
             if self.norm in [np.inf, "inf"]:
                 grad = np.sign(grad)
             elif self.norm == 1:
@@ -363,7 +436,7 @@ class FastGradientMethod(EvasionAttack):
                 grad = grad / (np.sqrt(np.sum(np.square(grad), axis=ind, keepdims=True)) + tol)
             return grad
 
-        if batch.dtype == np.object:
+        if batch.dtype == object:
             for i_sample in range(batch.shape[0]):
                 grad[i_sample] = _apply_norm(grad[i_sample], object_type=True)
                 assert batch[i_sample].shape == grad[i_sample].shape
@@ -377,11 +450,26 @@ class FastGradientMethod(EvasionAttack):
     def _apply_perturbation(
         self, batch: np.ndarray, perturbation: np.ndarray, eps_step: Union[int, float, np.ndarray]
     ) -> np.ndarray:
-        batch = batch + eps_step * perturbation
 
+        perturbation_step = eps_step * perturbation
+        if perturbation_step.dtype != object:
+            perturbation_step[np.isnan(perturbation_step)] = 0
+        else:
+            for i, _ in enumerate(perturbation_step):
+                perturbation_step_i_array = perturbation_step[i].astype(np.float32)
+                if np.isnan(perturbation_step_i_array).any():
+                    perturbation_step[i] = np.where(
+                        np.isnan(perturbation_step_i_array), 0.0, perturbation_step_i_array
+                    ).astype(object)
+
+        batch = batch + perturbation_step
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
-            batch = np.clip(batch, clip_min, clip_max)
+            if batch.dtype == np.object:
+                for i_obj in range(batch.shape[0]):
+                    batch[i_obj] = np.clip(batch[i_obj], clip_min, clip_max)
+            else:
+                batch = np.clip(batch, clip_min, clip_max)
 
         return batch
 
@@ -395,6 +483,7 @@ class FastGradientMethod(EvasionAttack):
         eps_step: Union[int, float, np.ndarray],
         project: bool,
         random_init: bool,
+        batch_id_ext: Optional[int] = None,
     ) -> np.ndarray:
         if random_init:
             n = x.shape[0]
@@ -408,13 +497,17 @@ class FastGradientMethod(EvasionAttack):
                 clip_min, clip_max = self.estimator.clip_values
                 x_adv = np.clip(x_adv, clip_min, clip_max)
         else:
-            if x.dtype == np.object:
+            if x.dtype == object:
                 x_adv = x.copy()
             else:
                 x_adv = x.astype(ART_NUMPY_DTYPE)
 
         # Compute perturbation with implicit batching
         for batch_id in range(int(np.ceil(x.shape[0] / float(self.batch_size)))):
+            if batch_id_ext is None:
+                self._batch_id = batch_id
+            else:
+                self._batch_id = batch_id_ext
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
             batch_index_2 = min(batch_index_2, x.shape[0])
             batch = x_adv[batch_index_1:batch_index_2]
@@ -431,7 +524,7 @@ class FastGradientMethod(EvasionAttack):
             perturbation = self._compute_perturbation(batch, batch_labels, mask_batch)
 
             # Compute batch_eps and batch_eps_step
-            if isinstance(eps, np.ndarray):
+            if isinstance(eps, np.ndarray) and isinstance(eps_step, np.ndarray):
                 if len(eps.shape) == len(x.shape) and eps.shape[0] == x.shape[0]:
                     batch_eps = eps[batch_index_1:batch_index_2]
                     batch_eps_step = eps_step[batch_index_1:batch_index_2]
@@ -448,7 +541,7 @@ class FastGradientMethod(EvasionAttack):
             x_adv[batch_index_1:batch_index_2] = self._apply_perturbation(batch, perturbation, batch_eps_step)
 
             if project:
-                if x_adv.dtype == np.object:
+                if x_adv.dtype == object:
                     for i_sample in range(batch_index_1, batch_index_2):
                         if isinstance(batch_eps, np.ndarray) and batch_eps.shape[0] == x_adv.shape[0]:
                             perturbation = projection(
@@ -483,16 +576,16 @@ class FastGradientMethod(EvasionAttack):
         mask = kwargs.get("mask")
 
         if mask is not None:
-            if mask.ndim > x.ndim:
+            if mask.ndim > x.ndim:  # pragma: no cover
                 raise ValueError("Mask shape must be broadcastable to input shape.")
 
-            if not (np.issubdtype(mask.dtype, np.floating) or mask.dtype == np.bool):
+            if not (np.issubdtype(mask.dtype, np.floating) or mask.dtype == bool):  # pragma: no cover
                 raise ValueError(
-                    "The `mask` has to be either of type np.float32, np.float64 or np.bool. The provided"
+                    "The `mask` has to be either of type np.float32, np.float64 or bool. The provided"
                     "`mask` is of type {}.".format(mask.dtype)
                 )
 
-            if np.issubdtype(mask.dtype, np.floating) and np.amin(mask) < 0.0:
+            if np.issubdtype(mask.dtype, np.floating) and np.amin(mask) < 0.0:  # pragma: no cover
                 raise ValueError(
                     "The `mask` of type np.float32 or np.float64 requires all elements to be either zero"
                     "or positive values."
