@@ -27,7 +27,7 @@ import logging
 from typing import Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
@@ -65,6 +65,7 @@ class HopSkipJump(EvasionAttack):
     def __init__(
         self,
         classifier: "CLASSIFIER_TYPE",
+        batch_size: int = 64,
         targeted: bool = False,
         norm: Union[int, float, str] = 2,
         max_iter: int = 50,
@@ -77,6 +78,7 @@ class HopSkipJump(EvasionAttack):
         Create a HopSkipJump attack instance.
 
         :param classifier: A trained classifier.
+        :param batch_size: The size of the batch used by the estimator during inference.
         :param targeted: Should the attack target one specific class.
         :param norm: Order of the norm. Possible values: "inf", np.inf or 2.
         :param max_iter: Maximum number of iterations.
@@ -93,7 +95,7 @@ class HopSkipJump(EvasionAttack):
         self.init_eval = init_eval
         self.init_size = init_size
         self.curr_iter = 0
-        self.batch_size = 1
+        self.batch_size = batch_size
         self.verbose = verbose
         self._check_params()
         self.curr_iter = 0
@@ -125,6 +127,11 @@ class HopSkipJump(EvasionAttack):
 
         y = check_and_transform_label_format(y, self.estimator.nb_classes)
 
+        if y is not None and self.estimator.nb_classes == 2 and y.shape[1] == 1:  # pragma: no cover
+            raise ValueError(
+                "This attack has not yet been tested for binary classification with a single output classifier."
+            )
+
         # Check whether users need a stateful attack
         resume = kwargs.get("resume")
 
@@ -168,7 +175,7 @@ class HopSkipJump(EvasionAttack):
             x_adv_init = [None] * len(x)
 
         # Assert that, if attack is targeted, y is provided
-        if self.targeted and y is None:
+        if self.targeted and y is None:  # pragma: no cover
             raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
         # Some initial setups
@@ -299,7 +306,8 @@ class HopSkipJump(EvasionAttack):
                     random_img = random_img * mask + x * (1 - mask)
 
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class == y:
@@ -333,7 +341,8 @@ class HopSkipJump(EvasionAttack):
                     random_img = random_img * mask + x * (1 - mask)
 
                 random_class = np.argmax(
-                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size), axis=1,
+                    self.estimator.predict(np.array([random_img]), batch_size=self.batch_size),
+                    axis=1,
                 )[0]
 
                 if random_class != y_p:
@@ -385,7 +394,10 @@ class HopSkipJump(EvasionAttack):
         for _ in range(self.max_iter):
             # First compute delta
             delta = self._compute_delta(
-                current_sample=current_sample, original_sample=original_sample, clip_min=clip_min, clip_max=clip_max,
+                current_sample=current_sample,
+                original_sample=original_sample,
+                clip_min=clip_min,
+                clip_max=clip_max,
             )
 
             # Then run binary search
@@ -424,7 +436,10 @@ class HopSkipJump(EvasionAttack):
                 epsilon /= 2.0
                 potential_sample = current_sample + epsilon * update
                 success = self._adversarial_satisfactory(
-                    samples=potential_sample[None], target=target, clip_min=clip_min, clip_max=clip_max,
+                    samples=potential_sample[None],
+                    target=target,
+                    clip_min=clip_min,
+                    clip_max=clip_max,
                 )
 
             # Update current sample
@@ -433,6 +448,11 @@ class HopSkipJump(EvasionAttack):
             # Update current iteration
             self.curr_iter += 1
 
+            # If attack failed. return original sample
+            if np.isnan(current_sample).any():  # pragma: no cover
+                logger.debug("NaN detected in sample, returning original sample.")
+                return original_sample
+
         return current_sample
 
     def _binary_search(
@@ -478,24 +498,37 @@ class HopSkipJump(EvasionAttack):
             # Interpolation point
             alpha = (upper_bound + lower_bound) / 2.0
             interpolated_sample = self._interpolate(
-                current_sample=current_sample, original_sample=original_sample, alpha=alpha, norm=norm,
+                current_sample=current_sample,
+                original_sample=original_sample,
+                alpha=alpha,
+                norm=norm,
             )
 
             # Update upper_bound and lower_bound
             satisfied = self._adversarial_satisfactory(
-                samples=interpolated_sample[None], target=target, clip_min=clip_min, clip_max=clip_max,
+                samples=interpolated_sample[None],
+                target=target,
+                clip_min=clip_min,
+                clip_max=clip_max,
             )[0]
             lower_bound = np.where(satisfied == 0, alpha, lower_bound)
             upper_bound = np.where(satisfied == 1, alpha, upper_bound)
 
         result = self._interpolate(
-            current_sample=current_sample, original_sample=original_sample, alpha=upper_bound, norm=norm,
+            current_sample=current_sample,
+            original_sample=original_sample,
+            alpha=upper_bound,
+            norm=norm,
         )
 
         return result
 
     def _compute_delta(
-        self, current_sample: np.ndarray, original_sample: np.ndarray, clip_min: float, clip_max: float,
+        self,
+        current_sample: np.ndarray,
+        original_sample: np.ndarray,
+        clip_min: float,
+        clip_max: float,
     ) -> float:
         """
         Compute the delta parameter.
@@ -557,7 +590,11 @@ class HopSkipJump(EvasionAttack):
 
         # Normalize random noise to fit into the range of input data
         rnd_noise = rnd_noise / np.sqrt(
-            np.sum(rnd_noise ** 2, axis=tuple(range(len(rnd_noise_shape)))[1:], keepdims=True,)
+            np.sum(
+                rnd_noise ** 2,
+                axis=tuple(range(len(rnd_noise_shape)))[1:],
+                keepdims=True,
+            )
         )
         eval_samples = np.clip(current_sample + delta * rnd_noise, clip_min, clip_max)
         rnd_noise = (eval_samples - current_sample) / delta
@@ -633,19 +670,19 @@ class HopSkipJump(EvasionAttack):
         if self.norm not in [2, np.inf, "inf"]:
             raise ValueError('Norm order must be either 2, `np.inf` or "inf".')
 
-        if not isinstance(self.max_iter, (int, np.int)) or self.max_iter < 0:
+        if not isinstance(self.max_iter, int) or self.max_iter < 0:
             raise ValueError("The number of iterations must be a non-negative integer.")
 
-        if not isinstance(self.max_eval, (int, np.int)) or self.max_eval <= 0:
+        if not isinstance(self.max_eval, int) or self.max_eval <= 0:
             raise ValueError("The maximum number of evaluations must be a positive integer.")
 
-        if not isinstance(self.init_eval, (int, np.int)) or self.init_eval <= 0:
+        if not isinstance(self.init_eval, int) or self.init_eval <= 0:
             raise ValueError("The initial number of evaluations must be a positive integer.")
 
         if self.init_eval > self.max_eval:
             raise ValueError("The maximum number of evaluations must be larger than the initial number of evaluations.")
 
-        if not isinstance(self.init_size, (int, np.int)) or self.init_size <= 0:
+        if not isinstance(self.init_size, int) or self.init_size <= 0:
             raise ValueError("The number of initial trials must be a positive integer.")
 
         if not isinstance(self.verbose, bool):
