@@ -30,13 +30,14 @@ from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
 from scipy.stats import truncnorm
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.attacks.evasion.fast_gradient import FastGradientMethod
 from art.config import ART_NUMPY_DTYPE
 from art.estimators.classification.classifier import ClassifierMixin
 from art.estimators.estimator import BaseEstimator, LossGradientsMixin
 from art.utils import compute_success, get_labels_np_array, check_and_transform_label_format, compute_success_array
+from art.summary_writer import SummaryWriter
 
 if TYPE_CHECKING:
     from art.utils import CLASSIFIER_LOSS_GRADIENTS_TYPE, OBJECT_DETECTOR_TYPE
@@ -68,6 +69,7 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
         num_random_init: int = 0,
         batch_size: int = 32,
         random_eps: bool = False,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ) -> None:
         """
@@ -86,6 +88,13 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
         :param num_random_init: Number of random initialisations within the epsilon ball. For num_random_init=0
             starting at the original input.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         :param verbose: Show progress bars.
         """
         super().__init__(
@@ -97,6 +106,7 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
             num_random_init=num_random_init,
             batch_size=batch_size,
             minimal=False,
+            summary_writer=summary_writer,
         )
         self.max_iter = max_iter
         self.random_eps = random_eps
@@ -106,12 +116,12 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
         if self.random_eps:
             if isinstance(eps, (int, float)):
                 lower, upper = 0, eps
-                mu, sigma = 0, (eps / 2)
+                var_mu, sigma = 0, (eps / 2)
             else:
                 lower, upper = np.zeros_like(eps), eps
-                mu, sigma = np.zeros_like(eps), (eps / 2)
+                var_mu, sigma = np.zeros_like(eps), (eps / 2)
 
-            self.norm_dist = truncnorm((lower - mu) / sigma, (upper - mu) / sigma, loc=mu, scale=sigma)
+            self.norm_dist = truncnorm((lower - var_mu) / sigma, (upper - var_mu) / sigma, loc=var_mu, scale=sigma)
 
     def _random_eps(self):
         """
@@ -144,7 +154,7 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
 
         if y is None:
             # Throw error if attack is targeted, but no targets are provided
-            if self.targeted:
+            if self.targeted:  # pragma: no cover
                 raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
             # Use model predictions as correct outputs
@@ -158,17 +168,59 @@ class ProjectedGradientDescentCommon(FastGradientMethod):
 
         return targets
 
-    def _check_params(self) -> None:
-        super(ProjectedGradientDescentCommon, self)._check_params()
+    def _check_params(self) -> None:  # pragma: no cover
 
-        if (self.norm in ["inf", np.inf]) and (
-            (isinstance(self.eps, (int, float)) and self.eps_step > self.eps)
-            or (isinstance(self.eps, np.ndarray) and (self.eps_step > self.eps).any())
+        if self.norm not in [1, 2, np.inf, "inf"]:
+            raise ValueError('Norm order must be either 1, 2, `np.inf` or "inf".')
+
+        if not (
+            isinstance(self.eps, (int, float))
+            and isinstance(self.eps_step, (int, float))
+            or isinstance(self.eps, np.ndarray)
+            and isinstance(self.eps_step, np.ndarray)
         ):
-            raise ValueError("The iteration step `eps_step` has to be smaller than the total attack `eps`.")
+            raise TypeError(
+                "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same type of `int`"
+                ", `float`, or `np.ndarray`."
+            )
 
-        if self.max_iter <= 0:
-            raise ValueError("The number of iterations `max_iter` has to be a positive integer.")
+        if isinstance(self.eps, (int, float)):
+            if self.eps < 0:
+                raise ValueError("The perturbation size `eps` has to be non-negative.")
+        else:
+            if (self.eps < 0).any():
+                raise ValueError("The perturbation size `eps` has to be non-negative.")
+
+        if isinstance(self.eps_step, (int, float)):
+            if self.eps_step <= 0:
+                raise ValueError("The perturbation step-size `eps_step` has to be positive.")
+        else:
+            if (self.eps_step <= 0).any():
+                raise ValueError("The perturbation step-size `eps_step` has to be positive.")
+
+        if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
+            if self.eps.shape != self.eps_step.shape:
+                raise ValueError(
+                    "The perturbation size `eps` and the perturbation step-size `eps_step` must have the same shape."
+                )
+
+        if not isinstance(self.targeted, bool):
+            raise ValueError("The flag `targeted` has to be of type bool.")
+
+        if not isinstance(self.num_random_init, int):
+            raise TypeError("The number of random initialisations has to be of type integer.")
+
+        if self.num_random_init < 0:
+            raise ValueError("The number of random initialisations `random_init` has to be greater than or equal to 0.")
+
+        if self.batch_size <= 0:
+            raise ValueError("The batch size `batch_size` has to be positive.")
+
+        if self.max_iter < 0:
+            raise ValueError("The number of iterations `max_iter` has to be a non-negative integer.")
+
+        if not isinstance(self.verbose, bool):
+            raise ValueError("The verbose has to be a Boolean.")
 
 
 class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
@@ -191,6 +243,7 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
         num_random_init: int = 0,
         batch_size: int = 32,
         random_eps: bool = False,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ) -> None:
         """
@@ -209,8 +262,18 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
         :param num_random_init: Number of random initialisations within the epsilon ball. For num_random_init=0 starting
                                 at the original input.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         :param verbose: Show progress bars.
         """
+        if summary_writer and num_random_init > 1:
+            raise ValueError("TensorBoard is not yet supported for more than 1 random restart (num_random_init>1).")
+
         super().__init__(
             estimator=estimator,
             norm=norm,
@@ -221,6 +284,7 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
             num_random_init=num_random_init,
             batch_size=batch_size,
             random_eps=random_eps,
+            summary_writer=summary_writer,
             verbose=verbose,
         )
 
@@ -258,6 +322,9 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
             adv_x = x.astype(ART_NUMPY_DTYPE)
 
             for batch_id in range(int(np.ceil(x.shape[0] / float(self.batch_size)))):
+
+                self._batch_id = batch_id
+
                 for rand_init_num in trange(
                     max(1, self.num_random_init), desc="PGD - Random Initializations", disable=not self.verbose
                 ):
@@ -274,6 +341,8 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
                     for i_max_iter in trange(
                         self.max_iter, desc="PGD - Iterations", leave=False, disable=not self.verbose
                     ):
+                        self._i_max_iter = i_max_iter
+
                         batch = self._compute(
                             batch,
                             x[batch_index_1:batch_index_2],
@@ -283,6 +352,7 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
                             self.eps_step,
                             self._project,
                             self.num_random_init > 0 and i_max_iter == 0,
+                            self._batch_id,
                         )
 
                     if rand_init_num == 0:
@@ -292,7 +362,7 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
                     else:
                         # replace adversarial examples if they are successful
                         attack_success = compute_success_array(
-                            self.estimator,
+                            self.estimator,  # type: ignore
                             x[batch_index_1:batch_index_2],
                             targets[batch_index_1:batch_index_2],
                             batch,
@@ -305,23 +375,30 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
                 "Success rate of attack: %.2f%%",
                 100
                 * compute_success(
-                    self.estimator, x, targets, adv_x, self.targeted, batch_size=self.batch_size,  # type: ignore
+                    self.estimator,  # type: ignore
+                    x,
+                    targets,
+                    adv_x,
+                    self.targeted,
+                    batch_size=self.batch_size,  # type: ignore
                 ),
             )
         else:
-            if self.num_random_init > 0:
+            if self.num_random_init > 0:  # pragma: no cover
                 raise ValueError("Random initialisation is only supported for classification.")
 
             # Set up targets
             targets = self._set_targets(x, y, classifier_mixin=False)
 
             # Start to compute adversarial examples
-            if x.dtype == np.object:
+            if x.dtype == object:
                 adv_x = x.copy()
             else:
                 adv_x = x.astype(ART_NUMPY_DTYPE)
 
             for i_max_iter in trange(self.max_iter, desc="PGD - Iterations", disable=not self.verbose):
+                self._i_max_iter = i_max_iter
+
                 adv_x = self._compute(
                     adv_x,
                     x,
@@ -333,4 +410,7 @@ class ProjectedGradientDescentNumpy(ProjectedGradientDescentCommon):
                     self.num_random_init > 0 and i_max_iter == 0,
                 )
 
+        if self.summary_writer is not None:
+            self.summary_writer.reset()
+
         return adv_x
