@@ -29,9 +29,10 @@ import logging
 from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from art.config import ART_NUMPY_DTYPE
+from art.summary_writer import SummaryWriter
 from art.estimators.estimator import BaseEstimator, LossGradientsMixin
 from art.estimators.classification.classifier import ClassifierMixin
 from art.attacks.evasion.projected_gradient_descent.projected_gradient_descent_numpy import (
@@ -40,6 +41,7 @@ from art.attacks.evasion.projected_gradient_descent.projected_gradient_descent_n
 from art.utils import compute_success, random_sphere, compute_success_array
 
 if TYPE_CHECKING:
+    # pylint: disable=C0412
     import torch
     from art.estimators.classification.pytorch import PyTorchClassifier
 
@@ -55,7 +57,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
     | Paper link: https://arxiv.org/abs/1706.06083
     """
 
-    _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)
+    _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)  # type: ignore
 
     def __init__(
         self,
@@ -68,6 +70,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         num_random_init: int = 0,
         batch_size: int = 32,
         random_eps: bool = False,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ):
         """
@@ -86,6 +89,13 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         :param num_random_init: Number of random initialisations within the epsilon ball. For num_random_init=0 starting
                                 at the original input.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         :param verbose: Show progress bars.
         """
         if not estimator.all_framework_preprocessing:
@@ -93,7 +103,10 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                 "The framework-specific implementation only supports framework-specific preprocessing."
             )
 
-        super(ProjectedGradientDescentPyTorch, self).__init__(
+        if summary_writer and num_random_init > 1:
+            raise ValueError("TensorBoard is not yet supported for more than 1 random restart (num_random_init>1).")
+
+        super().__init__(
             estimator=estimator,
             norm=norm,
             eps=eps,
@@ -104,8 +117,12 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
             batch_size=batch_size,
             random_eps=random_eps,
             verbose=verbose,
+            summary_writer=summary_writer,
         )
 
+        self._batch_id = 0
+        self._i_max_iter = 0
+
     def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
         """
         Generate adversarial samples and return them in an array.
@@ -154,7 +171,8 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         else:
             dataset = torch.utils.data.TensorDataset(
-                torch.from_numpy(x.astype(ART_NUMPY_DTYPE)), torch.from_numpy(targets.astype(ART_NUMPY_DTYPE)),
+                torch.from_numpy(x.astype(ART_NUMPY_DTYPE)),
+                torch.from_numpy(targets.astype(ART_NUMPY_DTYPE)),
             )
 
         data_loader = torch.utils.data.DataLoader(
@@ -168,6 +186,9 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         for (batch_id, batch_all) in enumerate(
             tqdm(data_loader, desc="PGD - Batches", leave=False, disable=not self.verbose)
         ):
+
+            self._batch_id = batch_id
+
             if mask is not None:
                 (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], batch_all[2]
             else:
@@ -176,7 +197,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
 
             # Compute batch_eps and batch_eps_step
-            if isinstance(self.eps, np.ndarray):
+            if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
                 if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
                     batch_eps = self.eps[batch_index_1:batch_index_2]
                     batch_eps_step = self.eps_step[batch_index_1:batch_index_2]
@@ -213,9 +234,12 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         logger.info(
             "Success rate of attack: %.2f%%",
-            100 * compute_success(self.estimator, x, y, adv_x, self.targeted, batch_size=self.batch_size),
+            100 * compute_success(self.estimator, x, targets, adv_x, self.targeted, batch_size=self.batch_size),
         )
 
+        if self.summary_writer is not None:
+            self.summary_writer.reset()
+
         return adv_x
 
     def _generate_batch(
@@ -238,21 +262,30 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         :param eps_step: Attack step size (input variation) at each iteration.
         :return: Adversarial examples.
         """
+        import torch  # lgtm [py/repeated-import]
+
         inputs = x.to(self.estimator.device)
         targets = targets.to(self.estimator.device)
-        adv_x = inputs
+        adv_x = torch.clone(inputs)
 
         if mask is not None:
             mask = mask.to(self.estimator.device)
 
         for i_max_iter in range(self.max_iter):
+            self._i_max_iter = i_max_iter
             adv_x = self._compute_torch(
-                adv_x, inputs, targets, mask, eps, eps_step, self.num_random_init > 0 and i_max_iter == 0,
+                adv_x,
+                inputs,
+                targets,
+                mask,
+                eps,
+                eps_step,
+                self.num_random_init > 0 and i_max_iter == 0,
             )
 
         return adv_x.cpu().detach().numpy()
 
-    def _compute_perturbation(
+    def _compute_perturbation(  # pylint: disable=W0221
         self, x: "torch.Tensor", y: "torch.Tensor", mask: Optional["torch.Tensor"]
     ) -> "torch.Tensor":
         """
@@ -276,9 +309,27 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         # Get gradient wrt loss; invert it if attack is targeted
         grad = self.estimator.loss_gradient(x=x, y=y) * (1 - 2 * int(self.targeted))
 
+        # Write summary
+        if self.summary_writer is not None:  # pragma: no cover
+            self.summary_writer.update(
+                batch_id=self._batch_id,
+                global_step=self._i_max_iter,
+                grad=grad.cpu().detach().numpy(),
+                patch=None,
+                estimator=self.estimator,
+                x=x.cpu().detach().numpy(),
+                y=y.cpu().detach().numpy(),
+                targeted=self.targeted,
+            )
+
+        # Check for nan before normalisation an replace with 0
+        if torch.any(grad.isnan()):  # pragma: no cover
+            logger.warning("Elements of the loss gradient are NaN and have been replaced with 0.0.")
+            grad[grad.isnan()] = 0.0
+
         # Apply mask
         if mask is not None:
-            grad = torch.where(mask == 0.0, torch.tensor(0.0), grad)
+            grad = torch.where(mask == 0.0, torch.tensor(0.0).to(self.estimator.device), grad)
 
         # Apply norm bound
         if self.norm in ["inf", np.inf]:
@@ -296,7 +347,7 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
 
         return grad
 
-    def _apply_perturbation(
+    def _apply_perturbation(  # pylint: disable=W0221
         self, x: "torch.Tensor", perturbation: "torch.Tensor", eps_step: Union[int, float, np.ndarray]
     ) -> "torch.Tensor":
         """
@@ -310,8 +361,9 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
         import torch  # lgtm [py/repeated-import]
 
         eps_step = np.array(eps_step, dtype=ART_NUMPY_DTYPE)
-        x = x + torch.tensor(eps_step).to(self.estimator.device) * perturbation
-
+        perturbation_step = torch.tensor(eps_step).to(self.estimator.device) * perturbation
+        perturbation_step[torch.isnan(perturbation_step)] = 0
+        x = x + perturbation_step
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
             x = torch.max(
@@ -410,10 +462,13 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                     "The parameter `eps` of type `np.ndarray` is not supported to use with norm 2."
                 )
 
-            values_tmp = values_tmp * torch.min(
-                torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
-                eps / (torch.norm(values_tmp, p=2, dim=1) + tol),
-            ).unsqueeze_(-1)
+            values_tmp = (
+                values_tmp
+                * torch.min(
+                    torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
+                    eps / (torch.norm(values_tmp, p=2, dim=1) + tol),
+                ).unsqueeze_(-1)
+            )
 
         elif norm_p == 1:
             if isinstance(eps, np.ndarray):
@@ -421,10 +476,13 @@ class ProjectedGradientDescentPyTorch(ProjectedGradientDescentCommon):
                     "The parameter `eps` of type `np.ndarray` is not supported to use with norm 1."
                 )
 
-            values_tmp = values_tmp * torch.min(
-                torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
-                eps / (torch.norm(values_tmp, p=1, dim=1) + tol),
-            ).unsqueeze_(-1)
+            values_tmp = (
+                values_tmp
+                * torch.min(
+                    torch.tensor([1.0], dtype=torch.float32).to(self.estimator.device),
+                    eps / (torch.norm(values_tmp, p=1, dim=1) + tol),
+                ).unsqueeze_(-1)
+            )
 
         elif norm_p in [np.inf, "inf"]:
             if isinstance(eps, np.ndarray):
