@@ -29,7 +29,7 @@ import logging
 from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from art.config import ART_NUMPY_DTYPE
 from art.estimators.estimator import BaseEstimator, LossGradientsMixin
@@ -38,8 +38,10 @@ from art.attacks.evasion.projected_gradient_descent.projected_gradient_descent_n
     ProjectedGradientDescentCommon,
 )
 from art.utils import compute_success, random_sphere, compute_success_array
+from art.summary_writer import SummaryWriter
 
 if TYPE_CHECKING:
+    # pylint: disable=C0412
     import tensorflow as tf
     from art.estimators.classification.tensorflow import TensorFlowV2Classifier
 
@@ -55,7 +57,7 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
     | Paper link: https://arxiv.org/abs/1706.06083
     """
 
-    _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)
+    _estimator_requirements = (BaseEstimator, LossGradientsMixin, ClassifierMixin)  # type: ignore
 
     def __init__(
         self,
@@ -68,6 +70,7 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
         num_random_init: int = 0,
         batch_size: int = 32,
         random_eps: bool = False,
+        summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ):
         """
@@ -86,6 +89,13 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
         :param num_random_init: Number of random initialisations within the epsilon ball. For num_random_init=0 starting
                                 at the original input.
         :param batch_size: Size of the batch on which adversarial samples are generated.
+        :param summary_writer: Activate summary writer for TensorBoard.
+                               Default is `False` and deactivated summary writer.
+                               If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
+                               If of type `str` save in path.
+                               If of type `SummaryWriter` apply provided custom summary writer.
+                               Use hierarchical folder structure to compare between runs easily. e.g. pass in
+                               ‘runs/exp1’, ‘runs/exp2’, etc. for each new experiment to compare across them.
         :param verbose: Show progress bars.
         """
         if not estimator.all_framework_preprocessing:
@@ -93,6 +103,9 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
                 "The framework-specific implementation only supports framework-specific preprocessing."
             )
 
+        if summary_writer and num_random_init > 1:
+            raise ValueError("TensorBoard is not yet supported for more than 1 random restart (num_random_init>1).")
+
         super().__init__(
             estimator=estimator,
             norm=norm,
@@ -103,6 +116,7 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
             num_random_init=num_random_init,
             batch_size=batch_size,
             random_eps=random_eps,
+            summary_writer=summary_writer,
             verbose=verbose,
         )
 
@@ -140,7 +154,11 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
             # those for the current batch. Otherwise (i.e. mask is meant to be broadcasted), keep it as it is.
             if len(mask.shape) == len(x.shape):
                 dataset = tf.data.Dataset.from_tensor_slices(
-                    (x.astype(ART_NUMPY_DTYPE), targets.astype(ART_NUMPY_DTYPE), mask.astype(ART_NUMPY_DTYPE),)
+                    (
+                        x.astype(ART_NUMPY_DTYPE),
+                        targets.astype(ART_NUMPY_DTYPE),
+                        mask.astype(ART_NUMPY_DTYPE),
+                    )
                 ).batch(self.batch_size, drop_remainder=False)
 
             else:
@@ -154,7 +172,10 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
 
         else:
             dataset = tf.data.Dataset.from_tensor_slices(
-                (x.astype(ART_NUMPY_DTYPE), targets.astype(ART_NUMPY_DTYPE),)
+                (
+                    x.astype(ART_NUMPY_DTYPE),
+                    targets.astype(ART_NUMPY_DTYPE),
+                )
             ).batch(self.batch_size, drop_remainder=False)
 
         # Start to compute adversarial examples
@@ -165,6 +186,9 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
         for (batch_id, batch_all) in enumerate(
             tqdm(data_loader, desc="PGD - Batches", leave=False, disable=not self.verbose)
         ):
+
+            self._batch_id = batch_id
+
             if mask is not None:
                 (batch, batch_labels, mask_batch) = batch_all[0], batch_all[1], batch_all[2]
             else:
@@ -173,7 +197,7 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
             batch_index_1, batch_index_2 = batch_id * self.batch_size, (batch_id + 1) * self.batch_size
 
             # Compute batch_eps and batch_eps_step
-            if isinstance(self.eps, np.ndarray):
+            if isinstance(self.eps, np.ndarray) and isinstance(self.eps_step, np.ndarray):
                 if len(self.eps.shape) == len(x.shape) and self.eps.shape[0] == x.shape[0]:
                     batch_eps = self.eps[batch_index_1:batch_index_2]
                     batch_eps_step = self.eps_step[batch_index_1:batch_index_2]
@@ -209,9 +233,12 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
 
         logger.info(
             "Success rate of attack: %.2f%%",
-            100 * compute_success(self.estimator, x, y, adv_x, self.targeted, batch_size=self.batch_size),
+            100 * compute_success(self.estimator, x, targets, adv_x, self.targeted, batch_size=self.batch_size),
         )
 
+        if self.summary_writer is not None:
+            self.summary_writer.reset()
+
         return adv_x
 
     def _generate_batch(
@@ -234,15 +261,27 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
         :param eps_step: Attack step size (input variation) at each iteration.
         :return: Adversarial examples.
         """
-        adv_x = x
+        import tensorflow as tf  # lgtm [py/repeated-import]
+
+        adv_x = tf.identity(x)
+
         for i_max_iter in range(self.max_iter):
+            self._i_max_iter = i_max_iter
             adv_x = self._compute_tf(
-                adv_x, x, targets, mask, eps, eps_step, self.num_random_init > 0 and i_max_iter == 0,
+                adv_x,
+                x,
+                targets,
+                mask,
+                eps,
+                eps_step,
+                self.num_random_init > 0 and i_max_iter == 0,
             )
 
         return adv_x
 
-    def _compute_perturbation(self, x: "tf.Tensor", y: "tf.Tensor", mask: Optional["tf.Tensor"]) -> "tf.Tensor":
+    def _compute_perturbation(  # pylint: disable=W0221
+        self, x: "tf.Tensor", y: "tf.Tensor", mask: Optional["tf.Tensor"]
+    ) -> "tf.Tensor":
         """
         Compute perturbations.
 
@@ -266,6 +305,24 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
             1 - 2 * int(self.targeted), dtype=ART_NUMPY_DTYPE
         )
 
+        # Write summary
+        if self.summary_writer is not None:  # pragma: no cover
+            self.summary_writer.update(
+                batch_id=self._batch_id,
+                global_step=self._i_max_iter,
+                grad=grad.numpy(),
+                patch=None,
+                estimator=self.estimator,
+                x=x.numpy(),
+                y=y.numpy(),
+                targeted=self.targeted,
+            )
+
+        # Check for NaN before normalisation an replace with 0
+        if tf.reduce_any(tf.math.is_nan(grad)):  # pragma: no cover
+            logger.warning("Elements of the loss gradient are NaN and have been replaced with 0.0.")
+            grad = tf.where(tf.math.is_nan(grad), tf.zeros_like(grad), grad)
+
         # Apply mask
         if mask is not None:
             grad = tf.where(mask == 0.0, 0.0, grad)
@@ -288,7 +345,7 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
 
         return grad
 
-    def _apply_perturbation(
+    def _apply_perturbation(  # pylint: disable=W0221
         self, x: "tf.Tensor", perturbation: "tf.Tensor", eps_step: Union[int, float, np.ndarray]
     ) -> "tf.Tensor":
         """
@@ -301,11 +358,12 @@ class ProjectedGradientDescentTensorFlowV2(ProjectedGradientDescentCommon):
         """
         import tensorflow as tf  # lgtm [py/repeated-import]
 
-        x = x + tf.constant(eps_step, dtype=ART_NUMPY_DTYPE) * perturbation
-
+        perturbation_step = tf.constant(eps_step, dtype=ART_NUMPY_DTYPE) * perturbation
+        perturbation_step = tf.where(tf.math.is_nan(perturbation_step), 0, perturbation_step)
+        x = x + perturbation_step
         if self.estimator.clip_values is not None:
             clip_min, clip_max = self.estimator.clip_values
-            x = tf.clip_by_value(x, clip_min, clip_max)
+            x = tf.clip_by_value(x, clip_value_min=clip_min, clip_value_max=clip_max)
 
         return x
 
