@@ -26,7 +26,7 @@ import logging
 from typing import Optional, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.attacks.attack import EvasionAttack
 from art.config import ART_NUMPY_DTYPE
@@ -98,6 +98,11 @@ class SaliencyMapMethod(EvasionAttack):
 
             targets = np.argmax(random_targets(preds, self.estimator.nb_classes), axis=1)
         else:
+            if self.estimator.nb_classes == 2 and y.shape[1] == 1:  # pragma: no cover
+                raise ValueError(
+                    "This attack has not yet been tested for binary classification with a single output classifier."
+                )
+
             targets = np.argmax(y, axis=1)
 
         # Compute perturbation with implicit batching
@@ -109,13 +114,15 @@ class SaliencyMapMethod(EvasionAttack):
 
             # Main algorithm for each batch
             # Initialize the search space; optimize to remove features that can't be changed
-            search_space = np.zeros(batch.shape)
             if self.estimator.clip_values is not None:
+                search_space = np.zeros(batch.shape)
                 clip_min, clip_max = self.estimator.clip_values
                 if self.theta > 0:
                     search_space[batch < clip_max] = 1
-                else:
+                else:  # pragma: no cover
                     search_space[batch > clip_min] = 1
+            else:
+                search_space = np.ones(batch.shape)
 
             # Get current predictions
             current_pred = preds[batch_index_1:batch_index_2]
@@ -140,16 +147,18 @@ class SaliencyMapMethod(EvasionAttack):
                     # Prepare update depending of theta
                     if self.theta > 0:
                         clip_func, clip_value = np.minimum, clip_max
-                    else:
+                    else:  # pragma: no cover
                         clip_func, clip_value = np.maximum, clip_min
 
                     # Update adversarial examples
                     tmp_batch = batch[active_indices]
                     tmp_batch[np.arange(len(active_indices)), feat_ind[:, 0]] = clip_func(
-                        clip_value, tmp_batch[np.arange(len(active_indices)), feat_ind[:, 0]] + self.theta,
+                        clip_value,
+                        tmp_batch[np.arange(len(active_indices)), feat_ind[:, 0]] + self.theta,
                     )
                     tmp_batch[np.arange(len(active_indices)), feat_ind[:, 1]] = clip_func(
-                        clip_value, tmp_batch[np.arange(len(active_indices)), feat_ind[:, 1]] + self.theta,
+                        clip_value,
+                        tmp_batch[np.arange(len(active_indices)), feat_ind[:, 1]] + self.theta,
                     )
                     batch[active_indices] = tmp_batch
 
@@ -164,7 +173,10 @@ class SaliencyMapMethod(EvasionAttack):
                     batch[active_indices] = tmp_batch
 
                 # Recompute model prediction
-                current_pred = np.argmax(self.estimator.predict(np.reshape(batch, [batch.shape[0]] + dims)), axis=1,)
+                current_pred = np.argmax(
+                    self.estimator.predict(np.reshape(batch, [batch.shape[0]] + dims)),
+                    axis=1,
+                )
 
                 # Update active_indices
                 active_indices = np.where(
@@ -204,7 +216,7 @@ class SaliencyMapMethod(EvasionAttack):
 
         if self.theta > 0:
             ind = np.argpartition(grads, -2, axis=1)[:, -2:]
-        else:
+        else:  # pragma: no cover
             ind = np.argpartition(-grads, -2, axis=1)[:, -2:]
 
         return ind
