@@ -24,14 +24,14 @@ import bisect
 import logging
 import math
 import random
-from typing import Optional, Union, TYPE_CHECKING
+from typing import Optional, Union, Callable, TYPE_CHECKING
 
 import numpy as np
 from tqdm.auto import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
-from art.estimators.estimator import BaseEstimator
+from art.estimators.estimator import BaseEstimator, NeuralNetworkMixin
 from art.estimators.classification.classifier import ClassifierMixin
 from art.utils import check_and_transform_label_format, get_labels_np_array
 
@@ -42,9 +42,16 @@ logger = logging.getLogger(__name__)
 
 
 class SquareAttack(EvasionAttack):
+    """
+    This class implements the `SquareAttack` attack.
+
+    | Paper link: https://arxiv.org/abs/1912.00049
+    """
 
     attack_params = EvasionAttack.attack_params + [
         "norm",
+        "adv_criterion",
+        "loss",
         "max_iter",
         "eps",
         "p_init",
@@ -53,12 +60,14 @@ class SquareAttack(EvasionAttack):
         "verbose",
     ]
 
-    _estimator_requirements = (BaseEstimator, ClassifierMixin)
+    _estimator_requirements = (BaseEstimator, NeuralNetworkMixin)
 
     def __init__(
         self,
         estimator: "CLASSIFIER_TYPE",
         norm: Union[int, float, str] = np.inf,
+        adv_criterion: Union[Callable[[np.ndarray, np.ndarray], bool], None] = None,
+        loss: Union[Callable[[np.ndarray, np.ndarray], np.ndarray], None] = None,
         max_iter: int = 100,
         eps: float = 0.3,
         p_init: float = 0.8,
@@ -71,6 +80,8 @@ class SquareAttack(EvasionAttack):
 
         :param estimator: An trained estimator.
         :param norm: The norm of the adversarial perturbation. Possible values: "inf", np.inf, 1 or 2.
+        :param adv_criterion: The criterion which the attack should use in determining adversariality.
+        :param loss: The loss function which the attack should use for optimization.
         :param max_iter: Maximum number of iterations.
         :param eps: Maximum perturbation that the attacker can introduce.
         :param p_init: Initial fraction of elements.
@@ -81,6 +92,21 @@ class SquareAttack(EvasionAttack):
         super().__init__(estimator=estimator)
 
         self.norm = norm
+
+        if adv_criterion is not None:
+            self.adv_criterion = adv_criterion
+        elif isinstance(self.estimator, ClassifierMixin):
+            self.adv_criterion = lambda y_pred, y: np.argmax(y_pred, axis=1) != np.argmax(y, axis=1)
+        else:  # pragma: no cover
+            raise ValueError("No acceptable adversarial criterion available.")
+
+        if loss is not None:
+            self.loss = loss
+        elif isinstance(self.estimator, ClassifierMixin):
+            self.loss = self._get_logits_diff
+        else:  # pragma: no cover
+            raise ValueError("No acceptable loss available.")
+
         self.max_iter = max_iter
         self.eps = eps
         self.p_init = p_init
@@ -118,17 +144,26 @@ class SquareAttack(EvasionAttack):
                   (explained in this paper: https://arxiv.org/abs/1611.01236). Default is `None`.
         :return: An array holding the adversarial examples.
         """
-        if x.ndim != 4:
+        if x.ndim != 4:  # pragma: no cover
             raise ValueError("Unrecognized input dimension. Attack can only be applied to image data.")
 
         x_adv = x.astype(ART_NUMPY_DTYPE)
 
-        y = check_and_transform_label_format(y, self.estimator.nb_classes)
+        if isinstance(self.estimator, ClassifierMixin):
+            y = check_and_transform_label_format(y, self.estimator.nb_classes)
 
         if y is None:
             # Use model predictions as true labels
             logger.info("Using model predictions as true labels.")
-            y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))
+            y = self.estimator.predict(x, batch_size=self.batch_size)
+            if isinstance(self.estimator, ClassifierMixin):
+                y = get_labels_np_array(y)
+
+        if isinstance(self.estimator, ClassifierMixin):
+            if self.estimator.nb_classes == 2 and y.shape[1] == 1:  # pragma: no cover
+                raise ValueError(
+                    "This attack has not yet been tested for binary classification with a single output classifier."
+                )
 
         if self.estimator.channels_first:
             channels = x.shape[1]
@@ -143,15 +178,15 @@ class SquareAttack(EvasionAttack):
 
             # Determine correctly predicted samples
             y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
-            sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
+            sample_is_robust = np.logical_not(self.adv_criterion(y_pred, y))
 
-            if np.sum(sample_is_robust) == 0:
+            if np.sum(sample_is_robust) == 0:  # pragma: no cover
                 break
 
             # x_robust = x_adv[sample_is_robust]
             x_robust = x[sample_is_robust]
             y_robust = y[sample_is_robust]
-            sample_logits_diff_init = self._get_logits_diff(x_robust, y_robust)
+            sample_loss_init = self.loss(x_robust, y_robust)
 
             if self.norm in [np.inf, "inf"]:
 
@@ -167,10 +202,10 @@ class SquareAttack(EvasionAttack):
                     a_max=self.estimator.clip_values[1],
                 ).astype(ART_NUMPY_DTYPE)
 
-                sample_logits_diff_new = self._get_logits_diff(x_robust_new, y_robust)
-                logits_diff_improved = (sample_logits_diff_new - sample_logits_diff_init) < 0.0
+                sample_loss_new = self.loss(x_robust_new, y_robust)
+                loss_improved = (sample_loss_new - sample_loss_init) < 0.0
 
-                x_robust[logits_diff_improved] = x_robust_new[logits_diff_improved]
+                x_robust[loss_improved] = x_robust_new[loss_improved]
 
                 x_adv[sample_is_robust] = x_robust
 
@@ -182,16 +217,16 @@ class SquareAttack(EvasionAttack):
 
                     # Determine correctly predicted samples
                     y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
-                    sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
+                    sample_is_robust = np.logical_not(self.adv_criterion(y_pred, y))
 
-                    if np.sum(sample_is_robust) == 0:
+                    if np.sum(sample_is_robust) == 0:  # pragma: no cover
                         break
 
                     x_robust = x_adv[sample_is_robust]
                     x_init = x[sample_is_robust]
                     y_robust = y[sample_is_robust]
 
-                    sample_logits_diff_init = self._get_logits_diff(x_robust, y_robust)
+                    sample_loss_init = self.loss(x_robust, y_robust)
 
                     height_tile = max(int(round(math.sqrt(percentage_of_elements * height * width))), 1)
 
@@ -217,10 +252,10 @@ class SquareAttack(EvasionAttack):
                         x_robust_new, a_min=self.estimator.clip_values[0], a_max=self.estimator.clip_values[1]
                     ).astype(ART_NUMPY_DTYPE)
 
-                    sample_logits_diff_new = self._get_logits_diff(x_robust_new, y_robust)
-                    logits_diff_improved = (sample_logits_diff_new - sample_logits_diff_init) < 0.0
+                    sample_loss_new = self.loss(x_robust_new, y_robust)
+                    loss_improved = (sample_loss_new - sample_loss_init) < 0.0
 
-                    x_robust[logits_diff_improved] = x_robust_new[logits_diff_improved]
+                    x_robust[loss_improved] = x_robust_new[loss_improved]
 
                     x_adv[sample_is_robust] = x_robust
 
@@ -230,24 +265,25 @@ class SquareAttack(EvasionAttack):
 
                 height_tile = height // n_tiles
 
-                def _get_perturbation(h):
-                    delta = np.zeros([h, h])
-                    gaussian_perturbation = np.zeros([h // 2, h])
+                def _get_perturbation(height):
+                    delta = np.zeros([height, height])
+                    gaussian_perturbation = np.zeros([height // 2, height])
 
-                    x_c = h // 4
-                    y_c = h // 2
+                    x_c = height // 4
+                    y_c = height // 2
 
                     for i_y in range(y_c):
                         gaussian_perturbation[
-                            max(x_c, 0) : min(x_c + (2 * i_y + 1), h // 2), max(0, y_c) : min(y_c + (2 * i_y + 1), h)
+                            max(x_c, 0) : min(x_c + (2 * i_y + 1), height // 2),
+                            max(0, y_c) : min(y_c + (2 * i_y + 1), height),
                         ] += 1.0 / ((i_y + 1) ** 2)
                         x_c -= 1
                         y_c -= 1
 
                     gaussian_perturbation /= np.sqrt(np.sum(gaussian_perturbation ** 2))
 
-                    delta[: h // 2] = gaussian_perturbation
-                    delta[h // 2 : h // 2 + gaussian_perturbation.shape[0]] = -gaussian_perturbation
+                    delta[: height // 2] = gaussian_perturbation
+                    delta[height // 2 : height // 2 + gaussian_perturbation.shape[0]] = -gaussian_perturbation
 
                     delta /= np.sqrt(np.sum(delta ** 2))
 
@@ -259,7 +295,7 @@ class SquareAttack(EvasionAttack):
 
                     return delta
 
-                delta_init = np.zeros(x_robust.shape)
+                delta_init = np.zeros(x_robust.shape, dtype=ART_NUMPY_DTYPE)
 
                 height_start = 0
                 for _ in range(n_tiles):
@@ -293,10 +329,10 @@ class SquareAttack(EvasionAttack):
                     self.estimator.clip_values[1],
                 )
 
-                sample_logits_diff_new = self._get_logits_diff(x_robust_new, y_robust)
-                logits_diff_improved = (sample_logits_diff_new - sample_logits_diff_init) < 0.0
+                sample_loss_new = self.loss(x_robust_new, y_robust)
+                loss_improved = (sample_loss_new - sample_loss_init) < 0.0
 
-                x_robust[logits_diff_improved] = x_robust_new[logits_diff_improved]
+                x_robust[loss_improved] = x_robust_new[loss_improved]
 
                 x_adv[sample_is_robust] = x_robust
 
@@ -308,16 +344,16 @@ class SquareAttack(EvasionAttack):
 
                     # Determine correctly predicted samples
                     y_pred = self.estimator.predict(x_adv, batch_size=self.batch_size)
-                    sample_is_robust = np.argmax(y_pred, axis=1) == np.argmax(y, axis=1)
+                    sample_is_robust = np.logical_not(self.adv_criterion(y_pred, y))
 
-                    if np.sum(sample_is_robust) == 0:
+                    if np.sum(sample_is_robust) == 0:  # pragma: no cover
                         break
 
                     x_robust = x_adv[sample_is_robust]
                     x_init = x[sample_is_robust]
                     y_robust = y[sample_is_robust]
 
-                    sample_logits_diff_init = self._get_logits_diff(x_robust, y_robust)
+                    sample_loss_init = self.loss(x_robust, y_robust)
 
                     delta_x_robust_init = x_robust - x_init
 
@@ -397,11 +433,11 @@ class SquareAttack(EvasionAttack):
                     if self.estimator.channels_first:
                         new_deltas_size = [x_init.shape[0], channels, height_tile, height_tile]
                         random_choice_size = [x_init.shape[0], channels, 1, 1]
-                        perturbation_size = [1, 1, height_tile, height_tile]
+                        perturbation_size = (1, 1, height_tile, height_tile)
                     else:
                         new_deltas_size = [x_init.shape[0], height_tile, height_tile, channels]
                         random_choice_size = [x_init.shape[0], 1, 1, channels]
-                        perturbation_size = [1, height_tile, height_tile, 1]
+                        perturbation_size = (1, height_tile, height_tile, 1)
 
                     delta_new = (
                         np.ones(new_deltas_size)
@@ -457,10 +493,10 @@ class SquareAttack(EvasionAttack):
                         self.estimator.clip_values[1],
                     )
 
-                    sample_logits_diff_new = self._get_logits_diff(x_robust_new, y_robust)
-                    logits_diff_improved = (sample_logits_diff_new - sample_logits_diff_init) < 0.0
+                    sample_loss_new = self.loss(x_robust_new, y_robust)
+                    loss_improved = (sample_loss_new - sample_loss_init) < 0.0
 
-                    x_robust[logits_diff_improved] = x_robust_new[logits_diff_improved]
+                    x_robust[loss_improved] = x_robust_new[loss_improved]
 
                     x_adv[sample_is_robust] = x_robust
 
