@@ -29,7 +29,7 @@ import types
 from typing import Any, Dict, Optional, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from art.attacks.attack import EvasionAttack
 from art.estimators.estimator import BaseEstimator
@@ -160,6 +160,11 @@ class UniversalPerturbation(EvasionAttack):
             logger.info("Using model predictions as true labels.")
             y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))
 
+        if self.estimator.nb_classes == 2 and y.shape[1] == 1:
+            raise ValueError(
+                "This attack has not yet been tested for binary classification with a single output classifier."
+            )
+
         y_index = np.argmax(y, axis=1)
 
         # Init universal perturbation
@@ -172,7 +177,7 @@ class UniversalPerturbation(EvasionAttack):
 
         # Generate the adversarial examples
         nb_iter = 0
-        pbar = tqdm(self.max_iter, desc="Universal perturbation", disable=not self.verbose)
+        pbar = tqdm(total=self.max_iter, desc="Universal perturbation", disable=not self.verbose)
 
         while fooling_rate < 1.0 - self.delta and nb_iter < self.max_iter:
             # Go through all the examples randomly
@@ -255,13 +260,13 @@ class UniversalPerturbation(EvasionAttack):
         if not isinstance(self.delta, (float, int)) or self.delta < 0 or self.delta > 1:
             raise ValueError("The desired accuracy must be in the range [0, 1].")
 
-        if not isinstance(self.max_iter, (int, np.int)) or self.max_iter <= 0:
+        if not isinstance(self.max_iter, int) or self.max_iter <= 0:
             raise ValueError("The number of iterations must be a positive integer.")
 
         if not isinstance(self.eps, (float, int)) or self.eps <= 0:
             raise ValueError("The eps coefficient must be a positive float.")
 
-        if not isinstance(self.batch_size, (int, np.int)) or self.batch_size <= 0:
+        if not isinstance(self.batch_size, int) or self.batch_size <= 0:
             raise ValueError("The batch_size must be a positive integer.")
 
         if not isinstance(self.verbose, bool):
