@@ -29,7 +29,7 @@ from typing import Optional, Tuple, TYPE_CHECKING
 
 import numpy as np
 from scipy.ndimage import zoom
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.config import ART_NUMPY_DTYPE
 from art.attacks.attack import EvasionAttack
@@ -146,18 +146,19 @@ class ZooAttack(EvasionAttack):
         self._init_size = 32
         if self.abort_early:
             self._early_stop_iters = self.max_iter // 10 if self.max_iter >= 10 else self.max_iter
-        self.nb_parallel = nb_parallel
 
         # Initialize noise variable to zero
         if self.input_is_feature_vector:
             self.use_resize = False
             self.use_importance = False
-            logger.info("Disable resizing and importance sampling because feature vector input has been detected.")
+            logger.info(  # pragma: no cover
+                "Disable resizing and importance sampling because feature vector input has been detected."
+            )
 
         if self.use_resize:
             if not self.estimator.channels_first:
                 dims = (batch_size, self._init_size, self._init_size, self.estimator.input_shape[-1])
-            else:
+            else:  # pragma: no cover
                 dims = (batch_size, self.estimator.input_shape[0], self._init_size, self._init_size)
             self._current_noise = np.zeros(dims, dtype=ART_NUMPY_DTYPE)
         else:
@@ -186,7 +187,10 @@ class ZooAttack(EvasionAttack):
         ]
         preds = self.estimator.predict(np.array(zoom(x_adv, zoom=ratios)), batch_size=self.batch_size)
         z_target = np.sum(preds * target, axis=1)
-        z_other = np.max(preds * (1 - target) + (np.min(preds, axis=1) - 1)[:, np.newaxis] * target, axis=1,)
+        z_other = np.max(
+            preds * (1 - target) + (np.min(preds, axis=1) - 1)[:, np.newaxis] * target,
+            axis=1,
+        )
 
         if self.targeted:
             # If targeted, optimize for making the target class most likely
@@ -209,13 +213,18 @@ class ZooAttack(EvasionAttack):
         y = check_and_transform_label_format(y, self.estimator.nb_classes)
 
         # Check that `y` is provided for targeted attacks
-        if self.targeted and y is None:
+        if self.targeted and y is None:  # pragma: no cover
             raise ValueError("Target labels `y` need to be provided for a targeted attack.")
 
         # No labels provided, use model prediction as correct class
         if y is None:
             y = get_labels_np_array(self.estimator.predict(x, batch_size=self.batch_size))
 
+        if self.estimator.nb_classes == 2 and y.shape[1] == 1:  # pragma: no cover
+            raise ValueError(
+                "This attack has not yet been tested for binary classification with a single output classifier."
+            )
+
         # Compute adversarial examples with implicit batching
         nb_batches = int(np.ceil(x.shape[0] / float(self.batch_size)))
         x_adv = []
@@ -260,7 +269,10 @@ class ZooAttack(EvasionAttack):
         # Start with a binary search
         for bss in range(self.binary_search_steps):
             logger.debug(
-                "Binary search step %i out of %i (c_mean==%f)", bss, self.binary_search_steps, np.mean(c_current),
+                "Binary search step %i out of %i (c_mean==%f)",
+                bss,
+                self.binary_search_steps,
+                np.mean(c_current),
             )
 
             # Run with 1 specific binary search step
@@ -390,9 +402,7 @@ class ZooAttack(EvasionAttack):
             # Reset Adam if a valid example has been found to avoid overshoot
             mask_fine_tune = (~fine_tuning) & (loss == l2dist) & (prev_loss != prev_l2dist)
             fine_tuning[mask_fine_tune] = True
-            self._reset_adam(
-                self.adam_mean.size, np.repeat(mask_fine_tune, x_adv[0].size)  # type: ignore
-            )
+            self._reset_adam(self.adam_mean.size, np.repeat(mask_fine_tune, x_adv[0].size))  # type: ignore
             prev_l2dist = l2dist
 
             # Abort early if no improvement is obtained
@@ -415,12 +425,22 @@ class ZooAttack(EvasionAttack):
             if not self.estimator.channels_first:
                 best_attack = zoom(
                     best_attack,
-                    [1, int(x_batch.shape[1]) / best_attack.shape[1], int(x_batch.shape[2]) / best_attack.shape[2], 1,],
+                    [
+                        1,
+                        int(x_batch.shape[1]) / best_attack.shape[1],
+                        int(x_batch.shape[2]) / best_attack.shape[2],
+                        1,
+                    ],
                 )
             else:
                 best_attack = zoom(
                     best_attack,
-                    [1, 1, int(x_batch.shape[2]) / best_attack.shape[2], int(x_batch.shape[2]) / best_attack.shape[3],],
+                    [
+                        1,
+                        1,
+                        int(x_batch.shape[2]) / best_attack.shape[2],
+                        int(x_batch.shape[2]) / best_attack.shape[3],
+                    ],
                 )
 
         return best_dist, best_label, best_attack
@@ -442,12 +462,23 @@ class ZooAttack(EvasionAttack):
                 % coord_batch.shape[-1]
             )
         else:
-            indices = (
-                np.random.choice(
-                    coord_batch.shape[-1] * x.shape[0], self.nb_parallel * self._current_noise.shape[0], replace=False,
+            try:
+                indices = (
+                    np.random.choice(
+                        coord_batch.shape[-1] * x.shape[0],
+                        self.nb_parallel * self._current_noise.shape[0],
+                        replace=False,
+                    )
+                    % coord_batch.shape[-1]
                 )
-                % coord_batch.shape[-1]
-            )
+            except ValueError as error:  # pragma: no cover
+                if "Cannot take a larger sample than population when 'replace=False'" in str(error):
+                    raise ValueError(
+                        "Too many samples are requested for the random indices. Try to reduce the number of parallel"
+                        "coordinate updates `nb_parallel`."
+                    ) from error
+
+                raise error
 
         # Create the batch of modifications to run
         for i in range(self.nb_parallel * self._current_noise.shape[0]):
@@ -459,7 +490,10 @@ class ZooAttack(EvasionAttack):
         expanded_targets = np.repeat(targets, 2 * self.nb_parallel, axis=0).reshape((-1,) + targets.shape[1:])
         expanded_c = np.repeat(c_batch, 2 * self.nb_parallel)
         _, _, loss = self._loss(
-            expanded_x, expanded_x + coord_batch.reshape(expanded_x.shape), expanded_targets, expanded_c,
+            expanded_x,
+            expanded_x + coord_batch.reshape(expanded_x.shape),
+            expanded_targets,
+            expanded_c,
         )
         self._current_noise = self._optimizer_adam_coordinate(
             loss,
@@ -527,7 +561,7 @@ class ZooAttack(EvasionAttack):
             # Allocate Adam variables
             self.adam_mean = np.zeros(nb_vars, dtype=ART_NUMPY_DTYPE)
             self.adam_var = np.zeros(nb_vars, dtype=ART_NUMPY_DTYPE)
-            self.adam_epochs = np.ones(nb_vars, dtype=np.int32)
+            self.adam_epochs = np.ones(nb_vars, dtype=int)
 
     def _resize_image(self, x: np.ndarray, size_x: int, size_y: int, reset: bool = False) -> np.ndarray:
         if not self.estimator.channels_first:
@@ -545,7 +579,15 @@ class ZooAttack(EvasionAttack):
                 else:
                     self._current_noise = np.zeros(x.shape, dtype=ART_NUMPY_DTYPE)
             else:
-                resized_x = zoom(x, (1, dims[1] / x.shape[1], dims[2] / x.shape[2], dims[3] / x.shape[3],),)
+                resized_x = zoom(
+                    x,
+                    (
+                        1,
+                        dims[1] / x.shape[1],
+                        dims[2] / x.shape[2],
+                        dims[3] / x.shape[3],
+                    ),
+                )
                 self._current_noise = np.zeros(dims, dtype=ART_NUMPY_DTYPE)
             self._sample_prob = np.ones(nb_vars, dtype=ART_NUMPY_DTYPE) / nb_vars
         else:
@@ -594,22 +636,24 @@ class ZooAttack(EvasionAttack):
         for i in range(0, image.shape[1], kernel_size):
             for j in range(0, image.shape[2], kernel_size):
                 img_pool[:, i : i + kernel_size, j : j + kernel_size] = np.max(
-                    image[:, i : i + kernel_size, j : j + kernel_size], axis=(1, 2), keepdims=True,
+                    image[:, i : i + kernel_size, j : j + kernel_size],
+                    axis=(1, 2),
+                    keepdims=True,
                 )
 
         return img_pool
 
     def _check_params(self) -> None:
-        if not isinstance(self.binary_search_steps, (int, np.int)) or self.binary_search_steps < 0:
+        if not isinstance(self.binary_search_steps, int) or self.binary_search_steps < 0:
             raise ValueError("The number of binary search steps must be a non-negative integer.")
 
-        if not isinstance(self.max_iter, (int, np.int)) or self.max_iter < 0:
+        if not isinstance(self.max_iter, int) or self.max_iter < 0:
             raise ValueError("The number of iterations must be a non-negative integer.")
 
-        if not isinstance(self.nb_parallel, (int, np.int)) or self.nb_parallel < 1:
+        if not isinstance(self.nb_parallel, int) or self.nb_parallel < 1:
             raise ValueError("The number of parallel coordinates must be an integer greater than zero.")
 
-        if not isinstance(self.batch_size, (int, np.int)) or self.batch_size < 1:
+        if not isinstance(self.batch_size, int) or self.batch_size < 1:
             raise ValueError("The batch size must be an integer greater than zero.")
 
         if not isinstance(self.verbose, bool):
