@@ -18,6 +18,7 @@
 """
 This module implements a wrapper class for GPy Gaussian Process classification models.
 """
+# pylint: disable=C0103
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
@@ -52,7 +53,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance GPY Gaussian Process classification models.
@@ -68,7 +69,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
         """
         from GPy.models import GPClassification
 
-        if not isinstance(model, GPClassification):
+        if not isinstance(model, GPClassification):  # pragma: no cover
             raise TypeError("Model must be of type GPy.models.GPClassification")
 
         super().__init__(
@@ -78,7 +79,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
             postprocessing_defences=postprocessing_defences,
             preprocessing=preprocessing,
         )
-        self._nb_classes = 2  # always binary
+        self.nb_classes = 2  # always binary
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -91,7 +92,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
 
     # pylint: disable=W0221
     def class_gradient(  # type: ignore
-        self, x: np.ndarray, label: Union[int, List[int], None] = None, eps: float = 0.0001,
+        self, x: np.ndarray, label: Union[int, List[int], None] = None, eps: float = 0.0001, **kwargs
     ) -> np.ndarray:
         """
         Compute per-class derivatives w.r.t. `x`.
@@ -162,7 +163,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :param logits: `True` if the prediction should be done without squashing function.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
@@ -189,7 +190,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
         """
         Perform uncertainty prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :return: Array of uncertainty predictions of shape `(nb_inputs)`.
         """
         # Apply preprocessing
@@ -212,7 +213,7 @@ class GPyGaussianProcessClassifier(ClassifierClassLossGradients):
         """
         raise NotImplementedError
 
-    def save(self, filename: str, path: Optional[str] = None) -> None:
+    def save(self, filename: str, path: Optional[str] = None) -> None:  # pragma: no cover
         """
         Save a model to file in the format specific to the backend framework.
 
