@@ -36,7 +36,7 @@ class InputFilter(ABCMeta):
     Metaclass to ensure that inputs are ndarray for all of the subclass generate and extract calls.
     """
 
-    def __init__(cls, name, bases, clsdict):
+    def __init__(cls, name, bases, clsdict):  # pylint: disable=W0231,W0613
         """
         This function overrides any existing generate or extract methods with a new method that
         ensures the input is an ndarray. There is an assumption that the input object has implemented
@@ -52,17 +52,17 @@ class InputFilter(ABCMeta):
                 if len(args) > 0:
                     lst = list(args)
 
-                if "x" in kwargs:
+                if "x" in kwargs:  # pragma: no cover
                     if not isinstance(kwargs["x"], np.ndarray):
                         kwargs["x"] = np.array(kwargs["x"])
                 else:
                     if not isinstance(args[0], np.ndarray):
                         lst[0] = np.array(args[0])
 
-                if "y" in kwargs:
+                if "y" in kwargs:  # pragma: no cover
                     if kwargs["y"] is not None and not isinstance(kwargs["y"], np.ndarray):
                         kwargs["y"] = np.array(kwargs["y"])
-                elif has_y:
+                elif has_y:  # pragma: no cover
                     if not isinstance(args[1], np.ndarray):
                         lst[1] = np.array(args[1])
 
@@ -74,7 +74,7 @@ class InputFilter(ABCMeta):
             replacement_function.__name__ = "new_" + func_name
             return replacement_function
 
-        replacement_list_no_y = ["predict", "get_activations"]
+        replacement_list_no_y = ["predict"]
         replacement_list_has_y = ["fit"]
 
         for item in replacement_list_no_y:
@@ -92,8 +92,10 @@ class ClassifierMixin(ABC, metaclass=InputFilter):
     Mixin abstract base class defining functionality for classifiers.
     """
 
+    estimator_params = ["nb_classes"]
+
     def __init__(self, **kwargs) -> None:
-        super().__init__(**kwargs)
+        super().__init__(**kwargs)  # type: ignore
         self._nb_classes: int = -1
 
     @property
@@ -105,6 +107,16 @@ class ClassifierMixin(ABC, metaclass=InputFilter):
         """
         return self._nb_classes  # type: ignore
 
+    @nb_classes.setter
+    def nb_classes(self, nb_classes: int):
+        """
+        Set the number of output classes.
+        """
+        if nb_classes is None or nb_classes < 2:
+            raise ValueError("nb_classes must be greater than or equal to 2.")
+
+        self._nb_classes = nb_classes
+
 
 class ClassGradientsMixin(ABC):
     """
@@ -136,7 +148,7 @@ class Classifier(ClassifierMixin, BaseEstimator, ABC):
     Typing variable definition.
     """
 
-    pass
+    estimator_params = BaseEstimator.estimator_params + ClassifierMixin.estimator_params
 
 
 class ClassifierLossGradients(ClassifierMixin, LossGradientsMixin, BaseEstimator, ABC):
@@ -144,7 +156,7 @@ class ClassifierLossGradients(ClassifierMixin, LossGradientsMixin, BaseEstimator
     Typing variable definition.
     """
 
-    pass
+    estimator_params = BaseEstimator.estimator_params + ClassifierMixin.estimator_params
 
 
 class ClassifierClassLossGradients(ClassGradientsMixin, ClassifierMixin, LossGradientsMixin, BaseEstimator, ABC):
@@ -152,7 +164,7 @@ class ClassifierClassLossGradients(ClassGradientsMixin, ClassifierMixin, LossGra
     Typing variable definition.
     """
 
-    pass
+    estimator_params = BaseEstimator.estimator_params + ClassifierMixin.estimator_params
 
 
 class ClassifierNeuralNetwork(  # lgtm [py/conflicting-attributes]
@@ -162,6 +174,10 @@ class ClassifierNeuralNetwork(  # lgtm [py/conflicting-attributes]
     Typing variable definition.
     """
 
+    estimator_params = (
+        BaseEstimator.estimator_params + NeuralNetworkMixin.estimator_params + ClassifierMixin.estimator_params
+    )
+
     @abstractmethod
     def save(self, filename: str, path: Optional[str] = None) -> None:
         """
@@ -180,4 +196,4 @@ class ClassifierDecisionTree(DecisionTreeMixin, ClassifierMixin, BaseEstimator,
     Typing variable definition.
     """
 
-    pass
+    estimator_params = BaseEstimator.estimator_params + ClassifierMixin.estimator_params
