@@ -44,7 +44,7 @@ from art.estimators.classification.classifier import (
     ClassifierMixin,
     ClassGradientsMixin,
 )
-from art.utils import Deprecated, deprecated_keyword_arg, check_and_transform_label_format
+from art.utils import check_and_transform_label_format
 
 if TYPE_CHECKING:
     # pylint: disable=C0412
@@ -58,7 +58,7 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-KERAS_MODEL_TYPE = Union["keras.models.Model", "tf.keras.models.Model"]
+KERAS_MODEL_TYPE = Union["keras.models.Model", "tf.keras.models.Model"]  # pylint: disable=C0103
 
 
 class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
@@ -66,17 +66,21 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
     Wrapper class for importing Keras models.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
+    estimator_params = (
+        KerasEstimator.estimator_params
+        + ClassifierMixin.estimator_params
+        + ["use_logits", "input_layer", "output_layer"]
+    )
+
     def __init__(
         self,
         model: KERAS_MODEL_TYPE,
         use_logits: bool = False,
-        channel_index=Deprecated,
         channels_first: bool = False,
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
         input_layer: int = 0,
         output_layer: int = 0,
     ) -> None:
@@ -86,8 +90,6 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         :param model: Keras model, neural network or other.
         :param use_logits: True if the output of the model are logits; false for probabilities or any other type of
                outputs. Logits output should be favored when possible to ensure attack efficiency.
-        :param channel_index: Index of the axis in data containing the color channels or features.
-        :type channel_index: `int`
         :param channels_first: Set channels first or last.
         :param clip_values: Tuple of the form `(min, max)` of floats or `np.ndarray` representing the minimum and
                maximum values allowed for features. If floats are provided, these will be used as the range of all
@@ -105,21 +107,12 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                              with this index will be considered for computing gradients. For models with only one output
                              layer this values is not required.
         """
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
-
         super().__init__(
             model=model,
             clip_values=clip_values,
             preprocessing_defences=preprocessing_defences,
             postprocessing_defences=postprocessing_defences,
             preprocessing=preprocessing,
-            channel_index=channel_index,
             channels_first=channels_first,
         )
 
@@ -130,13 +123,17 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             self.is_tensorflow = True
         elif "<class 'keras" in str(type(model).__mro__):
             self.is_tensorflow = False
-        else:
+        else:  # pragma: no cover
             raise TypeError("Type of model not recognized:" + str(type(model)))
 
         self._initialize_params(model, use_logits, input_layer, output_layer)
 
     def _initialize_params(
-        self, model: KERAS_MODEL_TYPE, use_logits: bool, input_layer: int, output_layer: int,
+        self,
+        model: KERAS_MODEL_TYPE,
+        use_logits: bool,
+        input_layer: int,
+        output_layer: int,
     ):
         """
         Initialize most parameters of the classifier. This is a convenience function called by `__init__` and
@@ -151,7 +148,7 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         if self.is_tensorflow:
             import tensorflow as tf  # lgtm [py/repeated-import]
 
-            if tf.executing_eagerly():
+            if tf.executing_eagerly():  # pragma: no cover
                 raise ValueError("TensorFlow is executing eagerly. Please disable eager execution.")
             import tensorflow.keras as keras
             import tensorflow.keras.backend as k
@@ -180,10 +177,17 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             self._output = model.output
             self._output_layer = 0
 
-        _, self._nb_classes = k.int_shape(self._output)
+        _, nb_classes = k.int_shape(self._output)
+        # Check for binary classification
+        if nb_classes == 1:
+            nb_classes = 2
+        self.nb_classes = nb_classes
+
         self._input_shape = k.int_shape(self._input)[1:]
         logger.debug(
-            "Inferred %i classes and %s as input shape for Keras classifier.", self.nb_classes, str(self.input_shape),
+            "Inferred %i classes and %s as input shape for Keras classifier.",
+            self.nb_classes,
+            str(self.input_shape),
         )
 
         self._use_logits = use_logits
@@ -238,14 +242,19 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                     keras.losses.KLDivergence,
                 ),
             )
-        except AttributeError:
+        except AttributeError:  # pragma: no cover
             flag_is_instance = False
 
         # Check if the labels have to be reduced to index labels and create placeholder for labels
         if (
             "__name__" in dir(loss_function)
             and loss_function.__name__
-            in ["categorical_hinge", "categorical_crossentropy", "binary_crossentropy", "kullback_leibler_divergence",]
+            in [
+                "categorical_hinge",
+                "categorical_crossentropy",
+                "binary_crossentropy",
+                "kullback_leibler_divergence",
+            ]
         ) or flag_is_instance:
             self._reduce_labels = False
             label_ph = k.placeholder(shape=self._output.shape)
@@ -253,8 +262,12 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             "__name__" in dir(loss_function) and loss_function.__name__ in ["sparse_categorical_crossentropy"]
         ) or isinstance(loss_function, keras.losses.SparseCategoricalCrossentropy):
             self._reduce_labels = True
-            label_ph = k.placeholder(shape=[None,])
-        else:
+            label_ph = k.placeholder(
+                shape=[
+                    None,
+                ]
+            )
+        else:  # pragma: no cover
             raise ValueError("Loss function not recognised.")
 
         # Define the loss using the loss function
@@ -288,14 +301,14 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         if k.backend() == "tensorflow":
             loss_gradients = loss_gradients[0]
-        elif k.backend() == "cntk":
+        elif k.backend() == "cntk":  # pragma: no cover
             raise NotImplementedError("Only TensorFlow is supported as backend for Keras.")
 
         # Set loss, gradients and prediction functions
         self._predictions_op = self._output
         self._loss_function = loss_function
         self._loss = loss_
-        self._loss_gradients = k.function([self._input, label_ph], [loss_gradients])
+        self._loss_gradients = k.function([self._input, label_ph, k.learning_phase()], [loss_gradients])
 
         # Get the internal layer
         self._layer_names = self._get_layers()
@@ -309,7 +322,38 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         """
         return self._input_shape  # type: ignore
 
-    def loss(self, x: np.ndarray, y: np.ndarray, reduction: str = "none", **kwargs) -> np.ndarray:
+    @property
+    def use_logits(self) -> bool:
+        """
+        A boolean representing whether the outputs of the model are logits.
+
+        :return: a boolean representing whether the outputs of the model are logits.
+        """
+        return self._use_logits  # type: ignore
+
+    @property
+    def input_layer(self) -> int:
+        """
+        The index of the layer considered as input for models with multiple input layers.
+        For models with only one input layer the index is 0.
+
+        :return: The index of the layer considered as input for models with multiple input layers.
+        """
+        return self._input_layer  # type: ignore
+
+    @property
+    def output_layer(self) -> int:
+        """
+        The index of the layer considered as output for models with multiple output layers.
+        For models with only one output layer the index is 0.
+
+        :return: The index of the layer considered as output for models with multiple output layers.
+        """
+        return self._output_layer  # type: ignore
+
+    def compute_loss(  # pylint: disable=W0221
+        self, x: np.ndarray, y: np.ndarray, reduction: str = "none", **kwargs
+    ) -> np.ndarray:
         """
         Compute the loss of the neural network for samples `x`.
 
@@ -334,7 +378,7 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
         shape_match = [i is None or i == j for i, j in zip(self._input_shape, x_preprocessed.shape[1:])]
-        if not all(shape_match):
+        if not all(shape_match):  # pragma: no cover
             raise ValueError(
                 "Error when checking x: expected preprocessed x to have shape {} but got array with "
                 "shape {}.".format(self._input_shape, x_preprocessed.shape[1:])
@@ -348,7 +392,8 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         if self._orig_loss and hasattr(self._orig_loss, "reduction"):
             prev_reduction = self._orig_loss.reduction
-            self._orig_loss.reduction = self._losses.Reduction.NONE
+            if hasattr(self._losses, "Reduction"):
+                self._orig_loss.reduction = self._losses.Reduction.NONE
             loss = self._orig_loss(y_preprocessed, predictions)
             self._orig_loss.reduction = prev_reduction
         else:
@@ -357,7 +402,8 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             y_preprocessed = k.constant(y_preprocessed)
             for loss_function in self._model.loss_functions:
                 prev_reduction.append(loss_function.reduction)
-                loss_function.reduction = self._losses.Reduction.NONE
+                if hasattr(self._losses, "Reduction"):
+                    loss_function.reduction = self._losses.Reduction.NONE
             loss = self._loss_function(y_preprocessed, predictions)
             for i, loss_function in enumerate(self._model.loss_functions):
                 loss_function.reduction = prev_reduction[i]
@@ -373,19 +419,22 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         return loss_value
 
-    def loss_gradient(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+    def loss_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, y: np.ndarray, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
         :param x: Sample input with shape as expected by the model.
         :param y: Target values (class labels) one-hot-encoded of shape (nb_samples, nb_classes) or indices of shape
                   (nb_samples,).
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of gradients of the same shape as `x`.
         """
         # Check shape of preprocessed `x` because of custom function for `_loss_gradients`
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
         shape_match = [i is None or i == j for i, j in zip(self._input_shape, x_preprocessed.shape[1:])]
-        if not all(shape_match):
+        if not all(shape_match):  # pragma: no cover
             raise ValueError(
                 "Error when checking x: expected preprocessed x to have shape {} but got array with shape {}".format(
                     self._input_shape, x_preprocessed.shape[1:]
@@ -397,14 +446,16 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             y_preprocessed = np.argmax(y_preprocessed, axis=1)
 
         # Compute gradients
-        gradients = self._loss_gradients([x_preprocessed, y_preprocessed])[0]
+        gradients = self._loss_gradients([x_preprocessed, y_preprocessed, int(training_mode)])[0]
         assert gradients.shape == x_preprocessed.shape
         gradients = self._apply_preprocessing_gradient(x, gradients)
         assert gradients.shape == x.shape
 
         return gradients
 
-    def class_gradient(self, x: np.ndarray, label: Optional[Union[int, List[int]]] = None, **kwargs) -> np.ndarray:
+    def class_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, label: Optional[Union[int, List[int]]] = None, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute per-class derivatives w.r.t. `x`.
 
@@ -413,6 +464,7 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                       output is computed for all samples. If multiple values are provided, the first dimension should
                       match the batch size of `x`, and each value will be used as target for its corresponding sample in
                       `x`. If `None`, then gradients for all classes will be computed for each sample.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of gradients of input features w.r.t. each class in the form
                  `(batch_size, nb_classes, input_shape)` when computing for all classes, otherwise shape becomes
                  `(batch_size, 1, input_shape)` when `label` parameter is specified.
@@ -428,12 +480,12 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                 and label.shape[0] == x.shape[0]
             )
         ):
-            raise ValueError("Label %s is out of range." % str(label))
+            raise ValueError("Label %s is out of range." % str(label))  # pragma: no cover
 
         # Check shape of preprocessed `x` because of custom function for `_class_gradients`
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
         shape_match = [i is None or i == j for i, j in zip(self._input_shape, x_preprocessed.shape[1:])]
-        if not all(shape_match):
+        if not all(shape_match):  # pragma: no cover
             raise ValueError(
                 "Error when checking x: expected preprocessed x to have shape {} but got array with shape {}".format(
                     self._input_shape, x_preprocessed.shape[1:]
@@ -448,13 +500,24 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         elif isinstance(label, (int, np.integer)):
             # Compute the gradients only w.r.t. the provided label
-            gradients = np.swapaxes(np.array(self._class_gradients_idx[label]([x_preprocessed])), 0, 1)  # type: ignore
+            grad_fn = self._class_gradients_idx[label]
+            if grad_fn is not None:
+                gradients = np.swapaxes(np.array(grad_fn([x_preprocessed, int(training_mode)])), axis1=0, axis2=1)
+            else:  # pragma: no cover
+                raise ValueError("Class gradient operation is not defined.")
             assert gradients.shape == (x_preprocessed.shape[0], 1) + x_preprocessed.shape[1:]
 
         else:
             # For each sample, compute the gradients w.r.t. the indicated target class (possibly distinct)
             unique_label = list(np.unique(label))
-            gradients = np.array([self._class_gradients_idx[l]([x_preprocessed]) for l in unique_label])
+            gradients_list = list()
+            for u_l in unique_label:
+                grad_fn = self._class_gradients_idx[u_l]
+                if grad_fn is not None:
+                    gradients_list.append(grad_fn([x_preprocessed, int(training_mode)]))
+                else:  # pragma: no cover
+                    raise ValueError("Class gradient operation is not defined.")
+            gradients = np.array(gradients_list)
             gradients = np.swapaxes(np.squeeze(gradients, axis=1), 0, 1)
             lst = [unique_label.index(i) for i in label]
             gradients = np.expand_dims(gradients[np.arange(len(gradients)), lst], axis=1)
@@ -463,27 +526,25 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         return gradients
 
-    def predict(self, x: np.ndarray, batch_size: int = 128, **kwargs) -> np.ndarray:
+    def predict(  # pylint: disable=W0221
+        self, x: np.ndarray, batch_size: int = 128, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :param batch_size: Size of batches.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
-        from art.config import ART_NUMPY_DTYPE
-
         # Apply preprocessing
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
 
         # Run predictions with batching
-        predictions = np.zeros((x_preprocessed.shape[0], self.nb_classes), dtype=ART_NUMPY_DTYPE)
-        for batch_index in range(int(np.ceil(x_preprocessed.shape[0] / float(batch_size)))):
-            begin, end = (
-                batch_index * batch_size,
-                min((batch_index + 1) * batch_size, x_preprocessed.shape[0]),
-            )
-            predictions[begin:end] = self._model.predict([x_preprocessed[begin:end]])
+        if training_mode:
+            predictions = self._model(x_preprocessed, training=training_mode)
+        else:
+            predictions = self._model.predict(x_preprocessed, batch_size=batch_size)
 
         # Apply postprocessing
         predictions = self._apply_postprocessing(preds=predictions, fit=False)
@@ -503,18 +564,17 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                `fit_generator` function in Keras and will be passed to this function as such. Including the number of
                epochs or the number of steps per epoch as part of this argument will result in as error.
         """
+        y_ndim = y.ndim
         y = check_and_transform_label_format(y, self.nb_classes)
 
         # Apply preprocessing
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=True)
 
         # Adjust the shape of y for loss functions that do not take labels in one-hot encoding
-        if self._reduce_labels:
+        if self._reduce_labels or y_ndim == 1:
             y_preprocessed = np.argmax(y_preprocessed, axis=1)
 
-        gen = generator_fit(x_preprocessed, y_preprocessed, batch_size)
-        steps_per_epoch = max(int(x_preprocessed.shape[0] / batch_size), 1)
-        self._model.fit_generator(gen, steps_per_epoch=steps_per_epoch, epochs=nb_epochs, **kwargs)
+        self._model.fit(x=x_preprocessed, y=y_preprocessed, batch_size=batch_size, epochs=nb_epochs, **kwargs)
 
     def fit_generator(self, generator: "DataGenerator", nb_epochs: int = 20, **kwargs) -> None:
         """
@@ -530,21 +590,33 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         from art.data_generators import KerasDataGenerator
 
         # Try to use the generator as a Keras native generator, otherwise use it through the `DataGenerator` interface
-        if isinstance(generator, KerasDataGenerator) and not self.preprocessing:
+        from art.preprocessing.standardisation_mean_std.numpy import StandardisationMeanStd
+
+        if isinstance(generator, KerasDataGenerator) and (
+            self.preprocessing is None
+            or (
+                isinstance(self.preprocessing, StandardisationMeanStd)
+                and (
+                    self.preprocessing.mean,
+                    self.preprocessing.std,
+                )
+                == (0, 1)
+            )
+        ):
             try:
                 self._model.fit_generator(generator.iterator, epochs=nb_epochs, **kwargs)
-            except ValueError:
+            except ValueError:  # pragma: no cover
                 logger.info("Unable to use data generator as Keras generator. Now treating as framework-independent.")
                 if "verbose" not in kwargs.keys():
                     kwargs["verbose"] = 0
                 super().fit_generator(generator, nb_epochs=nb_epochs, **kwargs)
-        else:
+        else:  # pragma: no cover
             if "verbose" not in kwargs.keys():
                 kwargs["verbose"] = 0
             super().fit_generator(generator, nb_epochs=nb_epochs, **kwargs)
 
     def get_activations(
-        self, x: np.ndarray, layer: Union[int, str], batch_size: int, framework: bool = False
+        self, x: np.ndarray, layer: Union[int, str], batch_size: int = 128, framework: bool = False
     ) -> np.ndarray:
         """
         Return the output of the specified layer for input `x`. `layer` is specified by layer index (between 0 and
@@ -565,16 +637,16 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         from art.config import ART_NUMPY_DTYPE
 
         if isinstance(layer, six.string_types):
-            if layer not in self._layer_names:
+            if layer not in self._layer_names:  # pragma: no cover
                 raise ValueError("Layer name %s is not part of the graph." % layer)
             layer_name = layer
         elif isinstance(layer, int):
-            if layer < 0 or layer >= len(self._layer_names):
+            if layer < 0 or layer >= len(self._layer_names):  # pragma: no cover
                 raise ValueError(
                     "Layer index %d is outside of range (0 to %d included)." % (layer, len(self._layer_names) - 1)
                 )
             layer_name = self._layer_names[layer]
-        else:
+        else:  # pragma: no cover
             raise TypeError("Layer must be of type `str` or `int`.")
 
         if x.shape == self.input_shape:
@@ -595,10 +667,10 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                 layer_output = keras_layer.get_output_at(0)
             else:
                 layer_output = keras_layer.output
-            self._activations_func[layer_name] = k.function([self._input], [layer_output])
+            self._activations_func[layer_name] = k.function([self._input, k.learning_phase()], [layer_output])
 
         # Determine shape of expected output and prepare array
-        output_shape = self._activations_func[layer_name]([x_preprocessed[0][None, ...]])[0].shape
+        output_shape = self._activations_func[layer_name]([x_preprocessed[0][None, ...], int(False)])[0].shape
         activations = np.zeros((x_preprocessed.shape[0],) + output_shape[1:], dtype=ART_NUMPY_DTYPE)
 
         # Get activations with batching
@@ -607,13 +679,13 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
                 batch_index * batch_size,
                 min((batch_index + 1) * batch_size, x_preprocessed.shape[0]),
             )
-            activations[begin:end] = self._activations_func[layer_name]([x_preprocessed[begin:end]])[0]
+            activations[begin:end] = self._activations_func[layer_name]([x_preprocessed[begin:end], 0])[0]
 
         if framework:
             placeholder = k.placeholder(shape=x.shape)
             return placeholder, keras_layer(placeholder)
-        else:
-            return activations
+
+        return activations
 
     def custom_loss_gradient(self, nn_function, tensors, input_values, name="default"):
         """
@@ -630,7 +702,10 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         :return: the gradient of the function w.r.t vars
         :rtype: `np.ndarray`
         """
-        import keras.backend as k
+        if self.is_tensorflow:
+            import tensorflow.keras.backend as k
+        else:
+            import keras.backend as k
 
         if not hasattr(self, "_custom_loss_func"):
             self._custom_loss_func = {}
@@ -651,7 +726,7 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         if len(self._output.shape) == 2:
             nb_outputs = self._output.shape[1]
-        else:
+        else:  # pragma: no cover
             raise ValueError("Unexpected output shape for classification in Keras model.")
 
         if label is None:
@@ -673,7 +748,9 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
             for current_label in unique_labels:
                 if self._class_gradients_idx[current_label] is None:
                     class_gradients = [k.gradients(self._predictions_op[:, current_label], self._input)[0]]
-                    self._class_gradients_idx[current_label] = k.function([self._input], class_gradients)
+                    self._class_gradients_idx[current_label] = k.function(
+                        [self._input, k.learning_phase()], class_gradients
+                    )
 
     def _get_layers(self) -> List[str]:
         """
@@ -692,22 +769,6 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
         return layer_names
 
-    def set_learning_phase(self, train: bool) -> None:
-        """
-        Set the learning phase for the backend framework.
-
-        :param train: True to set the learning phase to training, False to set it to prediction.
-        """
-        # pylint: disable=E0401
-        if self.is_tensorflow:
-            import tensorflow.keras.backend as k
-        else:
-            import keras.backend as k
-
-        if isinstance(train, bool):
-            self._learning_phase = train
-            k.set_learning_phase(int(train))
-
     def save(self, filename: str, path: Optional[str] = None) -> None:
         """
         Save a model to file in the format specific to the backend framework. For Keras, .h5 format is used.
@@ -787,13 +848,12 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
 
     def __repr__(self):
         repr_ = (
-            "%s(model=%r, use_logits=%r, channel_index=%r, channels_first=%r, clip_values=%r, preprocessing_defences=%r"
+            "%s(model=%r, use_logits=%r, channels_first=%r, clip_values=%r, preprocessing_defences=%r"
             ", postprocessing_defences=%r, preprocessing=%r, input_layer=%r, output_layer=%r)"
             % (
                 self.__module__ + "." + self.__class__.__name__,
                 self._model,
                 self._use_logits,
-                self.channel_index,
                 self.channels_first,
                 self.clip_values,
                 self.preprocessing_defences,
@@ -807,7 +867,9 @@ class KerasClassifier(ClassGradientsMixin, ClassifierMixin, KerasEstimator):
         return repr_
 
 
-def generator_fit(x: np.ndarray, y: np.ndarray, batch_size: int = 128) -> Iterator[Tuple[np.ndarray, np.ndarray]]:
+def generator_fit(
+    x: np.ndarray, y: np.ndarray, batch_size: int = 128
+) -> Iterator[Tuple[np.ndarray, np.ndarray]]:  # pragma: no cover
     """
     Minimal data generator for randomly batching large datasets.
 
