@@ -45,7 +45,7 @@ logger = logging.getLogger(__name__)
 
 class LightGBMClassifier(ClassifierDecisionTree):
     """
-    Wrapper class for importing LightGBM models.
+    Class for importing LightGBM models.
     """
 
     def __init__(
@@ -54,7 +54,7 @@ class LightGBMClassifier(ClassifierDecisionTree):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a LightGBM model.
@@ -70,7 +70,7 @@ class LightGBMClassifier(ClassifierDecisionTree):
         """
         from lightgbm import Booster  # type: ignore
 
-        if not isinstance(model, Booster):
+        if not isinstance(model, Booster):  # pragma: no cover
             raise TypeError("Model must be of type lightgbm.Booster")
 
         super().__init__(
@@ -82,7 +82,7 @@ class LightGBMClassifier(ClassifierDecisionTree):
         )
 
         self._input_shape = (self._model.num_feature(),)
-        self._nb_classes = self._get_nb_classes()
+        self.nb_classes = self._get_nb_classes()
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -109,7 +109,7 @@ class LightGBMClassifier(ClassifierDecisionTree):
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
         # Apply preprocessing
@@ -132,7 +132,7 @@ class LightGBMClassifier(ClassifierDecisionTree):
         # pylint: disable=W0212
         return self._model._Booster__num_class
 
-    def save(self, filename: str, path: Optional[str] = None) -> None:
+    def save(self, filename: str, path: Optional[str] = None) -> None:  # pragma: no cover
         """
         Save a model to file in the format specific to the backend framework.
 
