@@ -30,7 +30,7 @@ import six
 from art import config
 from art.estimators.mxnet import MXEstimator
 from art.estimators.classification.classifier import ClassGradientsMixin, ClassifierMixin
-from art.utils import Deprecated, deprecated_keyword_arg, check_and_transform_label_format
+from art.utils import check_and_transform_label_format
 
 if TYPE_CHECKING:
     # pylint: disable=C0412
@@ -46,10 +46,22 @@ logger = logging.getLogger(__name__)
 
 class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [py/missing-call-to-init]
     """
-    Wrapper class for importing MXNet Gluon models.
+    Class for importing MXNet Gluon models.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
+    estimator_params = (
+        MXEstimator.estimator_params
+        + ClassifierMixin.estimator_params
+        + [
+            "loss",
+            "input_shape",
+            "nb_classes",
+            "optimizer",
+            "ctx",
+            "channels_first",
+        ]
+    )
+
     def __init__(
         self,
         model: "mx.gluon.Block",
@@ -58,12 +70,11 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         nb_classes: int,
         optimizer: Optional["mx.gluon.Trainer"] = None,
         ctx: Optional["mx.context.Context"] = None,
-        channel_index=Deprecated,
         channels_first: bool = True,
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Initialize an `MXClassifier` object. Assumes the `model` passed as parameter is a Gluon model.
@@ -76,8 +87,6 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         :param optimizer: The optimizer used to train the classifier. This parameter is only required if fitting will
                           be done with method fit.
         :param ctx: The device on which the model runs (CPU or GPU). If not provided, CPU is assumed.
-        :param channel_index: Index of the axis in data containing the color channels or features.
-        :type channel_index: `int`
         :param channels_first: Set channels first or last.
         :param clip_values: Tuple of the form `(min, max)` of floats or `np.ndarray` representing the minimum and
                maximum values allowed for features. If floats are provided, these will be used as the range of all
@@ -91,18 +100,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         """
         import mxnet as mx  # lgtm [py/repeated-import]
 
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
-
         super().__init__(
             model=model,
             clip_values=clip_values,
-            channel_index=channel_index,
             channels_first=channels_first,
             preprocessing_defences=preprocessing_defences,
             postprocessing_defences=postprocessing_defences,
@@ -110,7 +110,7 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         )
 
         self._loss = loss
-        self._nb_classes = nb_classes
+        self.nb_classes = nb_classes
         self._input_shape = input_shape
         self._device = ctx
         self._optimizer = optimizer
@@ -132,7 +132,36 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         """
         return self._input_shape  # type: ignore
 
-    def fit(self, x: np.ndarray, y: np.ndarray, batch_size: int = 128, nb_epochs: int = 20, **kwargs) -> None:
+    @property
+    def loss(self) -> Union["mx.nd.loss", "mx.gluon.loss"]:
+        """
+        Return the loss function.
+
+        :return: The loss function.
+        """
+        return self._loss  # type: ignore
+
+    @property
+    def optimizer(self) -> "mx.gluon.Trainer":
+        """
+        Return the optimizer used to train the classifier.
+
+        :return: The optimizer used to train the classifier.
+        """
+        return self._optimizer  # type: ignore
+
+    @property
+    def ctx(self) -> "mx.context.Context":
+        """
+        Return the device on which the model runs.
+
+        :return: The device on which the model runs (CPU or GPU).
+        """
+        return self._ctx  # type: ignore
+
+    def fit(
+        self, x: np.ndarray, y: np.ndarray, batch_size: int = 128, nb_epochs: int = 20, **kwargs
+    ) -> None:  # pragma: no cover
         """
         Fit the classifier on the training set `(inputs, outputs)`.
 
@@ -146,9 +175,10 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         """
         import mxnet as mx  # lgtm [py/repeated-import]
 
-        if self._optimizer is None:
+        if self.optimizer is None:
             raise ValueError("An MXNet optimizer is required for fitting the model.")
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else True
+
+        training_mode = True
 
         y = check_and_transform_label_format(y, self.nb_classes)
 
@@ -166,12 +196,12 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
             for m in range(nb_batch):
                 x_batch = mx.nd.array(
                     x_preprocessed[ind[m * batch_size : (m + 1) * batch_size]].astype(config.ART_NUMPY_DTYPE)
-                ).as_in_context(self._ctx)
+                ).as_in_context(self.ctx)
                 y_batch = mx.nd.array(y_preprocessed[ind[m * batch_size : (m + 1) * batch_size]]).as_in_context(
-                    self._ctx
+                    self.ctx
                 )
 
-                with mx.autograd.record(train_mode=train_mode):
+                with mx.autograd.record(train_mode=training_mode):
                     # Perform prediction
                     preds = self._model(x_batch)
 
@@ -179,14 +209,14 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
                     preds = self._apply_postprocessing(preds=preds, fit=True)
 
                     # Form the loss function
-                    loss = self._loss(preds, y_batch)
+                    loss = self.loss(preds, y_batch)
 
                 loss.backward()
 
                 # Update parameters
-                self._optimizer.step(batch_size)
+                self.optimizer.step(batch_size)
 
-    def fit_generator(self, generator: "DataGenerator", nb_epochs: int = 20, **kwargs) -> None:
+    def fit_generator(self, generator: "DataGenerator", nb_epochs: int = 20, **kwargs) -> None:  # pragma: no cover
         """
         Fit the classifier using the generator that yields batches as specified.
 
@@ -198,9 +228,10 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         import mxnet as mx  # lgtm [py/repeated-import]
         from art.data_generators import MXDataGenerator
 
-        if self._optimizer is None:
+        if self.optimizer is None:
             raise ValueError("An MXNet optimizer is required for fitting the model.")
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else True
+
+        training_mode = True
 
         if (
             isinstance(generator, MXDataGenerator)
@@ -210,37 +241,38 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
             # Train directly in MXNet
             for _ in range(nb_epochs):
                 for x_batch, y_batch in generator.iterator:
-                    x_batch = mx.nd.array(x_batch.astype(config.ART_NUMPY_DTYPE)).as_in_context(self._ctx)
+                    x_batch = mx.nd.array(x_batch.astype(config.ART_NUMPY_DTYPE)).as_in_context(self.ctx)
                     y_batch = mx.nd.argmax(y_batch, axis=1)
-                    y_batch = mx.nd.array(y_batch).as_in_context(self._ctx)
+                    y_batch = mx.nd.array(y_batch).as_in_context(self.ctx)
 
-                    with mx.autograd.record(train_mode=train_mode):
+                    with mx.autograd.record(train_mode=training_mode):
                         # Perform prediction
                         preds = self._model(x_batch)
 
                         # Form the loss function
-                        loss = self._loss(preds, y_batch)
+                        loss = self.loss(preds, y_batch)
 
                     loss.backward()
 
                     # Update parameters
-                    self._optimizer.step(x_batch.shape[0])
+                    self.optimizer.step(x_batch.shape[0])
         else:
             # Fit a generic data generator through the API
             super().fit_generator(generator, nb_epochs=nb_epochs)
 
-    def predict(self, x: np.ndarray, batch_size: int = 128, **kwargs) -> np.ndarray:
+    def predict(  # pylint: disable=W0221
+        self, x: np.ndarray, batch_size: int = 128, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :param batch_size: Size of batches.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
         import mxnet as mx  # lgtm [py/repeated-import]
 
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else False
-
         # Apply preprocessing
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
 
@@ -255,9 +287,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
             )
 
             # Predict
-            x_batch = mx.nd.array(x_preprocessed[begin:end].astype(config.ART_NUMPY_DTYPE), ctx=self._ctx)
+            x_batch = mx.nd.array(x_preprocessed[begin:end].astype(config.ART_NUMPY_DTYPE), ctx=self.ctx)
             x_batch.attach_grad()
-            with mx.autograd.record(train_mode=train_mode):
+            with mx.autograd.record(train_mode=training_mode):
                 preds = self._model(x_batch)
 
             results[begin:end] = preds.asnumpy()
@@ -267,7 +299,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
 
         return predictions
 
-    def class_gradient(self, x: np.ndarray, label: Union[int, List[int], None] = None, **kwargs) -> np.ndarray:
+    def class_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, label: Union[int, List[int], None] = None, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute per-class derivatives w.r.t. `x`.
 
@@ -276,6 +310,7 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
                       output is computed for all samples. If multiple values as provided, the first dimension should
                       match the batch size of `x`, and each value will be used as target for its corresponding sample in
                       `x`. If `None`, then gradients for all classes will be computed for each sample.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of gradients of input features w.r.t. each class in the form
                  `(batch_size, nb_classes, input_shape)` when computing for all classes, otherwise shape becomes
                  `(batch_size, 1, input_shape)` when `label` parameter is specified.
@@ -283,9 +318,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         import mxnet as mx  # lgtm [py/repeated-import]
 
         # Check value of label for computing gradients
-        if not (
+        if not (  # pragma: no cover
             label is None
-            or (isinstance(label, (int, np.integer)) and label in range(self.nb_classes))
+            or (isinstance(label, int) and label in range(self.nb_classes))
             or (
                 isinstance(label, np.ndarray)
                 and len(label.shape) == 1
@@ -295,11 +330,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         ):
             raise ValueError("Label %s is out of range." % str(label))
 
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else False
-
         # Apply preprocessing
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
-        x_preprocessed = mx.nd.array(x_preprocessed.astype(config.ART_NUMPY_DTYPE), ctx=self._ctx)
+        x_preprocessed = mx.nd.array(x_preprocessed.astype(config.ART_NUMPY_DTYPE), ctx=self.ctx)
         x_preprocessed.attach_grad()
 
         if label is None:
@@ -307,14 +340,14 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
                 preds = self._model(x_preprocessed)
                 class_slices = [preds[:, i] for i in range(self.nb_classes)]
 
-            grads = []
+            grads_list = list()
             for slice_ in class_slices:
                 slice_.backward(retain_graph=True)
                 grad = x_preprocessed.grad.asnumpy()
-                grads.append(grad)
-            grads = np.swapaxes(np.array(grads), 0, 1)
-        elif isinstance(label, (int, np.integer)):
-            with mx.autograd.record(train_mode=train_mode):
+                grads_list.append(grad)
+            grads = np.swapaxes(np.array(grads_list), 0, 1)
+        elif isinstance(label, int):
+            with mx.autograd.record(train_mode=training_mode):
                 preds = self._model(x_preprocessed)
                 class_slice = preds[:, label]
 
@@ -323,17 +356,17 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         else:
             unique_labels = list(np.unique(label))
 
-            with mx.autograd.record(train_mode=train_mode):
+            with mx.autograd.record(train_mode=training_mode):
                 preds = self._model(x_preprocessed)
                 class_slices = [preds[:, i] for i in unique_labels]
 
-            grads = []
+            grads_list = list()
             for slice_ in class_slices:
                 slice_.backward(retain_graph=True)
                 grad = x_preprocessed.grad.asnumpy()
-                grads.append(grad)
+                grads_list.append(grad)
 
-            grads = np.swapaxes(np.array(grads), 0, 1)
+            grads = np.swapaxes(np.array(grads_list), 0, 1)
             lst = [unique_labels.index(i) for i in label]
             grads = grads[np.arange(len(grads)), lst]
             grads = np.expand_dims(grads, axis=1)
@@ -342,28 +375,29 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
 
         return grads
 
-    def loss_gradient(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+    def loss_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, y: np.ndarray, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
         :param x: Sample input with shape as expected by the model.
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices of shape
                   `(nb_samples,)`.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of gradients of the same shape as `x`.
         """
         import mxnet as mx  # lgtm [py/repeated-import]
 
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else False
-
         # Apply preprocessing
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
-        y_preprocessed = mx.nd.array([np.argmax(y_preprocessed, axis=1)], ctx=self._ctx).T
-        x_preprocessed = mx.nd.array(x_preprocessed.astype(config.ART_NUMPY_DTYPE), ctx=self._ctx)
+        y_preprocessed = mx.nd.array([np.argmax(y_preprocessed, axis=1)], ctx=self.ctx).T
+        x_preprocessed = mx.nd.array(x_preprocessed.astype(config.ART_NUMPY_DTYPE), ctx=self.ctx)
         x_preprocessed.attach_grad()
 
-        with mx.autograd.record(train_mode=train_mode):
+        with mx.autograd.record(train_mode=training_mode):
             preds = self._model(x_preprocessed)
-            loss = self._loss(preds, y_preprocessed)
+            loss = self.loss(preds, y_preprocessed)
 
         loss.backward()
 
@@ -374,7 +408,7 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
 
         return grads
 
-    def loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+    def compute_loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
         """
         Compute the loss of the neural network for samples `x`.
 
@@ -403,7 +437,7 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
 
     def get_activations(
         self, x: np.ndarray, layer: Union[int, str], batch_size: int = 128, framework: bool = False
-    ) -> np.ndarray:
+    ) -> np.ndarray:  # pragma: no cover
         """
         Return the output of the specified layer for input `x`. `layer` is specified by layer index (between 0 and
         `nb_layers - 1`) or by name. The number of layers can be determined by counting the results returned by
@@ -417,8 +451,6 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         """
         import mxnet as mx  # lgtm [py/repeated-import]
 
-        train_mode = self._learning_phase if hasattr(self, "_learning_phase") else False
-
         if isinstance(layer, six.string_types):
             if layer not in self._layer_names:
                 raise ValueError("Layer name %s is not part of the model." % layer)
@@ -454,9 +486,9 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
             )
 
             # Predict
-            x_batch = mx.nd.array(x_preprocessed[begin:end].astype(config.ART_NUMPY_DTYPE), ctx=self._ctx)
+            x_batch = mx.nd.array(x_preprocessed[begin:end].astype(config.ART_NUMPY_DTYPE), ctx=self.ctx)
             x_batch.attach_grad()
-            with mx.autograd.record(train_mode=train_mode):
+            with mx.autograd.record(train_mode=False):
                 preds = self._model[layer_ind](x_batch)
 
             activations.append(preds.asnumpy())
@@ -464,15 +496,6 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
         activations = np.vstack(activations)
         return activations
 
-    def set_learning_phase(self, train: bool) -> None:
-        """
-        Set the learning phase for the backend framework.
-
-        :param train: True to set the learning phase to training, False to set it to prediction.
-        """
-        if isinstance(train, bool):
-            self._learning_phase = train
-
     def save(self, filename: str, path: Optional[str] = None) -> None:
         """
         Save a model to file in the format specific to the backend framework. For Gluon, only parameters are saved in
@@ -496,18 +519,17 @@ class MXClassifier(ClassGradientsMixin, ClassifierMixin, MXEstimator):  # lgtm [
 
     def __repr__(self):
         repr_ = (
-            "%s(model=%r, loss=%r, input_shape=%r, nb_classes=%r, optimizer=%r, ctx=%r, channel_index=%r,"
+            "%s(model=%r, loss=%r, input_shape=%r, nb_classes=%r, optimizer=%r, ctx=%r, "
             " channels_first=%r, clip_values=%r, preprocessing=%r, postprocessing_defences=%r,"
             " preprocessing=%r)"
             % (
                 self.__module__ + "." + self.__class__.__name__,
                 self._model,
-                self._loss,
+                self.loss,
                 self.input_shape,
                 self.nb_classes,
-                self._optimizer,
-                self._ctx,
-                self.channel_index,
+                self.optimizer,
+                self.ctx,
                 self.channels_first,
                 self.clip_values,
                 self.preprocessing,
