@@ -18,6 +18,7 @@
 """
 This module implements the classifier `PyTorchClassifier` for PyTorch models.
 """
+# pylint: disable=C0302
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import copy
@@ -36,10 +37,10 @@ from art.estimators.classification.classifier import (
     ClassifierMixin,
 )
 from art.estimators.pytorch import PyTorchEstimator
-from art.utils import Deprecated, deprecated_keyword_arg, check_and_transform_label_format
+from art.utils import check_and_transform_label_format
 
 if TYPE_CHECKING:
-    # pylint: disable=C0412
+    # pylint: disable=C0412, C0302
     import torch
 
     from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
@@ -55,7 +56,19 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
     This class implements a classifier with the PyTorch framework.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
+    estimator_params = (
+        PyTorchEstimator.estimator_params
+        + ClassifierMixin.estimator_params
+        + [
+            "loss",
+            "input_shape",
+            "optimizer",
+            "use_amp",
+            "opt_level",
+            "loss_scale",
+        ]
+    )
+
     def __init__(
         self,
         model: "torch.nn.Module",
@@ -66,12 +79,11 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         use_amp: bool = False,
         opt_level: str = "O1",
         loss_scale: Optional[Union[float, str]] = "dynamic",
-        channel_index=Deprecated,
         channels_first: bool = True,
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
         device_type: str = "gpu",
     ) -> None:
         """
@@ -92,8 +104,6 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                            representing a number, e.g., “1.0”, or the string “dynamic”.
         :param nb_classes: The number of classes of the model.
         :param optimizer: The optimizer used to train the classifier.
-        :param channel_index: Index of the axis in data containing the color channels or features.
-        :type channel_index: `int`
         :param channels_first: Set channels first or last.
         :param clip_values: Tuple of the form `(min, max)` of floats or `np.ndarray` representing the minimum and
                maximum values allowed for features. If floats are provided, these will be used as the range of all
@@ -108,44 +118,47 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
-
         super().__init__(
             model=model,
             clip_values=clip_values,
-            channel_index=channel_index,
             channels_first=channels_first,
             preprocessing_defences=preprocessing_defences,
             postprocessing_defences=postprocessing_defences,
             preprocessing=preprocessing,
             device_type=device_type,
         )
-        self._nb_classes = nb_classes
+        self.nb_classes = nb_classes
         self._input_shape = input_shape
         self._model = self._make_model_wrapper(model)
         self._loss = loss
         self._optimizer = optimizer
         self._use_amp = use_amp
         self._learning_phase: Optional[bool] = None
+        self._opt_level = opt_level
+        self._loss_scale = loss_scale
+
+        # Check if model is RNN-like to decide if freezing batch-norm and dropout layers might be required for loss and
+        # class gradient calculation
+        self.is_rnn = any((isinstance(m, torch.nn.modules.RNNBase) for m in self._model.modules()))
 
         # Get the internal layers
-        self._layer_names = self._model.get_layers
+        self._layer_names: List[str] = self._model.get_layers  # type: ignore
 
         self._model.to(self._device)
 
         # Index of layer at which the class gradients should be calculated
         self._layer_idx_gradients = -1
 
-        if isinstance(self._loss, (torch.nn.CrossEntropyLoss, torch.nn.NLLLoss, torch.nn.MultiMarginLoss),):
+        if isinstance(
+            self._loss,
+            (torch.nn.CrossEntropyLoss, torch.nn.NLLLoss, torch.nn.MultiMarginLoss),
+        ):
             self._reduce_labels = True
             self._int_labels = True
-        elif isinstance(self._loss, (torch.nn.BCELoss),):
+        elif isinstance(
+            self._loss,
+            (torch.nn.BCELoss),
+        ):
             self._reduce_labels = True
             self._int_labels = False
         else:
@@ -153,8 +166,8 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             self._int_labels = False
 
         # Setup for AMP use
-        if self._use_amp:
-            from apex import amp
+        if self._use_amp:  # pragma: no cover
+            from apex import amp  # pylint: disable=E0611
 
             if self._optimizer is None:
                 logger.warning(
@@ -190,7 +203,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
     @property
     def model(self) -> "torch.nn.Module":
-        return self._model._model
+        return self._model._model  # pylint: disable=W0212
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -201,43 +214,101 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         return self._input_shape  # type: ignore
 
+    @property
+    def loss(self) -> "torch.nn.modules.loss._Loss":
+        """
+        Return the loss function.
+
+        :return: The loss function.
+        """
+        return self._loss  # type: ignore
+
+    @property
+    def optimizer(self) -> "torch.optim.Optimizer":
+        """
+        Return the optimizer.
+
+        :return: The optimizer.
+        """
+        return self._optimizer  # type: ignore
+
+    @property
+    def use_amp(self) -> bool:
+        """
+        Return a boolean indicating whether to use the automatic mixed precision tool.
+
+        :return: Whether to use the automatic mixed precision tool.
+        """
+        return self._use_amp  # type: ignore
+
+    @property
+    def opt_level(self) -> str:
+        """
+        Return a string specifying a pure or mixed precision optimization level.
+
+        :return: A string specifying a pure or mixed precision optimization level. Possible
+                 values are `O0`, `O1`, `O2`, and `O3`.
+        """
+        return self._opt_level  # type: ignore
+
+    @property
+    def loss_scale(self) -> Union[float, str]:
+        """
+        Return the loss scaling value.
+
+        :return: Loss scaling. Possible values for string: a string representing a number, e.g., “1.0”,
+                 or the string “dynamic”.
+        """
+        return self._loss_scale  # type: ignore
+
     def reduce_labels(self, y: Union[np.ndarray, "torch.Tensor"]) -> Union[np.ndarray, "torch.Tensor"]:
+        """
+        Reduce labels from one-hot encoded to index labels.
+        """
+        # pylint: disable=R0911
         import torch  # lgtm [py/repeated-import]
 
         # Check if the loss function requires as input index labels instead of one-hot-encoded labels
-        if self._reduce_labels and self._int_labels:
-            if isinstance(y, torch.Tensor):
-                return torch.argmax(y, dim=1)
-            else:
+        # Checking for exactly 2 classes to support binary classification
+        if self.nb_classes > 2 or (self.nb_classes == 2 and len(y.shape) == 2 and y.shape[1] == 2):
+            if self._reduce_labels and self._int_labels:
+                if isinstance(y, torch.Tensor):
+                    return torch.argmax(y, dim=1)
                 return np.argmax(y, axis=1)
-        elif self._reduce_labels:  # float labels
-            if isinstance(y, torch.Tensor):
-                return torch.argmax(y, dim=1).type("torch.FloatTensor")
-            else:
+            if self._reduce_labels:  # float labels
+                if isinstance(y, torch.Tensor):
+                    return torch.argmax(y, dim=1).type("torch.FloatTensor")
                 y_index = np.argmax(y, axis=1).astype(np.float32)
                 y_index = np.expand_dims(y_index, axis=1)
                 return y_index
-        else:
             return y
 
-    def predict(self, x: np.ndarray, batch_size: int = 128, **kwargs) -> np.ndarray:
+        if isinstance(y, torch.Tensor):
+            return y.float()
+        return y.astype(np.float32)
+
+    def predict(  # pylint: disable=W0221
+        self, x: np.ndarray, batch_size: int = 128, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :param batch_size: Size of batches.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
         import torch  # lgtm [py/repeated-import]
 
-        # Put the model in the eval mode
-        self._model.eval()
+        # Set model mode
+        self._model.train(mode=training_mode)
 
         # Apply preprocessing
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
 
+        results_list = []
+
         # Run prediction with batch processing
-        results = np.zeros((x_preprocessed.shape[0], self.nb_classes), dtype=np.float32)
         num_batch = int(np.ceil(len(x_preprocessed) / float(batch_size)))
         for m in range(num_batch):
             # Batch indexes
@@ -249,13 +320,37 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             with torch.no_grad():
                 model_outputs = self._model(torch.from_numpy(x_preprocessed[begin:end]).to(self._device))
             output = model_outputs[-1]
-            results[begin:end] = output.detach().cpu().numpy()
+            output = output.detach().cpu().numpy().astype(np.float32)
+            if len(output.shape) == 1:
+                output = np.expand_dims(output.detach().cpu().numpy(), axis=1).astype(np.float32)
+
+            results_list.append(output)
+
+        results = np.vstack(results_list)
 
         # Apply postprocessing
         predictions = self._apply_postprocessing(preds=results, fit=False)
 
         return predictions
 
+    def _predict_framework(self, x: "torch.Tensor") -> "torch.Tensor":
+        """
+        Perform prediction for a batch of inputs.
+
+        :param x: Test set.
+        :return: Tensor of predictions of shape `(nb_inputs, nb_classes)`.
+        """
+        # Apply preprocessing
+        x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False, no_grad=False)
+
+        # Put the model in the eval mode
+        self._model.eval()
+
+        model_outputs = self._model(x_preprocessed)
+        output = model_outputs[-1]
+
+        return output
+
     def fit(self, x: np.ndarray, y: np.ndarray, batch_size: int = 128, nb_epochs: int = 10, **kwargs) -> None:
         """
         Fit the classifier on the training set `(x, y)`.
@@ -273,7 +368,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         # Put the model in the training mode
         self._model.train()
 
-        if self._optimizer is None:
+        if self._optimizer is None:  # pragma: no cover
             raise ValueError("An optimizer is needed to train the model, but none for provided.")
 
         y = check_and_transform_label_format(y, self.nb_classes)
@@ -304,11 +399,11 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                 model_outputs = self._model(i_batch)
 
                 # Form the loss function
-                loss = self._loss(model_outputs[-1], o_batch)
+                loss = self._loss(model_outputs[-1], o_batch)  # lgtm [py/call-to-non-callable]
 
                 # Do training
-                if self._use_amp:
-                    from apex import amp
+                if self._use_amp:  # pragma: no cover
+                    from apex import amp  # pylint: disable=E0611
 
                     with amp.scale_loss(loss, self._optimizer) as scaled_loss:
                         scaled_loss.backward()
@@ -333,11 +428,23 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         # Put the model in the training mode
         self._model.train()
 
-        if self._optimizer is None:
+        if self._optimizer is None:  # pragma: no cover
             raise ValueError("An optimizer is needed to train the model, but none for provided.")
 
         # Train directly in PyTorch
-        if isinstance(generator, PyTorchDataGenerator) and not self.preprocessing:
+        from art.preprocessing.standardisation_mean_std.pytorch import StandardisationMeanStdPyTorch
+
+        if isinstance(generator, PyTorchDataGenerator) and (
+            self.preprocessing is None
+            or (
+                isinstance(self.preprocessing, StandardisationMeanStdPyTorch)
+                and (
+                    self.preprocessing.mean,
+                    self.preprocessing.std,
+                )
+                == (0, 1)
+            )
+        ):
             for _ in range(nb_epochs):
                 for i_batch, o_batch in generator.iterator:
                     if isinstance(i_batch, np.ndarray):
@@ -360,8 +467,8 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                     loss = self._loss(model_outputs[-1], o_batch)
 
                     # Do training
-                    if self._use_amp:
-                        from apex import amp
+                    if self._use_amp:  # pragma: no cover
+                        from apex import amp  # pylint: disable=E0611
 
                         with amp.scale_loss(loss, self._optimizer) as scaled_loss:
                             scaled_loss.backward()
@@ -375,7 +482,38 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             # Fit a generic data generator through the API
             super().fit_generator(generator, nb_epochs=nb_epochs)
 
-    def class_gradient(self, x: np.ndarray, label: Union[int, List[int], None] = None, **kwargs) -> np.ndarray:
+    def clone_for_refitting(self) -> "PyTorchClassifier":  # lgtm [py/inheritance/incorrect-overridden-signature]
+        """
+        Create a copy of the classifier that can be refit from scratch. Will inherit same architecture, optimizer and
+        initialization as cloned model, but without weights.
+
+        :return: new estimator
+        """
+        model = copy.deepcopy(self.model)
+        clone = type(self)(model, self._loss, self.input_shape, self.nb_classes, optimizer=self._optimizer)
+        # reset weights
+        clone.reset()
+        params = self.get_params()
+        del params["model"]
+        clone.set_params(**params)
+        return clone
+
+    def reset(self) -> None:
+        """
+        Resets the weights of the classifier so that it can be refit from scratch.
+
+        """
+
+        def weight_reset(module):
+            reset_parameters = getattr(module, "reset_parameters", None)
+            if reset_parameters and callable(reset_parameters):
+                module.reset_parameters()
+
+        self.model.apply(weight_reset)
+
+    def class_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, label: Union[int, List[int], None] = None, training_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute per-class derivatives w.r.t. `x`.
 
@@ -384,23 +522,44 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                       output is computed for all samples. If multiple values as provided, the first dimension should
                       match the batch size of `x`, and each value will be used as target for its corresponding sample in
                       `x`. If `None`, then gradients for all classes will be computed for each sample.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
+                              Note on RNN-like models: Backpropagation through RNN modules in eval mode raises
+                              RuntimeError due to cudnn issues and require training mode, i.e. RuntimeError: cudnn RNN
+                              backward can only be called in training mode. Therefore, if the model is an RNN type we
+                              always use training mode but freeze batch-norm and dropout layers if
+                              `training_mode=False.`
         :return: Array of gradients of input features w.r.t. each class in the form
                  `(batch_size, nb_classes, input_shape)` when computing for all classes, otherwise shape becomes
                  `(batch_size, 1, input_shape)` when `label` parameter is specified.
         """
         import torch  # lgtm [py/repeated-import]
 
+        self._model.train(mode=training_mode)
+
+        # Backpropagation through RNN modules in eval mode raises RuntimeError due to cudnn issues and require training
+        # mode, i.e. RuntimeError: cudnn RNN backward can only be called in training mode. Therefore, if the model is
+        # an RNN type we always use training mode but freeze batch-norm and dropout layers if training_mode=False.
+        if self.is_rnn:
+            self._model.train(mode=True)
+            if not training_mode:
+                logger.debug(  # pragma: no cover
+                    "Freezing batch-norm and dropout layers for gradient calculation in train mode with eval parameters"
+                    "of batch-norm and dropout."
+                )
+                self.set_batchnorm(train=False)
+                self.set_dropout(train=False)
+
         if not (
             (label is None)
-            or (isinstance(label, (int, np.integer)) and label in range(self._nb_classes))
+            or (isinstance(label, (int, np.integer)) and label in range(self.nb_classes))
             or (
                 isinstance(label, np.ndarray)
                 and len(label.shape) == 1
-                and (label < self._nb_classes).all()
+                and (label < self.nb_classes).all()
                 and label.shape[0] == x.shape[0]
             )
         ):
-            raise ValueError("Label %s is out of range." % label)
+            raise ValueError("Label %s is out of range." % label)  # pragma: no cover
 
         # Apply preprocessing
         if self.all_framework_preprocessing:
@@ -428,11 +587,11 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         preds = model_outputs[-1]
 
         # Compute the gradient
-        grads = []
+        grads_list = list()
 
         def save_grad():
             def hook(grad):
-                grads.append(grad.cpu().numpy().copy())
+                grads_list.append(grad.cpu().numpy().copy())
                 grad.data.zero_()
 
             return hook
@@ -441,37 +600,57 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
         self._model.zero_grad()
         if label is None:
-            for i in range(self.nb_classes):
+            if len(preds.shape) == 1 or preds.shape[1] == 1:
+                num_outputs = 1
+            else:
+                num_outputs = self.nb_classes
+
+            for i in range(num_outputs):
                 torch.autograd.backward(
-                    preds[:, i], torch.tensor([1.0] * len(preds[:, 0])).to(self._device), retain_graph=True,
+                    preds[:, i],
+                    torch.tensor([1.0] * len(preds[:, 0])).to(self._device),
+                    retain_graph=True,
                 )
 
+            grads = np.swapaxes(np.array(grads_list), 0, 1)
+
         elif isinstance(label, (int, np.integer)):
             torch.autograd.backward(
-                preds[:, label], torch.tensor([1.0] * len(preds[:, 0])).to(self._device), retain_graph=True,
+                preds[:, label],
+                torch.tensor([1.0] * len(preds[:, 0])).to(self._device),
+                retain_graph=True,
             )
+            grads = np.swapaxes(np.array(grads_list), 0, 1)
         else:
             unique_label = list(np.unique(label))
             for i in unique_label:
                 torch.autograd.backward(
-                    preds[:, i], torch.tensor([1.0] * len(preds[:, 0])).to(self._device), retain_graph=True,
+                    preds[:, i],
+                    torch.tensor([1.0] * len(preds[:, 0])).to(self._device),
+                    retain_graph=True,
                 )
 
-            grads = np.swapaxes(np.array(grads), 0, 1)
+            grads = np.swapaxes(np.array(grads_list), 0, 1)
             lst = [unique_label.index(i) for i in label]
             grads = grads[np.arange(len(grads)), lst]
 
             grads = grads[None, ...]
+            grads = np.swapaxes(np.array(grads), 0, 1)
 
-        grads = np.swapaxes(np.array(grads), 0, 1)
         if not self.all_framework_preprocessing:
             grads = self._apply_preprocessing_gradient(x, grads)
 
         return grads
 
-    def loss(self, x: np.ndarray, y: np.ndarray, reduction: str = "none", **kwargs) -> np.ndarray:
+    def compute_loss(  # pylint: disable=W0221
+        self,
+        x: Union[np.ndarray, "torch.Tensor"],
+        y: Union[np.ndarray, "torch.Tensor"],
+        reduction: str = "none",
+        **kwargs
+    ) -> Union[np.ndarray, "torch.Tensor"]:
         """
-        Compute the loss function w.r.t. `x`.
+        Compute the loss.
 
         :param x: Sample input with shape as expected by the model.
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
@@ -484,17 +663,24 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
+        self._model.eval()
+
+        y = check_and_transform_label_format(y, self.nb_classes)
+
         # Apply preprocessing
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
 
         # Check label shape
         y_preprocessed = self.reduce_labels(y_preprocessed)
 
-        # Convert the inputs to Tensors
-        inputs_t = torch.from_numpy(x_preprocessed).to(self._device)
-
-        # Convert the labels to Tensors
-        labels_t = torch.from_numpy(y_preprocessed).to(self._device)
+        if isinstance(x, torch.Tensor):
+            inputs_t = x_preprocessed
+            labels_t = y_preprocessed
+        else:
+            # Convert the inputs to Tensors
+            inputs_t = torch.from_numpy(x_preprocessed).to(self._device)
+            # Convert the labels to Tensors
+            labels_t = torch.from_numpy(y_preprocessed).to(self._device)
 
         # Compute the loss and return
         model_outputs = self._model(inputs_t)
@@ -505,10 +691,37 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         loss = self._loss(model_outputs[-1], labels_t)
         self._loss.reduction = prev_reduction
 
-        return loss.detach().numpy()
+        if isinstance(x, torch.Tensor):
+            return loss
 
-    def loss_gradient(
-        self, x: Union[np.ndarray, "torch.Tensor"], y: Union[np.ndarray, "torch.Tensor"], **kwargs
+        return loss.detach().cpu().numpy()
+
+    def compute_losses(
+        self,
+        x: Union[np.ndarray, "torch.Tensor"],
+        y: Union[np.ndarray, "torch.Tensor"],
+        reduction: str = "none",
+    ) -> Dict[str, Union[np.ndarray, "torch.Tensor"]]:
+        """
+        Compute all loss components.
+
+        :param x: Sample input with shape as expected by the model.
+        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
+                  of shape `(nb_samples,)`.
+        :param reduction: Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
+                   'none': no reduction will be applied
+                   'mean': the sum of the output will be divided by the number of elements in the output,
+                   'sum': the output will be summed.
+        :return: Dictionary of loss components.
+        """
+        return {"total": self.compute_loss(x=x, y=y, reduction=reduction)}
+
+    def loss_gradient(  # pylint: disable=W0221
+        self,
+        x: Union[np.ndarray, "torch.Tensor"],
+        y: Union[np.ndarray, "torch.Tensor"],
+        training_mode: bool = False,
+        **kwargs
     ) -> Union[np.ndarray, "torch.Tensor"]:
         """
         Compute the gradient of the loss function w.r.t. `x`.
@@ -516,15 +729,42 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         :param x: Sample input with shape as expected by the model.
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices of shape
                   `(nb_samples,)`.
+        :param training_mode: `True` for model set to training mode and `'False` for model set to evaluation mode.
+                              Note on RNN-like models: Backpropagation through RNN modules in eval mode raises
+                              RuntimeError due to cudnn issues and require training mode, i.e. RuntimeError: cudnn RNN
+                              backward can only be called in training mode. Therefore, if the model is an RNN type we
+                              always use training mode but freeze batch-norm and dropout layers if
+                              `training_mode=False.`
         :return: Array of gradients of the same shape as `x`.
         """
         import torch  # lgtm [py/repeated-import]
 
+        self._model.train(mode=training_mode)
+
+        # Backpropagation through RNN modules in eval mode raises RuntimeError due to cudnn issues and require training
+        # mode, i.e. RuntimeError: cudnn RNN backward can only be called in training mode. Therefore, if the model is
+        # an RNN type we always use training mode but freeze batch-norm and dropout layers if training_mode=False.
+        if self.is_rnn:
+            self._model.train(mode=True)
+            if not training_mode:
+                logger.debug(
+                    "Freezing batch-norm and dropout layers for gradient calculation in train mode with eval parameters"
+                    "of batch-norm and dropout."
+                )
+                self.set_batchnorm(train=False)
+                self.set_dropout(train=False)
+
         # Apply preprocessing
         if self.all_framework_preprocessing:
-            x_grad = torch.tensor(x).to(self._device)
-            y_grad = torch.tensor(y).to(self._device)
-            x_grad.requires_grad = True
+            if isinstance(x, torch.Tensor):
+                x_grad = x.clone().detach().requires_grad_(True)
+            else:
+                x_grad = torch.tensor(x).to(self._device)
+                x_grad.requires_grad = True
+            if isinstance(y, torch.Tensor):
+                y_grad = y.clone().detach()
+            else:
+                y_grad = torch.tensor(y).to(self._device)
             inputs_t, y_preprocessed = self._apply_preprocessing(x_grad, y=y_grad, fit=False, no_grad=False)
         elif isinstance(x, np.ndarray):
             x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y=y, fit=False, no_grad=True)
@@ -544,14 +784,14 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
         # Compute the gradient and return
         model_outputs = self._model(inputs_t)
-        loss = self._loss(model_outputs[-1], labels_t)
+        loss = self._loss(model_outputs[-1], labels_t)  # lgtm [py/call-to-non-callable]
 
         # Clean gradients
         self._model.zero_grad()
 
         # Compute gradients
-        if self._use_amp:
-            from apex import amp
+        if self._use_amp:  # pragma: no cover
+            from apex import amp  # pylint: disable=E0611
 
             with amp.scale_loss(loss, self._optimizer) as scaled_loss:
                 scaled_loss.backward()
@@ -572,7 +812,11 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         return grads
 
     def get_activations(
-        self, x: np.ndarray, layer: Union[int, str], batch_size: int = 128, framework: bool = False
+        self,
+        x: Union[np.ndarray, "torch.Tensor"],
+        layer: Optional[Union[int, str]] = None,
+        batch_size: int = 128,
+        framework: bool = False,
     ) -> np.ndarray:
         """
         Return the output of the specified layer for input `x`. `layer` is specified by layer index (between 0 and
@@ -587,23 +831,31 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
+        self._model.eval()
+
         # Apply defences
-        x_preprocessed, _ = self._apply_preprocessing(x=x, y=None, fit=False)
+        if framework:
+            no_grad = False
+        else:
+            no_grad = True
+        x_preprocessed, _ = self._apply_preprocessing(x=x, y=None, fit=False, no_grad=no_grad)
 
         # Get index of the extracted layer
         if isinstance(layer, six.string_types):
-            if layer not in self._layer_names:
+            if layer not in self._layer_names:  # pragma: no cover
                 raise ValueError("Layer name %s not supported" % layer)
             layer_index = self._layer_names.index(layer)
 
-        elif isinstance(layer, (int, np.integer)):
+        elif isinstance(layer, int):
             layer_index = layer
 
-        else:
+        else:  # pragma: no cover
             raise TypeError("Layer must be of type str or int")
 
         if framework:
-            return self._model(torch.from_numpy(x).to(self._device))[layer_index]
+            if isinstance(x_preprocessed, torch.Tensor):
+                return self._model(x_preprocessed)[layer_index]
+            return self._model(torch.from_numpy(x_preprocessed).to(self._device))[layer_index]
 
         # Run prediction with batch processing
         results = []
@@ -624,16 +876,6 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
         return results
 
-    def set_learning_phase(self, train: bool) -> None:
-        """
-        Set the learning phase for the backend framework.
-
-        :param train: True to set the learning phase to training, False to set it to prediction.
-        """
-        if isinstance(train, bool):
-            self._learning_phase = train
-            self._model.train(train)
-
     def save(self, filename: str, path: Optional[str] = None) -> None:
         """
         Save a model to file in the format specific to the backend framework.
@@ -709,7 +951,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
 
     def __repr__(self):
         repr_ = (
-            "%s(model=%r, loss=%r, optimizer=%r, input_shape=%r, nb_classes=%r, channel_index=%r, channels_first=%r, "
+            "%s(model=%r, loss=%r, optimizer=%r, input_shape=%r, nb_classes=%r, channels_first=%r, "
             "clip_values=%r, preprocessing_defences=%r, postprocessing_defences=%r, preprocessing=%r)"
             % (
                 self.__module__ + "." + self.__class__.__name__,
@@ -718,7 +960,6 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                 self._optimizer,
                 self._input_shape,
                 self.nb_classes,
-                self.channel_index,
                 self.channels_first,
                 self.clip_values,
                 self.preprocessing_defences,
@@ -778,7 +1019,7 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                             x = self._model(x)
                             result.append(x)
 
-                        else:
+                        else:  # pragma: no cover
                             raise TypeError("The input model must inherit from `nn.Module`.")
 
                         return result
@@ -809,10 +1050,11 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
                         elif isinstance(self._model, nn.Module):
                             result.append("final_layer")
 
-                        else:
+                        else:  # pragma: no cover
                             raise TypeError("The input model must inherit from `nn.Module`.")
                         logger.info(
-                            "Inferred %i hidden layers on PyTorch classifier.", len(result),
+                            "Inferred %i hidden layers on PyTorch classifier.",
+                            len(result),
                         )
 
                         return result
@@ -823,5 +1065,5 @@ class PyTorchClassifier(ClassGradientsMixin, ClassifierMixin, PyTorchEstimator):
             # Use model wrapping class to wrap the PyTorch model received as argument
             return self._model_wrapper(model)
 
-        except ImportError:
+        except ImportError:  # pragma: no cover
             raise ImportError("Could not find PyTorch (`torch`) installation.") from ImportError
