@@ -18,6 +18,7 @@
 """
 This module implements the classifiers for scikit-learn models.
 """
+# pylint: disable=C0302
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from copy import deepcopy
@@ -34,6 +35,7 @@ from art.estimators.classification.classifier import (
     ClassGradientsMixin,
     ClassifierMixin,
 )
+from art.estimators.regression.scikitlearn import ScikitlearnDecisionTreeRegressor
 from art.estimators.scikitlearn import ScikitlearnEstimator
 from art.utils import to_categorical
 from art import config
@@ -56,11 +58,12 @@ def SklearnClassifier(
     clip_values: Optional["CLIP_VALUES_TYPE"] = None,
     preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
     postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-    preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+    preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
+    use_logits: bool = False,
 ) -> "ScikitlearnClassifier":
     """
     Create a `Classifier` instance from a scikit-learn Classifier model. This is a convenience function that
-    instantiates the correct wrapper class for the given scikit-learn model.
+    instantiates the correct class for the given scikit-learn model.
 
     :param model: scikit-learn Classifier model.
     :param clip_values: Tuple of the form `(min, max)` representing the minimum and maximum values allowed
@@ -71,7 +74,7 @@ def SklearnClassifier(
             used for data preprocessing. The first value will be subtracted from the input. The input will then
             be divided by the second one.
     """
-    if model.__class__.__module__.split(".")[0] != "sklearn":
+    if model.__class__.__module__.split(".")[0] != "sklearn":  # pragma: no cover
         raise TypeError("Model is not an sklearn model. Received '%s'" % model.__class__)
 
     sklearn_name = model.__class__.__name__
@@ -86,21 +89,31 @@ def SklearnClassifier(
         )
 
     # This basic class at least generically handles `fit`, `predict` and `save`
-    return ScikitlearnClassifier(model, clip_values, preprocessing_defences, postprocessing_defences, preprocessing,)
+    return ScikitlearnClassifier(
+        model,
+        clip_values,
+        preprocessing_defences,
+        postprocessing_defences,
+        preprocessing,
+        use_logits,
+    )
 
 
 class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/missing-call-to-init]
     """
-    Wrapper class for scikit-learn classifier models.
+    Class for scikit-learn classifier models.
     """
 
+    estimator_params = ClassifierMixin.estimator_params + ScikitlearnEstimator.estimator_params + ["use_logits"]
+
     def __init__(
         self,
         model: "sklearn.base.BaseEstimator",
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
+        use_logits: bool = False,
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn classifier model.
@@ -113,6 +126,8 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
         :param preprocessing: Tuple of the form `(subtrahend, divisor)` of floats or `np.ndarray` of values to be
                used for data preprocessing. The first value will be subtracted from the input. The input will then
                be divided by the second one.
+        :param use_logits: Determines whether predict() returns logits instead of probabilities if available. Some
+               adversarial attacks (DeepFool) may perform better if logits are used.
         """
         super().__init__(
             model=model,
@@ -122,7 +137,10 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
             preprocessing=preprocessing,
         )
         self._input_shape = self._get_input_shape(model)
-        self._nb_classes = self._get_nb_classes()
+        nb_classes = self._get_nb_classes()
+        if nb_classes is not None:
+            self.nb_classes = nb_classes
+        self._use_logits = use_logits
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -133,6 +151,15 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
         """
         return self._input_shape  # type: ignore
 
+    @property
+    def use_logits(self) -> bool:
+        """
+        Return the Boolean for using logits.
+
+        :return: Boolean for using logits.
+        """
+        return self._use_logits  # type: ignore
+
     def fit(self, x: np.ndarray, y: np.ndarray, **kwargs) -> None:
         """
         Fit the classifier on the training set `(x, y)`.
@@ -148,24 +175,35 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
 
         self.model.fit(x_preprocessed, y_preprocessed, **kwargs)
         self._input_shape = self._get_input_shape(self.model)
-        self._nb_classes = self._get_nb_classes()
+        self.nb_classes = self._get_nb_classes()
 
     def predict(self, x: np.ndarray, **kwargs) -> np.ndarray:
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         :raises `ValueError`: If the classifier does not have methods `predict` or `predict_proba`.
         """
         # Apply defences
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
 
-        if hasattr(self.model, "predict_proba") and callable(getattr(self.model, "predict_proba")):
+        if self._use_logits:
+            if callable(getattr(self.model, "predict_log_proba", None)):
+                y_pred = self.model.predict_log_proba(x_preprocessed)
+            else:  # pragma: no cover
+                logger.warning(
+                    "use_logits was True but classifier did not have callable predict_log_proba member. Falling back to"
+                    " probabilities"
+                )
+        elif callable(getattr(self.model, "predict_proba", None)):
             y_pred = self.model.predict_proba(x_preprocessed)
-        elif hasattr(self.model, "predict") and callable(getattr(self.model, "predict")):
-            y_pred = to_categorical(self.model.predict(x_preprocessed), nb_classes=self.model.classes_.shape[0],)
-        else:
+        elif callable(getattr(self.model, "predict", None)):
+            y_pred = to_categorical(
+                self.model.predict(x_preprocessed),
+                nb_classes=self.model.classes_.shape[0],
+            )
+        else:  # pragma: no cover
             raise ValueError("The provided model does not have methods `predict_proba` or `predict`.")
 
         # Apply postprocessing
@@ -186,31 +224,33 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
         else:
             full_path = os.path.join(path, filename)
         folder = os.path.split(full_path)[0]
-        if not os.path.exists(folder):
+        if not os.path.exists(folder):  # pragma: no cover
             os.makedirs(folder)
 
         with open(full_path + ".pickle", "wb") as file_pickle:
             pickle.dump(self.model, file=file_pickle)
 
-    def _get_input_shape(self, model) -> Optional[Tuple[int, ...]]:
-        _input_shape: Optional[Tuple[int, ...]]
-        if hasattr(model, "n_features_"):
-            _input_shape = (model.n_features_,)
-        elif hasattr(model, "feature_importances_"):
-            _input_shape = (len(model.feature_importances_),)
-        elif hasattr(model, "coef_"):
-            if len(model.coef_.shape) == 1:
-                _input_shape = (model.coef_.shape[0],)
-            else:
-                _input_shape = (model.coef_.shape[1],)
-        elif hasattr(model, "support_vectors_"):
-            _input_shape = (model.support_vectors_.shape[1],)
-        elif hasattr(model, "steps"):
-            _input_shape = self._get_input_shape(model.steps[0][1])
-        else:
-            logger.warning("Input shape not recognised. The model might not have been fitted.")
-            _input_shape = None
-        return _input_shape
+    def clone_for_refitting(self) -> "ScikitlearnClassifier":  # lgtm [py/inheritance/incorrect-overridden-signature]
+        """
+        Create a copy of the classifier that can be refit from scratch.
+
+        :return: new estimator
+        """
+        import sklearn  # lgtm [py/repeated-import]
+
+        clone = type(self)(sklearn.base.clone(self.model))
+        params = self.get_params()
+        del params["model"]
+        clone.set_params(**params)
+        return clone
+
+    def reset(self) -> None:
+        """
+        Resets the weights of the classifier so that it can be refit from scratch.
+
+        """
+        # No need to do anything since scikitlearn models start from scratch each time fit() is called
+        pass
 
     def _get_nb_classes(self) -> int:
         if hasattr(self.model, "n_classes_"):
@@ -225,7 +265,7 @@ class ScikitlearnClassifier(ClassifierMixin, ScikitlearnEstimator):  # lgtm [py/
 
 class ScikitlearnDecisionTreeClassifier(ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn Decision Tree Classifier models.
+    Class for scikit-learn Decision Tree Classifier models.
     """
 
     def __init__(
@@ -234,7 +274,7 @@ class ScikitlearnDecisionTreeClassifier(ScikitlearnClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Decision Tree Classifier model.
@@ -369,99 +409,9 @@ class ScikitlearnDecisionTreeClassifier(ScikitlearnClassifier):
         return leaf_nodes
 
 
-class ScikitlearnDecisionTreeRegressor(ScikitlearnDecisionTreeClassifier):
-    """
-    Wrapper class for scikit-learn Decision Tree Regressor models.
-    """
-
-    def __init__(
-        self,
-        model: "sklearn.tree.DecisionTreeRegressor",
-        clip_values: Optional["CLIP_VALUES_TYPE"] = None,
-        preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
-        postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
-    ) -> None:
-        """
-        Create a `Regressor` instance from a scikit-learn Decision Tree Regressor model.
-
-        :param model: scikit-learn Decision Tree Regressor model.
-        :param clip_values: Tuple of the form `(min, max)` representing the minimum and maximum values allowed
-               for features.
-        :param preprocessing_defences: Preprocessing defence(s) to be applied by the classifier.
-        :param postprocessing_defences: Postprocessing defence(s) to be applied by the classifier.
-        :param preprocessing: Tuple of the form `(subtrahend, divisor)` of floats or `np.ndarray` of values to be
-               used for data preprocessing. The first value will be subtracted from the input. The input will then
-               be divided by the second one.
-        """
-        # pylint: disable=E0001
-        import sklearn  # lgtm [py/repeated-import]
-
-        if not isinstance(model, sklearn.tree.DecisionTreeRegressor):
-            raise TypeError("Model must be of type sklearn.tree.DecisionTreeRegressor.")
-
-        ScikitlearnDecisionTreeClassifier.__init__(
-            self,
-            model=None,
-            clip_values=clip_values,
-            preprocessing_defences=preprocessing_defences,
-            postprocessing_defences=postprocessing_defences,
-            preprocessing=preprocessing,
-        )
-        self._model = model
-
-    def get_values_at_node(self, node_id: int) -> np.ndarray:
-        """
-        Returns the feature of given id for a node.
-
-        :return: Normalized values at node node_id.
-        """
-        return self.model.tree_.value[node_id]
-
-    def _get_leaf_nodes(self, node_id, i_tree, class_label, box) -> List["LeafNode"]:
-        from art.metrics.verification_decisions_trees import LeafNode, Box, Interval
-
-        leaf_nodes: List[LeafNode] = list()
-
-        if self.get_left_child(node_id) != self.get_right_child(node_id):
-
-            node_left = self.get_left_child(node_id)
-            node_right = self.get_right_child(node_id)
-
-            box_left = deepcopy(box)
-            box_right = deepcopy(box)
-
-            feature = self.get_feature_at_node(node_id)
-            box_split_left = Box(intervals={feature: Interval(-np.inf, self.get_threshold_at_node(node_id))})
-            box_split_right = Box(intervals={feature: Interval(self.get_threshold_at_node(node_id), np.inf)})
-
-            if box.intervals:
-                box_left.intersect_with_box(box_split_left)
-                box_right.intersect_with_box(box_split_right)
-            else:
-                box_left = box_split_left
-                box_right = box_split_right
-
-            leaf_nodes += self._get_leaf_nodes(node_left, i_tree, class_label, box_left)
-            leaf_nodes += self._get_leaf_nodes(node_right, i_tree, class_label, box_right)
-
-        else:
-            leaf_nodes.append(
-                LeafNode(
-                    tree_id=i_tree,
-                    class_label=class_label,
-                    node_id=node_id,
-                    box=box,
-                    value=self.get_values_at_node(node_id)[0, 0],
-                )
-            )
-
-        return leaf_nodes
-
-
 class ScikitlearnExtraTreeClassifier(ScikitlearnDecisionTreeClassifier):
     """
-    Wrapper class for scikit-learn Extra TreeClassifier Classifier models.
+    Class for scikit-learn Extra TreeClassifier Classifier models.
     """
 
     def __init__(
@@ -470,7 +420,7 @@ class ScikitlearnExtraTreeClassifier(ScikitlearnDecisionTreeClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Extra TreeClassifier Classifier model.
@@ -500,7 +450,7 @@ class ScikitlearnExtraTreeClassifier(ScikitlearnDecisionTreeClassifier):
 
 class ScikitlearnAdaBoostClassifier(ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn AdaBoost Classifier models.
+    Class for scikit-learn AdaBoost Classifier models.
     """
 
     def __init__(
@@ -509,7 +459,7 @@ class ScikitlearnAdaBoostClassifier(ScikitlearnClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn AdaBoost Classifier model.
@@ -539,7 +489,7 @@ class ScikitlearnAdaBoostClassifier(ScikitlearnClassifier):
 
 class ScikitlearnBaggingClassifier(ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn Bagging Classifier models.
+    Class for scikit-learn Bagging Classifier models.
     """
 
     def __init__(
@@ -548,7 +498,7 @@ class ScikitlearnBaggingClassifier(ScikitlearnClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Bagging Classifier model.
@@ -579,7 +529,7 @@ class ScikitlearnBaggingClassifier(ScikitlearnClassifier):
 
 class ScikitlearnExtraTreesClassifier(ScikitlearnClassifier, DecisionTreeMixin):
     """
-    Wrapper class for scikit-learn Extra Trees Classifier models.
+    Class for scikit-learn Extra Trees Classifier models.
     """
 
     def __init__(
@@ -588,7 +538,7 @@ class ScikitlearnExtraTreesClassifier(ScikitlearnClassifier, DecisionTreeMixin):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ):
         """
         Create a `Classifier` instance from a scikit-learn Extra Trees Classifier model.
@@ -652,7 +602,7 @@ class ScikitlearnExtraTreesClassifier(ScikitlearnClassifier, DecisionTreeMixin):
 
 class ScikitlearnGradientBoostingClassifier(ScikitlearnClassifier, DecisionTreeMixin):
     """
-    Wrapper class for scikit-learn Gradient Boosting Classifier models.
+    Class for scikit-learn Gradient Boosting Classifier models.
     """
 
     def __init__(
@@ -661,7 +611,7 @@ class ScikitlearnGradientBoostingClassifier(ScikitlearnClassifier, DecisionTreeM
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Gradient Boosting Classifier model.
@@ -726,7 +676,7 @@ class ScikitlearnGradientBoostingClassifier(ScikitlearnClassifier, DecisionTreeM
 
 class ScikitlearnRandomForestClassifier(ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn Random Forest Classifier models.
+    Class for scikit-learn Random Forest Classifier models.
     """
 
     def __init__(
@@ -735,7 +685,7 @@ class ScikitlearnRandomForestClassifier(ScikitlearnClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Random Forest Classifier model.
@@ -799,7 +749,7 @@ class ScikitlearnRandomForestClassifier(ScikitlearnClassifier):
 
 class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn Logistic Regression models.
+    Class for scikit-learn Logistic Regression models.
     """
 
     def __init__(
@@ -808,7 +758,7 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Logistic Regression model.
@@ -822,6 +772,12 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
                used for data preprocessing. The first value will be subtracted from the input. The input will then
                be divided by the second one.
         """
+        # pylint: disable=E0001
+        import sklearn  # lgtm [py/repeated-import]
+
+        if not isinstance(model, sklearn.linear_model.LogisticRegression):
+            raise TypeError("Model must be of type sklearn.linear_model.LogisticRegression).")
+
         super().__init__(
             model=model,
             clip_values=clip_values,
@@ -849,12 +805,12 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
             classes in the classifier is not known.
         :raises `TypeError`: If the requested label cannot be processed.
         """
-        if not hasattr(self.model, "coef_"):
+        if not hasattr(self.model, "coef_"):  # pragma: no cover
             raise ValueError(
                 """Model has not been fitted. Run function `fit(x, y)` of classifier first or provide a
             fitted model."""
             )
-        if self.nb_classes is None:
+        if self.nb_classes is None:  # pragma: no cover
             raise ValueError("Unknown number of classes in classifier.")
         nb_samples = x.shape[0]
 
@@ -885,7 +841,7 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
 
             gradients = np.swapaxes(np.array(class_gradients), 0, 1)
 
-        elif isinstance(label, (int, np.integer)):
+        elif isinstance(label, int):
             # Compute the gradients only w.r.t. the provided label
             class_gradient = np.zeros(x.shape)
             for i_sample in range(nb_samples):
@@ -926,9 +882,6 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
-        | Paper link: http://cs229.stanford.edu/proj2016/report/ItkinaWu-AdversarialAttacksonImageRecognition-report.pdf
-        | Typo in https://arxiv.org/abs/1605.07277 (equation 6)
-
         :param x: Sample input with shape as expected by the model.
         :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices of shape
                   `(nb_samples,)`.
@@ -938,7 +891,7 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
         # pylint: disable=E0001
         from sklearn.utils.class_weight import compute_class_weight
 
-        if not hasattr(self.model, "coef_"):
+        if not hasattr(self.model, "coef_"):  # pragma: no cover
             raise ValueError(
                 """Model has not been fitted. Run function `fit(x, y)` of classifier first or provide a
             fitted model."""
@@ -947,39 +900,28 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
         # Apply preprocessing
         x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=False)
 
-        num_samples, _ = x_preprocessed.shape
-        gradients = np.zeros(x_preprocessed.shape)
-
         y_index = np.argmax(y_preprocessed, axis=1)
         if self.model.class_weight is None or self.model.class_weight == "balanced":
             class_weight = np.ones(self.nb_classes)
         else:
             class_weight = compute_class_weight(
-                class_weight=self.model.class_weight, classes=self.model.classes_, y=y_index,
+                class_weight=self.model.class_weight,
+                classes=self.model.classes_,
+                y=y_index,
             )
 
-        y_pred = self.model.predict_proba(X=x_preprocessed)
+        y_pred = self.predict(x=x_preprocessed)
         weights = self.model.coef_
 
-        # Consider the special case of a binary logistic regression model:
-        if self.nb_classes == 2:
-            for i_sample in range(num_samples):
-                gradients[i_sample, :] += (
-                    class_weight[1] * (1.0 - y_preprocessed[i_sample, 1])
-                    - class_weight[0] * (1.0 - y_preprocessed[i_sample, 0])
-                ) * (y_pred[i_sample, 0] * y_pred[i_sample, 1] * weights[0, :])
-        else:
-            w_weighted = np.matmul(y_pred, weights)
+        errors = class_weight * (y_pred - y)
 
-            for i_sample in range(num_samples):
-                for i_class in range(self.nb_classes):  # type: ignore
-                    gradients[i_sample, :] += (
-                        class_weight[i_class]
-                        * (1.0 - y_preprocessed[i_sample, i_class])
-                        * (weights[i_class, :] - w_weighted[i_sample, :])
-                    )
+        if weights.shape[0] == 1:
+            weights = np.append(-weights, weights, axis=0)
+
+        gradients = (errors @ weights) / self.model.classes_.size
 
         gradients = self._apply_preprocessing_gradient(x, gradients)
+
         return gradients
 
     @staticmethod
@@ -994,7 +936,7 @@ class ScikitlearnLogisticRegression(ClassGradientsMixin, LossGradientsMixin, Sci
 
 class ScikitlearnGaussianNB(ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn Gaussian Naive Bayes models.
+    Class for scikit-learn Gaussian Naive Bayes models.
     """
 
     def __init__(
@@ -1003,7 +945,7 @@ class ScikitlearnGaussianNB(ScikitlearnClassifier):
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn Gaussian Naive Bayes (GaussianNB) model.
@@ -1020,7 +962,7 @@ class ScikitlearnGaussianNB(ScikitlearnClassifier):
         # pylint: disable=E0001
         import sklearn  # lgtm [py/repeated-import]
 
-        if not isinstance(model, sklearn.naive_bayes.GaussianNB):
+        if not isinstance(model, sklearn.naive_bayes.GaussianNB):  # pragma: no cover
             raise TypeError("Model must be of type sklearn.naive_bayes.GaussianNB. Found type {}".format(type(model)))
 
         super().__init__(
@@ -1043,7 +985,7 @@ class ScikitlearnGaussianNB(ScikitlearnClassifier):
 
 class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassifier):
     """
-    Wrapper class for scikit-learn C-Support Vector Classification models.
+    Class for scikit-learn C-Support Vector Classification models.
     """
 
     def __init__(
@@ -1052,7 +994,7 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
     ) -> None:
         """
         Create a `Classifier` instance from a scikit-learn C-Support Vector Classification model.
@@ -1105,7 +1047,7 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
         num_samples, _ = x_preprocessed.shape
 
         if isinstance(self.model, sklearn.svm.SVC):
-            if self.model.fit_status_:
+            if self.model.fit_status_:  # pragma: no cover
                 raise AssertionError("Model has not been fitted correctly.")
 
             support_indices = [0] + list(np.cumsum(self.model.n_support_))
@@ -1116,7 +1058,13 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                 sign_multiplier = 1
 
             if label is None:
-                gradients = np.zeros((x_preprocessed.shape[0], self.nb_classes, x_preprocessed.shape[1],))
+                gradients = np.zeros(
+                    (
+                        x_preprocessed.shape[0],
+                        self.nb_classes,
+                        x_preprocessed.shape[1],
+                    )
+                )
 
                 for i_label in range(self.nb_classes):  # type: ignore
                     for i_sample in range(num_samples):
@@ -1127,21 +1075,29 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                                 else:
                                     label_multiplier = 1
 
-                                for label_sv in range(support_indices[i_label], support_indices[i_label + 1],):
+                                for label_sv in range(
+                                    support_indices[i_label],
+                                    support_indices[i_label + 1],
+                                ):
                                     alpha_i_k_y_i = self.model.dual_coef_[
-                                        not_label if not_label < i_label else not_label - 1, label_sv,
+                                        not_label if not_label < i_label else not_label - 1,
+                                        label_sv,
                                     ]
                                     grad_kernel = self._get_kernel_gradient_sv(label_sv, x_preprocessed[i_sample])
                                     gradients[i_sample, i_label] += label_multiplier * alpha_i_k_y_i * grad_kernel
 
-                                for not_label_sv in range(support_indices[not_label], support_indices[not_label + 1],):
+                                for not_label_sv in range(
+                                    support_indices[not_label],
+                                    support_indices[not_label + 1],
+                                ):
                                     alpha_i_k_y_i = self.model.dual_coef_[
-                                        i_label if i_label < not_label else i_label - 1, not_label_sv,
+                                        i_label if i_label < not_label else i_label - 1,
+                                        not_label_sv,
                                     ]
                                     grad_kernel = self._get_kernel_gradient_sv(not_label_sv, x_preprocessed[i_sample])
                                     gradients[i_sample, i_label] += label_multiplier * alpha_i_k_y_i * grad_kernel
 
-            elif isinstance(label, (int, np.integer)):
+            elif isinstance(label, int):
                 gradients = np.zeros((x_preprocessed.shape[0], 1, x_preprocessed.shape[1]))
 
                 for i_sample in range(num_samples):
@@ -1154,14 +1110,19 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
 
                             for label_sv in range(support_indices[label], support_indices[label + 1]):
                                 alpha_i_k_y_i = self.model.dual_coef_[
-                                    not_label if not_label < label else not_label - 1, label_sv,
+                                    not_label if not_label < label else not_label - 1,
+                                    label_sv,
                                 ]
                                 grad_kernel = self._get_kernel_gradient_sv(label_sv, x_preprocessed[i_sample])
                                 gradients[i_sample, 0] += label_multiplier * alpha_i_k_y_i * grad_kernel
 
-                            for not_label_sv in range(support_indices[not_label], support_indices[not_label + 1],):
+                            for not_label_sv in range(
+                                support_indices[not_label],
+                                support_indices[not_label + 1],
+                            ):
                                 alpha_i_k_y_i = self.model.dual_coef_[
-                                    label if label < not_label else label - 1, not_label_sv,
+                                    label if label < not_label else label - 1,
+                                    not_label_sv,
                                 ]
                                 grad_kernel = self._get_kernel_gradient_sv(not_label_sv, x_preprocessed[i_sample])
                                 gradients[i_sample, 0] += label_multiplier * alpha_i_k_y_i * grad_kernel
@@ -1182,15 +1143,20 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                                 label_multiplier = 1
 
                             for label_sv in range(
-                                support_indices[label[i_sample]], support_indices[label[i_sample] + 1],
+                                support_indices[label[i_sample]],
+                                support_indices[label[i_sample] + 1],
                             ):
                                 alpha_i_k_y_i = self.model.dual_coef_[
-                                    not_label if not_label < label[i_sample] else not_label - 1, label_sv,
+                                    not_label if not_label < label[i_sample] else not_label - 1,
+                                    label_sv,
                                 ]
                                 grad_kernel = self._get_kernel_gradient_sv(label_sv, x_preprocessed[i_sample])
                                 gradients[i_sample, 0] += label_multiplier * alpha_i_k_y_i * grad_kernel
 
-                            for not_label_sv in range(support_indices[not_label], support_indices[not_label + 1],):
+                            for not_label_sv in range(
+                                support_indices[not_label],
+                                support_indices[not_label + 1],
+                            ):
                                 alpha_i_k_y_i = self.model.dual_coef_[
                                     label[i_sample] if label[i_sample] < not_label else label[i_sample] - 1,
                                     not_label_sv,
@@ -1205,7 +1171,13 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
 
         elif isinstance(self.model, sklearn.svm.LinearSVC):
             if label is None:
-                gradients = np.zeros((x_preprocessed.shape[0], self.nb_classes, x_preprocessed.shape[1],))
+                gradients = np.zeros(
+                    (
+                        x_preprocessed.shape[0],
+                        self.nb_classes,
+                        x_preprocessed.shape[1],
+                    )
+                )
 
                 for i in range(self.nb_classes):  # type: ignore
                     for i_sample in range(num_samples):
@@ -1214,7 +1186,7 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                         else:
                             gradients[i_sample, i] = self.model.coef_[i]
 
-            elif isinstance(label, (int, np.integer)):
+            elif isinstance(label, int):
                 gradients = np.zeros((x_preprocessed.shape[0], 1, x_preprocessed.shape[1]))
 
                 for i_sample in range(num_samples):
@@ -1265,7 +1237,7 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                 2
                 * self.model._gamma
                 * (-1)
-                * np.exp(-self.model._gamma * np.linalg.norm(x_sample - sv, ord=2))
+                * np.exp(-self.model._gamma * np.linalg.norm(x_sample - sv, ord=2) ** 2)
                 * (x_sample - sv)
             )
         elif self.model.kernel == "sigmoid":
@@ -1338,7 +1310,10 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
                             grad_kernel = self._get_kernel_gradient_sv(i_label_sv, x_preprocessed[i_sample])
                             gradients[i_sample, :] += sign_multiplier * alpha_i_k_y_i * grad_kernel
 
-                        for i_not_label_sv in range(support_indices[i_not_label], support_indices[i_not_label + 1],):
+                        for i_not_label_sv in range(
+                            support_indices[i_not_label],
+                            support_indices[i_not_label + 1],
+                        ):
                             alpha_i_k_y_i = self.model.dual_coef_[i_not_label_i, i_not_label_sv] * label_multiplier
                             grad_kernel = self._get_kernel_gradient_sv(i_not_label_sv, x_preprocessed[i_sample])
                             gradients[i_sample, :] += sign_multiplier * alpha_i_k_y_i * grad_kernel
@@ -1423,7 +1398,7 @@ class ScikitlearnSVC(ClassGradientsMixin, LossGradientsMixin, ScikitlearnClassif
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :return: Array of predictions of shape `(nb_inputs, nb_classes)`.
         """
         # pylint: disable=E0001
