@@ -29,7 +29,7 @@ from art.estimators.tensorflow import TensorFlowEstimator
 if TYPE_CHECKING:
     # pylint: disable=C0412
     import numpy as np
-    import tensorflow as tf
+    import tensorflow.compat.v1 as tf
 
     from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
     from art.defences.preprocessor import Preprocessor
@@ -43,6 +43,14 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
     This class implements an encoder model using the TensorFlow framework.
     """
 
+    estimator_params = TensorFlowEstimator.estimator_params + [
+        "input_ph",
+        "loss",
+        "sess",
+        "feed_dict",
+        "channels_first",
+    ]
+
     def __init__(
         self,
         input_ph: "tf.Placeholder",
@@ -53,7 +61,7 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
         feed_dict: Optional[Dict[Any, Any]] = None,
     ):
         """
@@ -77,7 +85,7 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
         :param feed_dict: A feed dictionary for the session run evaluating the classifier. This dictionary includes all
                           additionally required placeholders except the placeholders defined in this class.
         """
-        import tensorflow as tf  # lgtm [py/repeated-import]
+        import tensorflow.compat.v1 as tf  # lgtm [py/repeated-import]
 
         super().__init__(
             model=model,
@@ -99,13 +107,13 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
             self._feed_dict = feed_dict
 
         # Assign session
-        if sess is None:
+        if sess is None:  # pragma: no cover
             raise ValueError("A session cannot be None.")
         self._sess = sess
 
         # Get the loss gradients graph
-        if self._loss is not None:
-            self._loss_grads = tf.gradients(self._loss, self._input_ph)[0]
+        if self.loss is not None:
+            self._loss_grads = tf.gradients(self.loss, self.input_ph)[0]
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -116,18 +124,45 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
         """
         return self._input_shape  # type: ignore
 
+    @property
+    def input_ph(self) -> "tf.Placeholder":
+        """
+        Return the input placeholder.
+
+        :return: The input placeholder.
+        """
+        return self._input_ph  # type: ignore
+
+    @property
+    def loss(self) -> "tf.Tensor":
+        """
+        Return the loss function.
+
+        :return: The loss function.
+        """
+        return self._loss  # type: ignore
+
+    @property
+    def feed_dict(self) -> Dict[Any, Any]:
+        """
+        Return the feed dictionary for the session run evaluating the classifier.
+
+        :return: The feed dictionary for the session run evaluating the classifier.
+        """
+        return self._feed_dict  # type: ignore
+
     def predict(self, x: "np.ndarray", batch_size: int = 128, **kwargs):
         """
         Perform prediction for a batch of inputs.
 
-        :param x: Test set.
+        :param x: Input samples.
         :param batch_size: Batch size.
         :return: Array of encoding predictions of shape `(num_inputs, encoding_length)`.
         """
         logger.info("Encoding input")
-        feed_dict = {self._input_ph: x}
-        if self._feed_dict is not None:
-            feed_dict.update(self._feed_dict)
+        feed_dict = {self.input_ph: x}
+        if self.feed_dict is not None:
+            feed_dict.update(self.feed_dict)
         y = self._sess.run(self._model, feed_dict=feed_dict)
         return y
 
@@ -145,31 +180,15 @@ class TensorFlowEncoder(EncoderMixin, TensorFlowEstimator):  # lgtm [py/missing-
         """
         raise NotImplementedError
 
-    def set_learning_phase(self, train: bool) -> None:
-        """
-        Do nothing.
-        """
+    def compute_loss(self, x: "np.ndarray", y: "np.ndarray", **kwargs) -> "np.ndarray":
         raise NotImplementedError
 
-    def loss_gradient(self, x: "np.ndarray", y: "np.ndarray", **kwargs) -> "np.ndarray":
+    def loss_gradient(self, x: "np.ndarray", y: "np.ndarray", **kwargs) -> "np.ndarray":  # pylint: disable=W0221
         """
         No gradients to compute for this method; do nothing.
         """
         raise NotImplementedError
 
-    def loss(self, x: "np.ndarray", y: "np.ndarray", **kwargs) -> "np.ndarray":
-        """
-        Compute the loss of the neural network for samples `x`.
-
-        :param x: Samples of shape (nb_samples, nb_features) or (nb_samples, nb_pixels_1, nb_pixels_2,
-                  nb_channels) or (nb_samples, nb_channels, nb_pixels_1, nb_pixels_2).
-        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
-                  of shape `(nb_samples,)`.
-        :return: Loss values.
-        :rtype: Format as expected by the `model`
-        """
-        raise NotImplementedError
-
     @property
     def encoding_length(self) -> int:
         """
