@@ -22,10 +22,9 @@ from abc import ABC, abstractmethod
 from typing import Any, Dict, List, Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import trange
+from tqdm.auto import trange
 
 from art.config import ART_NUMPY_DTYPE
-from art.utils import Deprecated, deprecated, deprecated_keyword_arg
 
 if TYPE_CHECKING:
     # pylint: disable=R0401
@@ -56,7 +55,7 @@ class BaseEstimator(ABC):
         clip_values: Optional["CLIP_VALUES_TYPE"],
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: Union["PREPROCESSING_TYPE", "Preprocessor"] = (0.0, 1.0),
     ):
         """
         Initialize a `BaseEstimator` object.
@@ -72,41 +71,79 @@ class BaseEstimator(ABC):
                used for data preprocessing. The first value will be subtracted from the input and the results will be
                divided by the second value.
         """
-        from art.defences.postprocessor.postprocessor import Postprocessor
-        from art.defences.preprocessor.preprocessor import Preprocessor
-
         self._model = model
         self._clip_values = clip_values
 
-        self.preprocessing: List["Preprocessor"] = []
+        self.preprocessing = self._set_preprocessing(preprocessing)
+        self.preprocessing_defences = self._set_preprocessing_defences(preprocessing_defences)
+        self.postprocessing_defences = self._set_postprocessing_defences(postprocessing_defences)
+        self.preprocessing_operations: List["Preprocessor"] = []
+        BaseEstimator._update_preprocessing_operations(self)
+        BaseEstimator._check_params(self)
 
-        if preprocessing_defences is None:
+    def _update_preprocessing_operations(self):
+        from art.defences.preprocessor.preprocessor import Preprocessor
+
+        self.preprocessing_operations.clear()
+
+        if self.preprocessing_defences is None:
             pass
-        elif isinstance(preprocessing_defences, Preprocessor):
-            self.preprocessing.append(preprocessing_defences)
+        elif isinstance(self.preprocessing_defences, Preprocessor):
+            self.preprocessing_operations.append(self.preprocessing_defences)
         else:
-            self.preprocessing += preprocessing_defences
+            self.preprocessing_operations += self.preprocessing_defences
 
-        self.preprocessing_defences = preprocessing_defences
+        if self.preprocessing is None:
+            pass
+        elif isinstance(self.preprocessing, tuple):
+            from art.preprocessing.standardisation_mean_std.numpy import StandardisationMeanStd
+
+            self.preprocessing_operations.append(
+                StandardisationMeanStd(mean=self.preprocessing[0], std=self.preprocessing[1])
+            )
+        elif isinstance(self.preprocessing, Preprocessor):
+            self.preprocessing_operations.append(self.preprocessing)
+        else:  # pragma: no cover
+            raise ValueError("Preprocessing argument not recognised.")
+
+    @staticmethod
+    def _set_preprocessing(
+        preprocessing: Optional[Union["PREPROCESSING_TYPE", "Preprocessor"]]
+    ) -> Optional["Preprocessor"]:
+        from art.defences.preprocessor.preprocessor import Preprocessor
 
         if preprocessing is None:
-            pass
-        elif isinstance(preprocessing, tuple):
-            from art.preprocessing.standardisation_mean_std.standardisation_mean_std import StandardisationMeanStd
+            return None
+        if isinstance(preprocessing, tuple):
+            from art.preprocessing.standardisation_mean_std.numpy import StandardisationMeanStd
 
-            self.preprocessing.append(StandardisationMeanStd(mean=preprocessing[0], std=preprocessing[1]))
-        elif isinstance(preprocessing, Preprocessor):
-            self.preprocessing.append(preprocessing)
-        else:
-            self.preprocessing += preprocessing
+            return StandardisationMeanStd(mean=preprocessing[0], std=preprocessing[1])
+        if isinstance(preprocessing, Preprocessor):
+            return preprocessing
+
+        raise ValueError("Preprocessing argument not recognised.")  # pragma: no cover
+
+    @staticmethod
+    def _set_preprocessing_defences(
+        preprocessing_defences: Optional[Union["Preprocessor", List["Preprocessor"]]]
+    ) -> Optional[List["Preprocessor"]]:
+        from art.defences.preprocessor.preprocessor import Preprocessor
+
+        if isinstance(preprocessing_defences, Preprocessor):
+            return [preprocessing_defences]
+
+        return preprocessing_defences
+
+    @staticmethod
+    def _set_postprocessing_defences(
+        postprocessing_defences: Optional[Union["Postprocessor", List["Postprocessor"]]]
+    ) -> Optional[List["Postprocessor"]]:
+        from art.defences.postprocessor.postprocessor import Postprocessor
 
-        self.postprocessing_defences: Optional[List["Postprocessor"]]
         if isinstance(postprocessing_defences, Postprocessor):
-            self.postprocessing_defences = [postprocessing_defences]
-        else:
-            self.postprocessing_defences = postprocessing_defences
+            return [postprocessing_defences]
 
-        self._check_params()
+        return postprocessing_defences
 
     def set_params(self, **kwargs) -> None:
         """
@@ -116,12 +153,25 @@ class BaseEstimator(ABC):
         """
         for key, value in kwargs.items():
             if key in self.estimator_params:
-                if hasattr(BaseEstimator, key) and isinstance(getattr(BaseEstimator, key), property):
+                if hasattr(type(self), key) and isinstance(getattr(type(self), key), property):
+                    if getattr(type(self), key).fset is not None:
+                        setattr(self, key, value)
+                    else:
+                        setattr(self, "_" + key, value)
+                elif hasattr(self, "_" + key):
                     setattr(self, "_" + key, value)
                 else:
-                    setattr(self, key, value)
-            else:
-                raise ValueError("Unexpected parameter {} found in kwargs.".format(key))
+                    if key == "preprocessing":
+                        setattr(self, key, self._set_preprocessing(value))
+                    elif key == "preprocessing_defences":
+                        setattr(self, key, self._set_preprocessing_defences(value))
+                    elif key == "postprocessing_defences":
+                        setattr(self, key, self._set_postprocessing_defences(value))
+                    else:
+                        setattr(self, key, value)
+            else:  # pragma: no cover
+                raise ValueError("Unexpected parameter `{}` found in kwargs.".format(key))
+        self._update_preprocessing_operations()
         self._check_params()
 
     def get_params(self) -> Dict[str, Any]:
@@ -140,11 +190,11 @@ class BaseEstimator(ABC):
         from art.defences.preprocessor.preprocessor import Preprocessor
 
         if self._clip_values is not None:
-            if len(self._clip_values) != 2:
+            if len(self._clip_values) != 2:  # pragma: no cover
                 raise ValueError(
                     "`clip_values` should be a tuple of 2 floats or arrays containing the allowed data range."
                 )
-            if np.array(self._clip_values[0] >= self._clip_values[1]).any():
+            if np.array(self._clip_values[0] >= self._clip_values[1]).any():  # pragma: no cover
                 raise ValueError("Invalid `clip_values`: min >= max.")
 
             if isinstance(self._clip_values, np.ndarray):
@@ -152,14 +202,14 @@ class BaseEstimator(ABC):
             else:
                 self._clip_values = np.array(self._clip_values, dtype=ART_NUMPY_DTYPE)
 
-        if isinstance(self.preprocessing, list):
-            for preprocess in self.preprocessing:
-                if not isinstance(preprocess, Preprocessor):
+        if isinstance(self.preprocessing_operations, list):
+            for preprocess in self.preprocessing_operations:
+                if not isinstance(preprocess, Preprocessor):  # pragma: no cover
                     raise ValueError(
                         "All preprocessing defences have to be instance of "
                         "art.defences.preprocessor.preprocessor.Preprocessor."
                     )
-        else:
+        else:  # pragma: no cover
             raise ValueError(
                 "All preprocessing defences have to be instance of "
                 "art.defences.preprocessor.preprocessor.Preprocessor."
@@ -167,14 +217,14 @@ class BaseEstimator(ABC):
 
         if isinstance(self.postprocessing_defences, list):
             for postproc_defence in self.postprocessing_defences:
-                if not isinstance(postproc_defence, Postprocessor):
+                if not isinstance(postproc_defence, Postprocessor):  # pragma: no cover
                     raise ValueError(
                         "All postprocessing defences have to be instance of "
                         "art.defences.postprocessor.postprocessor.Postprocessor."
                     )
         elif self.postprocessing_defences is None:
             pass
-        else:
+        else:  # pragma: no cover
             raise ValueError(
                 "All postprocessing defences have to be instance of "
                 "art.defences.postprocessor.postprocessor.Postprocessor."
@@ -245,8 +295,8 @@ class BaseEstimator(ABC):
         :return: Tuple of `x` and `y` after applying the defences and standardisation.
         :rtype: Format as expected by the `model`
         """
-        if self.preprocessing:
-            for preprocess in self.preprocessing:
+        if self.preprocessing_operations:
+            for preprocess in self.preprocessing_operations:
                 if fit:
                     if preprocess.apply_fit:
                         x, y = preprocess(x, y)
@@ -277,12 +327,23 @@ class BaseEstimator(ABC):
 
         return post_preds
 
+    def compute_loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+        """
+        Compute the loss of the estimator for samples `x`.
+
+        :param x: Input samples.
+        :param y: Target values.
+        :return: Loss values.
+        :rtype: Format as expected by the `model`
+        """
+        raise NotImplementedError
+
     def __repr__(self):
         class_name = self.__class__.__name__
         attributes = {}
-        for k, v in self.__dict__.items():
+        for k, value in self.__dict__.items():
             k = k[1:] if k[0] == "_" else k
-            attributes[k] = v
+            attributes[k] = value
         attributes = ["{}={}".format(k, v) for k, v in attributes.items()]
         repr_string = class_name + "(" + ", ".join(attributes) + ")"
         return repr_string
@@ -322,8 +383,8 @@ class LossGradientsMixin(ABC):
         :return: Gradients after backward pass through normalization and preprocessing defences.
         :rtype: Format as expected by the `model`
         """
-        if self.preprocessing:
-            for preprocess in self.preprocessing[::-1]:
+        if self.preprocessing_operations:
+            for preprocess in self.preprocessing_operations[::-1]:
                 if fit:
                     if preprocess.apply_fit:
                         gradients = preprocess.estimate_gradient(x, gradients)
@@ -340,34 +401,23 @@ class NeuralNetworkMixin(ABC):
     has to be mixed in with class `BaseEstimator`.
     """
 
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
-    def __init__(self, channels_first: Optional[bool], channel_index=Deprecated, **kwargs) -> None:
+    estimator_params = ["channels_first"]
+
+    def __init__(self, channels_first: bool, **kwargs) -> None:
         """
         Initialize a neural network attributes.
 
-        :param channel_index: Index of the axis in samples `x` representing the color channels.
-        :type channel_index: `int`
         :param channels_first: Set channels first or last.
         """
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
-
-        self._channel_index = channel_index
-        self._channels_first: Optional[bool] = channels_first
-        super().__init__(**kwargs)
+        self._channels_first: bool = channels_first
+        super().__init__(**kwargs)  # type: ignore
 
     @abstractmethod
     def predict(self, x: np.ndarray, batch_size: int = 128, **kwargs):
         """
         Perform prediction of the neural network for samples `x`.
 
-        :param x: Samples of shape (nb_samples, nb_features) or (nb_samples, nb_pixels_1, nb_pixels_2,
-                  nb_channels) or (nb_samples, nb_channels, nb_pixels_1, nb_pixels_2).
+        :param x: Input samples.
         :param batch_size: Batch size.
         :return: Predictions.
         :rtype: Format as expected by the `model`
@@ -409,11 +459,8 @@ class NeuralNetworkMixin(ABC):
             ):  # type: ignore
                 x, y = generator.get_batch()
 
-                # Apply preprocessing and defences
-                x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=True)  # type: ignore
-
                 # Fit for current batch
-                self.fit(x_preprocessed, y_preprocessed, nb_epochs=1, batch_size=generator.batch_size, **kwargs)
+                self.fit(x, y, nb_epochs=1, batch_size=generator.batch_size, **kwargs)
 
     @abstractmethod
     def get_activations(
@@ -432,56 +479,13 @@ class NeuralNetworkMixin(ABC):
         """
         raise NotImplementedError
 
-    @abstractmethod
-    def set_learning_phase(self, train: bool) -> None:
-        """
-        Set the learning phase for the backend framework.
-
-        :param train: `True` if the learning phase is training, otherwise `False`.
-        """
-        raise NotImplementedError
-
-    @abstractmethod
-    def loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
-        """
-        Compute the loss of the neural network for samples `x`.
-
-        :param x: Samples of shape (nb_samples, nb_features) or (nb_samples, nb_pixels_1, nb_pixels_2,
-                  nb_channels) or (nb_samples, nb_channels, nb_pixels_1, nb_pixels_2).
-        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
-                  of shape `(nb_samples,)`.
-        :return: Loss values.
-        :rtype: Format as expected by the `model`
-        """
-        raise NotImplementedError
-
-    @property  # type: ignore
-    @deprecated(end_version="1.6.0", replaced_by="channels_first")
-    def channel_index(self) -> Optional[int]:
-        """
-        :return: Index of the axis containing the color channels in the samples `x`.
-        """
-        return self._channel_index
-
     @property
-    def channels_first(self) -> Optional[bool]:
+    def channels_first(self) -> bool:
         """
         :return: Boolean to indicate index of the color channels in the sample `x`.
         """
         return self._channels_first
 
-    @property
-    def learning_phase(self) -> Optional[bool]:
-        """
-        The learning phase set by the user. Possible values are `True` for training or `False` for prediction and
-        `None` if it has not been set by the library. In the latter case, the library does not do any explicit learning
-        phase manipulation and the current value of the backend framework is used. If a value has been set by the user
-        for this property, it will impact all following computations for model fitting, prediction and gradients.
-
-        :return: Learning phase.
-        """
-        return self._learning_phase  # type: ignore
-
     @property
     def layer_names(self) -> Optional[List[str]]:
         """
@@ -500,9 +504,9 @@ class NeuralNetworkMixin(ABC):
         name = self.__class__.__name__
 
         attributes = {}
-        for k, v in self.__dict__.items():
+        for k, value in self.__dict__.items():
             k = k[1:] if k[0] == "_" else k
-            attributes[k] = v
+            attributes[k] = value
         attrs = ["{}={}".format(k, v) for k, v in attributes.items()]
         repr_ = name + "(" + ", ".join(attrs) + ")"
 
