@@ -30,10 +30,9 @@ from art import config
 
 if TYPE_CHECKING:
     # pylint: disable=C0412
-    import tensorflow as tf
+    import tensorflow.compat.v1 as tf
     from object_detection.meta_architectures.faster_rcnn_meta_arch import FasterRCNNMetaArch
-    from tensorflow.python.framework.ops import Tensor
-    from tensorflow.python.client.session import Session
+    from tensorflow.python.client.session import Session  # pylint: disable=E0611
 
     from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
     from art.defences.preprocessor.preprocessor import Preprocessor
@@ -47,6 +46,8 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
     This class implements a model-specific object detector using Faster-RCNN and TensorFlow.
     """
 
+    estimator_params = TensorFlowEstimator.estimator_params + ["images", "sess", "is_training", "attack_losses"]
+
     def __init__(
         self,
         images: "tf.Tensor",
@@ -59,7 +60,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
         channels_first: bool = False,
         preprocessing_defences: Union["Preprocessor", List["Preprocessor"], None] = None,
         postprocessing_defences: Union["Postprocessor", List["Postprocessor"], None] = None,
-        preprocessing: "PREPROCESSING_TYPE" = (0, 1),
+        preprocessing: "PREPROCESSING_TYPE" = (0.0, 1.0),
         attack_losses: Tuple[str, ...] = (
             "Loss/RPNLoss/localization_loss",
             "Loss/RPNLoss/objectness_loss",
@@ -100,10 +101,11 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                               `first_stage_localization_loss`, `first_stage_objectness_loss`,
                               `second_stage_localization_loss`, `second_stage_classification_loss`.
         """
-        import tensorflow as tf  # lgtm [py/repeated-import]
+        import tensorflow.compat.v1 as tf  # lgtm [py/repeated-import]
 
         # Super initialization
         super().__init__(
+            model=model,
             clip_values=clip_values,
             channels_first=channels_first,
             preprocessing_defences=preprocessing_defences,
@@ -115,7 +117,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
         if self.clip_values is not None:
             if not np.all(self.clip_values[0] == 0):
                 raise ValueError("This classifier requires normalized input images with clip_vales=(0, 1).")
-            if not np.all(self.clip_values[1] == 1):
+            if not np.all(self.clip_values[1] == 1):  # pragma: no cover
                 raise ValueError("This classifier requires normalized input images with clip_vales=(0, 1).")
 
         # Check preprocessing and postprocessing defences
@@ -156,7 +158,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                     "faster_rcnn_inception_v2_coco_2017_11_08.tar.gz",
                 )
 
-            self._predictions, self._losses, self._detections = self._load_model(
+            self._model, self._predictions, self._losses, self._detections = self._load_model(
                 images=images,
                 filename=filename,
                 url=url,
@@ -168,7 +170,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
             )
 
         else:
-            self._predictions, self._losses, self._detections = self._load_model(
+            self._model, self._predictions, self._losses, self._detections = self._load_model(
                 images=images,
                 filename=None,
                 url=None,
@@ -189,13 +191,20 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
         if sess is None:
             logger.warning("A session cannot be None, create a new session.")
             self._sess = tf.Session()
-        else:
+        else:  # pragma: no cover
             self._sess = sess
 
         # Initialize variables
         self._sess.run(tf.global_variables_initializer())
         self._sess.run(tf.local_variables_initializer())
 
+    @property
+    def native_label_is_pytorch_format(self) -> bool:
+        """
+        Are the native labels in PyTorch format [x1, y1, x2, y2]?
+        """
+        return False
+
     @property
     def input_shape(self) -> Tuple[int, ...]:
         """
@@ -246,7 +255,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                               corresponding loss values.
                     - detections: a dictionary containing final detection results.
         """
-        import tensorflow as tf  # lgtm [py/repeated-import]
+        import tensorflow.compat.v1 as tf  # lgtm [py/repeated-import]
         from object_detection.utils import variables_helper
 
         if obj_detection_model is None:
@@ -255,7 +264,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
 
             # If obj_detection_model is None, then we need to have parameters filename and url to download, extract
             # and load the object detection model
-            if filename is None or url is None:
+            if filename is None or url is None:  # pragma: no cover
                 raise ValueError(
                     "Need input parameters `filename` and `url` to download, "
                     "extract and load the object detection model."
@@ -311,26 +320,31 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
         # Initialize from checkpoint
         tf.train.init_from_checkpoint(fine_tune_checkpoint_path, vars_in_ckpt)
 
-        return predictions, losses, detections
+        return obj_detection_model, predictions, losses, detections
 
-    def loss_gradient(self, x: np.ndarray, y: List[Dict[str, np.ndarray]], **kwargs) -> np.ndarray:
+    def loss_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, y: List[Dict[str, np.ndarray]], standardise_output: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
         :param x: Samples of shape (nb_samples, height, width, nb_channels).
-        :param y: A dictionary of target values. The fields of the dictionary are as follows:
+        :param y: Targets of format `List[Dict[str, np.ndarray]]`, one for each input image. The fields of the Dict are
+                  as follows:
 
-                  - `boxes`: A list of `nb_samples` size of 2-D tf.float32 tensors of shape [num_boxes, 4] containing
-                    coordinates of the groundtruth boxes. Groundtruth boxes are provided in [y_min, x_min, y_max, x_max]
-                    format and also assumed to be normalized as well as clipped relative to the image window with
-                    conditions y_min <= y_max and x_min <= x_max.
-                  - `labels`: A list of `nb_samples` size of 1-D tf.float32 tensors of shape [num_boxes] containing
-                    the class targets with the zero index assumed to map to the first non-background class.
-                  - `scores`: A list of `nb_samples` size of 1-D tf.float32 tensors of shape [num_boxes] containing
-                    weights for groundtruth boxes.
+                 - boxes [N, 4]: the boxes in [y1, x1, y2, x2] in scale [0, 1] (`standardise_output=False`) or
+                                 [x1, y1, x2, y2] in image scale (`standardise_output=True`) format,
+                                 with 0 <= x1 < x2 <= W and 0 <= y1 < y2 <= H.
+                 - labels [N]: the labels for each image in TensorFlow (`standardise_output=False`) or PyTorch
+                               (`standardise_output=True`) format
+
+        :param standardise_output: True if `y` is provided in standardised PyTorch format. Box coordinates will be
+                                   scaled back to [0, 1], label index will be decreased by 1 and the boxes will be
+                                   changed from [x1, y1, x2, y2] to [y1, x1, y2, x2] format, with
+                                   0 <= x1 < x2 <= W and 0 <= y1 < y2 <= H.
         :return: Loss gradients of the same shape as `x`.
         """
-        import tensorflow as tf  # lgtm [py/repeated-import]
+        import tensorflow.compat.v1 as tf  # lgtm [py/repeated-import]
 
         # Only do loss_gradient if is_training is False
         if self.is_training:
@@ -338,6 +352,11 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                 "This object detector was loaded in training mode and therefore not support loss_gradient."
             )
 
+        if standardise_output:
+            from art.estimators.object_detection.utils import convert_pt_to_tf
+
+            y = convert_pt_to_tf(y=y, height=x.shape[1], width=x.shape[2])
+
         # Apply preprocessing
         x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
 
@@ -350,7 +369,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                 else:
                     loss = loss + self._losses[loss_name]
 
-            self._loss_grads: Tensor = tf.gradients(loss, self.images)[0]
+            self._loss_grads: tf.Tensor = tf.gradients(loss, self.images)[0]
 
         # Create feed_dict
         feed_dict = {self.images: x_preprocessed}
@@ -362,7 +381,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
             feed_dict[placeholder] = value["labels"]
 
         for (placeholder, value) in zip(self._groundtruth_weights_list, y):
-            feed_dict[placeholder] = value["scores"]
+            feed_dict[placeholder] = [1.0] * len(value["labels"])
 
         # Compute gradients
         grads = self._sess.run(self._loss_grads, feed_dict=feed_dict)
@@ -371,7 +390,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
 
         return grads
 
-    def predict(
+    def predict(  # pylint: disable=W0221
         self, x: np.ndarray, batch_size: int = 128, standardise_output: bool = False, **kwargs
     ) -> List[Dict[str, np.ndarray]]:
         """
@@ -379,16 +398,18 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
 
         :param x: Samples of shape (nb_samples, height, width, nb_channels).
         :param batch_size: Batch size.
-        :param standardise_output: True if output should be standardised. Box coordinates will be normalised to [0, 1]
-                                   and label index will be decreased by 1 to adhere to COCO categories.
-        :return: A dictionary containing the following fields:
+        :param standardise_output: True if output should be standardised to PyTorch format. Box coordinates will be
+                                   scaled from [0, 1] to image dimensions, label index will be increased by 1 to adhere
+                                   to COCO categories and the boxes will be changed to [x1, y1, x2, y2] format, with
+                                   0 <= x1 < x2 <= W and 0 <= y1 < y2 <= H.
 
         :return: Predictions of format `List[Dict[str, np.ndarray]]`, one for each input image. The
                  fields of the Dict are as follows:
 
-                 - boxes [N, 4]: the predicted boxes in [x1, y1, x2, y2] format, with values \
-                   between 0 and H and 0 and W
-                 - labels [N]: the predicted labels for each image
+                 - boxes [N, 4]: the boxes in [y1, x1, y2, x2] format, with 0 <= x1 < x2 <= W and 0 <= y1 < y2 <= H.
+                                 Can be changed to PyTorch format with `standardise_output=True`.
+                 - labels [N]: the labels for each image in TensorFlow format. Can be changed to PyTorch format with
+                               `standardise_output=True`.
                  - scores [N]: the scores or each prediction.
         """
         # Only do prediction if is_training is False
@@ -402,7 +423,7 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
 
         # Check if batch processing is appropriately set
         if self.images is not None and self.images.shape[0].value is not None:
-            if x.shape[0] % self.images.shape[0].value != 0:
+            if x.shape[0] % self.images.shape[0].value != 0:  # pragma: no cover
                 raise ValueError("Number of prediction samples must be a multiple of input size.")
 
             logger.warning("Reset batch size to input size.")
@@ -427,20 +448,12 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
                 d_sample = dict()
 
                 d_sample["boxes"] = batch_results["detection_boxes"][i]
-                d_sample["labels"] = batch_results["detection_classes"][i].astype(np.int32)
+                d_sample["labels"] = batch_results["detection_classes"][i].astype(int)
 
                 if standardise_output:
-                    height = x.shape[1]
-                    width = x.shape[2]
+                    from art.estimators.object_detection.utils import convert_tf_to_pt
 
-                    d_sample["boxes"][:, 0] *= height
-                    d_sample["boxes"][:, 1] *= width
-                    d_sample["boxes"][:, 2] *= height
-                    d_sample["boxes"][:, 3] *= width
-
-                    d_sample["boxes"] = d_sample["boxes"][:, [1, 0, 3, 2]]
-
-                    d_sample["labels"] = d_sample["labels"] + 1
+                    d_sample = convert_tf_to_pt(y=[d_sample], height=x.shape[1], width=x.shape[2])[0]
 
                 d_sample["scores"] = batch_results["detection_scores"][i]
 
@@ -486,11 +499,86 @@ class TensorFlowFasterRCNN(ObjectDetectorMixin, TensorFlowEstimator):
         """
         return self._detections
 
-    def fit(self):
+    def fit(self, x: np.ndarray, y, batch_size: int = 128, nb_epochs: int = 20, **kwargs) -> None:
         raise NotImplementedError
 
-    def get_activations(self):
+    def get_activations(
+        self, x: np.ndarray, layer: Union[int, str], batch_size: int, framework: bool = False
+    ) -> np.ndarray:
         raise NotImplementedError
 
-    def set_learning_phase(self, train: bool) -> None:
-        raise NotImplementedError
+    def compute_loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+        """
+        Compute the loss.
+
+        :param x: Sample input with shape as expected by the model.
+        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
+                  of shape `(nb_samples,)`.
+        :return: Array of losses of the same shape as `x`.
+        """
+        # Apply preprocessing
+        x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
+
+        # Get the losses graph
+        if not hasattr(self, "_loss_total"):
+            loss = None
+            for loss_name in self.attack_losses:
+                if loss is None:
+                    loss = self._losses[loss_name]
+                else:
+                    loss = loss + self._losses[loss_name]
+            self._loss_total = loss
+
+        # Create feed_dict
+        feed_dict = {self.images: x_preprocessed}
+
+        for (placeholder, value) in zip(self._groundtruth_boxes_list, y):
+            feed_dict[placeholder] = value["boxes"]
+
+        for (placeholder, value) in zip(self._groundtruth_classes_list, y):
+            feed_dict[placeholder] = value["labels"]
+
+        for (placeholder, value) in zip(self._groundtruth_weights_list, y):
+            feed_dict[placeholder] = value["scores"]
+
+        loss_values = self._sess.run(self._loss_total, feed_dict=feed_dict)
+
+        return loss_values
+
+    def compute_losses(self, x: np.ndarray, y: np.ndarray) -> np.ndarray:
+        """
+        Compute all loss components.
+
+        :param x: Samples of shape (nb_samples, nb_features) or (nb_samples, nb_pixels_1, nb_pixels_2,
+                  nb_channels) or (nb_samples, nb_channels, nb_pixels_1, nb_pixels_2).
+        :param y: Target values (class labels) one-hot-encoded of shape `(nb_samples, nb_classes)` or indices
+                  of shape `(nb_samples,)`.
+        :return: Dictionary of loss components.
+        """
+        # Apply preprocessing
+        x_preprocessed, _ = self._apply_preprocessing(x, y=None, fit=False)
+
+        # Create feed_dict
+        feed_dict = {self.images: x_preprocessed}
+
+        for (placeholder, value) in zip(self._groundtruth_boxes_list, y):
+            feed_dict[placeholder] = value["boxes"]
+
+        for (placeholder, value) in zip(self._groundtruth_classes_list, y):
+            feed_dict[placeholder] = value["labels"]
+
+        for (placeholder, value) in zip(self._groundtruth_weights_list, y):
+            feed_dict[placeholder] = value["scores"]
+
+        # Get the losses graph
+        if not hasattr(self, "_losses_dict"):
+            self._losses_dict = dict()
+            for loss_name in self.attack_losses:
+                self._losses_dict[loss_name] = self._losses[loss_name]
+
+        losses = dict()
+        for loss_name in self.attack_losses:
+            loss_value = self._sess.run(self._losses_dict[loss_name], feed_dict=feed_dict)
+            losses[loss_name] = loss_value
+
+        return losses
