@@ -22,35 +22,38 @@ Mandarin in PyTorch.
 | Paper link: https://arxiv.org/abs/1512.02595
 """
 import logging
-from typing import List, Optional, Tuple, Union, TYPE_CHECKING
+from typing import TYPE_CHECKING, List, Optional, Tuple, Union
 
 import numpy as np
 
-from art.estimators.speech_recognition.speech_recognizer import SpeechRecognizerMixin
-from art.estimators.pytorch import PyTorchEstimator
-from art.utils import get_file
 from art import config
+from art.estimators.pytorch import PyTorchEstimator
+from art.estimators.speech_recognition.speech_recognizer import SpeechRecognizerMixin, PytorchSpeechRecognizerMixin
+from art.utils import get_file
 
 if TYPE_CHECKING:
+    # pylint: disable=C0412
     import torch
-
     from deepspeech_pytorch.model import DeepSpeech
 
-    from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
-    from art.defences.preprocessor.preprocessor import Preprocessor
     from art.defences.postprocessor.postprocessor import Postprocessor
+    from art.defences.preprocessor.preprocessor import Preprocessor
+    from art.utils import CLIP_VALUES_TYPE, PREPROCESSING_TYPE
 
 logger = logging.getLogger(__name__)
 
 
-class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
+class PyTorchDeepSpeech(PytorchSpeechRecognizerMixin, SpeechRecognizerMixin, PyTorchEstimator):
     """
     This class implements a model-specific automatic speech recognizer using the end-to-end speech recognizer
-    DeepSpeech and PyTorch.
+    DeepSpeech and PyTorch. It supports both version 2 and version 3 of DeepSpeech models as released at
+    https://github.com/SeanNaren/deepspeech.pytorch.
 
     | Paper link: https://arxiv.org/abs/1512.02595
     """
 
+    estimator_params = PyTorchEstimator.estimator_params + ["optimizer", "use_amp", "opt_level", "lm_config", "verbose"]
+
     def __init__(
         self,
         model: Optional["DeepSpeech"] = None,
@@ -124,7 +127,7 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                             if available otherwise run on CPU.
         """
         import torch  # lgtm [py/repeated-import]
-
+        from deepspeech_pytorch.model import DeepSpeech
         from deepspeech_pytorch.configs.inference_config import LMConfig
         from deepspeech_pytorch.enums import DecoderType
         from deepspeech_pytorch.utils import load_decoder, load_model
@@ -139,24 +142,32 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
             preprocessing=preprocessing,
         )
 
+        # Check DeepSpeech version
+        if str(DeepSpeech.__base__) == "<class 'torch.nn.modules.module.Module'>":
+            self._version = 2
+        elif str(DeepSpeech.__base__) == "<class 'pytorch_lightning.core.lightning.LightningModule'>":
+            self._version = 3
+        else:
+            raise NotImplementedError("Only DeepSpeech version 2 and DeepSpeech version 3 are currently supported.")
+
         self.verbose = verbose
 
         # Check clip values
         if self.clip_values is not None:
-            if not np.all(self.clip_values[0] == -1):
+            if not np.all(self.clip_values[0] == -1):  # pragma: no cover
                 raise ValueError("This estimator requires normalized input audios with clip_vales=(-1, 1).")
-            if not np.all(self.clip_values[1] == 1):
+            if not np.all(self.clip_values[1] == 1):  # pragma: no cover
                 raise ValueError("This estimator requires normalized input audios with clip_vales=(-1, 1).")
 
         # Check postprocessing defences
-        if self.postprocessing_defences is not None:
+        if self.postprocessing_defences is not None:  # pragma: no cover
             raise ValueError("This estimator does not support `postprocessing_defences`.")
 
         # Set cpu/gpu device
         self._device: torch.device
         if device_type == "cpu" or not torch.cuda.is_available():
             self._device = torch.device("cpu")
-        else:
+        else:  # pragma: no cover
             cuda_idx = torch.cuda.current_device()
             self._device = torch.device("cuda:{}".format(cuda_idx))
 
@@ -164,45 +175,87 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
 
         # Load model
         if model is None:
-            if pretrained_model == "an4":
-                filename, url = (
-                    "an4_pretrained_v2.pth",
-                    "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/an4_pretrained_v2.pth",
-                )
+            if self._version == 2:
+                if pretrained_model == "an4":  # pragma: no cover
+                    filename, url = (
+                        "an4_pretrained_v2.pth",
+                        "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/an4_pretrained_v2.pth",
+                    )
 
-            elif pretrained_model == "librispeech":
-                filename, url = (
-                    "librispeech_pretrained_v2.pth",
-                    "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/"
-                    "librispeech_pretrained_v2.pth",
-                )
-
-            elif pretrained_model == "tedlium":
-                filename, url = (
-                    "ted_pretrained_v2.pth",
-                    "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/ted_pretrained_v2.pth",
-                )
-
-            elif pretrained_model is None:
-                # If model is None and no pretrained model is selected, then we need to have parameters filename and
-                # url to download, extract and load the automatic speech recognition model
-                if filename is None or url is None:
+                elif pretrained_model == "librispeech":
                     filename, url = (
                         "librispeech_pretrained_v2.pth",
                         "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/"
                         "librispeech_pretrained_v2.pth",
                     )
 
+                elif pretrained_model == "tedlium":  # pragma: no cover
+                    filename, url = (
+                        "ted_pretrained_v2.pth",
+                        "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/ted_pretrained_v2.pth",
+                    )
+
+                elif pretrained_model is None:  # pragma: no cover
+                    # If model is None and no pretrained model is selected, then we need to have parameters filename
+                    # and url to download, extract and load the automatic speech recognition model
+                    if filename is None or url is None:
+                        filename, url = (
+                            "librispeech_pretrained_v2.pth",
+                            "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/v2.0/"
+                            "librispeech_pretrained_v2.pth",
+                        )
+
+                else:  # pragma: no cover
+                    raise ValueError("The input pretrained model %s is not supported." % pretrained_model)
+
+                # Download model
+                model_path = get_file(
+                    filename=filename, path=config.ART_DATA_PATH, url=url, extract=False, verbose=self.verbose
+                )
+
+                # Then load model
+                self._model = load_model(device=self._device, model_path=model_path, use_half=use_half)
+
             else:
-                raise ValueError("The input pretrained model %s is not supported." % pretrained_model)
+                if pretrained_model == "an4":  # pragma: no cover
+                    filename, url = (
+                        "an4_pretrained_v3.ckpt",
+                        "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/V3.0/an4_pretrained_v3.ckpt",
+                    )
 
-            # Download model
-            model_path = get_file(
-                filename=filename, path=config.ART_DATA_PATH, url=url, extract=False, verbose=self.verbose
-            )
+                elif pretrained_model == "librispeech":
+                    filename, url = (
+                        "librispeech_pretrained_v3.ckpt",
+                        "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/V3.0/"
+                        "librispeech_pretrained_v3.ckpt",
+                    )
 
-            # Then load model
-            self._model = load_model(device=self._device, model_path=model_path, use_half=use_half)
+                elif pretrained_model == "tedlium":  # pragma: no cover
+                    filename, url = (
+                        "ted_pretrained_v3.ckpt",
+                        "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/V3.0/ted_pretrained_v3.ckpt",
+                    )
+
+                elif pretrained_model is None:  # pragma: no cover
+                    # If model is None and no pretrained model is selected, then we need to have parameters filename and
+                    # url to download, extract and load the automatic speech recognition model
+                    if filename is None or url is None:
+                        filename, url = (
+                            "librispeech_pretrained_v3.ckpt",
+                            "https://github.com/SeanNaren/deepspeech.pytorch/releases/download/V3.0/"
+                            "librispeech_pretrained_v3.ckpt",
+                        )
+
+                else:  # pragma: no cover
+                    raise ValueError("The input pretrained model %s is not supported." % pretrained_model)
+
+                # Download model
+                model_path = get_file(
+                    filename=filename, path=config.ART_DATA_PATH, url=url, extract=False, verbose=self.verbose
+                )
+
+                # Then load model
+                self._model = load_model(device=self._device, model_path=model_path)
 
         else:
             self._model = model
@@ -210,9 +263,18 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
             # Push model to the corresponding device
             self._model.to(self._device)
 
+        # Set the loss function
+        if self._version == 2:
+            from warpctc_pytorch import CTCLoss
+
+            self.criterion = CTCLoss()
+        else:
+            self.criterion = self._model.criterion
+
         # Save first version of the optimizer
         self._optimizer = optimizer
         self._use_amp = use_amp
+        self._opt_level = opt_level
 
         # Now create a decoder
         # Create the language model config first
@@ -234,15 +296,16 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         lm_config.cutoff_prob = cutoff_prob
         lm_config.beam_width = beam_width
         lm_config.lm_workers = lm_workers
+        self.lm_config = lm_config
 
         # Create the decoder with the lm config
         self.decoder = load_decoder(labels=self._model.labels, cfg=lm_config)
 
         # Setup for AMP use
-        if self._use_amp:
-            from apex import amp
+        if self.use_amp:  # pragma: no cover
+            from apex import amp  # pylint: disable=E0611
 
-            if self._optimizer is None:
+            if self.optimizer is None:
                 logger.warning(
                     "An optimizer is needed to use the automatic mixed precision tool, but none for provided. "
                     "A default optimizer is used."
@@ -258,7 +321,11 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                 enabled = True
 
             self._model, self._optimizer = amp.initialize(
-                models=self._model, optimizers=self._optimizer, enabled=enabled, opt_level=opt_level, loss_scale=1.0,
+                models=self._model,
+                optimizers=self._optimizer,
+                enabled=enabled,
+                opt_level=opt_level,
+                loss_scale=1.0,
             )
 
     def predict(
@@ -273,10 +340,9 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         :param batch_size: Batch size.
         :param transcription_output: Indicate whether the function will produce probability or transcription as
                                      prediction output. If transcription_output is not available, then probability
-                                     output is returned.
-        :type transcription_output: `bool`
+                                     output is returned. Default: True
         :return: Predicted probability (if transcription_output False) or transcription (default, if
-                 transcription_output is True or None):
+                 transcription_output is True):
                  - Probability return is a tuple of (probs, sizes), where `probs` is the probability of characters of
                  shape (nb_samples, seq_length, nb_classes) and `sizes` is the real sequence length of shape
                  (nb_samples,).
@@ -285,24 +351,24 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
 
-        x_ = np.empty(len(x), dtype=object)
-        x_[:] = list(x)
+        x_in = np.empty(len(x), dtype=object)
+        x_in[:] = list(x)
 
         # Put the model in the eval mode
         self._model.eval()
 
         # Apply preprocessing
-        x_preprocessed, _ = self._apply_preprocessing(x_, y=None, fit=False)
+        x_preprocessed, _ = self._apply_preprocessing(x_in, y=None, fit=False)
 
         # Transform x into the model input space
-        inputs, targets, input_rates, target_sizes, batch_idx = self._transform_model_input(x=x_preprocessed)
+        inputs, _, input_rates, _, batch_idx = self._transform_model_input(x=x_preprocessed)
 
         # Compute real input sizes
         input_sizes = input_rates.mul_(inputs.size()[-1]).int()
 
         # Run prediction with batch processing
         results = []
-        result_output_sizes = np.zeros(x_preprocessed.shape[0], dtype=np.int)
+        result_output_sizes = np.zeros(x_preprocessed.shape[0], dtype=int)
         num_batch = int(np.ceil(len(x_preprocessed) / float(batch_size)))
 
         for m in range(num_batch):
@@ -344,9 +410,9 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         result_outputs[batch_idx] = result_outputs_
 
         # Check if users want transcription outputs
-        transcription_output = kwargs.get("transcription_output")
+        transcription_output = kwargs.get("transcription_output", True)
 
-        if transcription_output is None or transcription_output is False:
+        if transcription_output is False:
             return result_outputs, result_output_sizes
 
         # Now users want transcription outputs
@@ -370,19 +436,19 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                   lengths. A possible example of `y` could be: `y = np.array(['SIXTY ONE', 'HELLO'])`.
         :return: Loss gradients of the same shape as `x`.
         """
-        from warpctc_pytorch import CTCLoss
+        x_in = np.empty(len(x), dtype=object)
+        x_in[:] = list(x)
 
-        x_ = np.empty(len(x), dtype=object)
-        x_[:] = list(x)
-
-        # Put the model in the training mode
+        # Put the model in the training mode, otherwise CUDA can't backpropagate through the model.
+        # However, model uses batch norm layers which need to be frozen
         self._model.train()
+        self.set_batchnorm(train=False)
 
         # Apply preprocessing
-        x_preprocessed, y_preprocessed = self._apply_preprocessing(x_, y, fit=False)
+        x_preprocessed, y_preprocessed = self._apply_preprocessing(x_in, y, fit=False)
 
         # Transform data into the model input space
-        inputs, targets, input_rates, target_sizes, batch_idx = self._transform_model_input(
+        inputs, targets, input_rates, target_sizes, _ = self._transform_model_input(
             x=x_preprocessed, y=y_preprocessed, compute_gradient=True
         )
 
@@ -392,18 +458,24 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         # Call to DeepSpeech model for prediction
         outputs, output_sizes = self._model(inputs.to(self._device), input_sizes.to(self._device))
         outputs = outputs.transpose(0, 1)
-        float_outputs = outputs.float()
 
-        # Loss function
-        criterion = CTCLoss()
-        loss = criterion(float_outputs, targets, output_sizes, target_sizes).to(self._device)
-        loss = loss / inputs.size(0)
+        if self._version == 2:
+            outputs = outputs.float()
+        else:
+            outputs = outputs.log_softmax(-1)
+
+        # Compute the loss
+        loss = self.criterion(outputs, targets, output_sizes, target_sizes).to(self._device)
+
+        # Average the loss by mini batch if version 2 of DeepSpeech is used
+        if self._version == 2:
+            loss = loss / inputs.size(0)
 
         # Compute gradients
-        if self._use_amp:
-            from apex import amp
+        if self.use_amp:  # pragma: no cover
+            from apex import amp  # pylint: disable=E0611
 
-            with amp.scale_loss(loss, self._optimizer) as scaled_loss:
+            with amp.scale_loss(loss, self.optimizer) as scaled_loss:
                 scaled_loss.backward()
 
         else:
@@ -411,7 +483,7 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
 
         # Get results
         results_list = list()
-        for i in range(len(x_preprocessed)):
+        for i, _ in enumerate(x_preprocessed):
             results_list.append(x_preprocessed[i].grad.cpu().numpy().copy())
 
         results = np.array(results_list)
@@ -421,12 +493,15 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
             results_[:] = list(results)
             results = results_
 
-        results = self._apply_preprocessing_gradient(x_, results)
+        results = self._apply_preprocessing_gradient(x_in, results)
 
-        if x.dtype != np.object:
-            results = np.array([i for i in results], dtype=x.dtype)
+        if x.dtype != object:
+            results = np.array([i for i in results], dtype=x.dtype)  # pylint: disable=R1721
             assert results.shape == x.shape and results.dtype == x.dtype
 
+        # Unfreeze batch norm layers again
+        self.set_batchnorm(train=True)
+
         return results
 
     def fit(self, x: np.ndarray, y: np.ndarray, batch_size: int = 128, nb_epochs: int = 10, **kwargs) -> None:
@@ -445,27 +520,22 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         """
         import random
 
-        from warpctc_pytorch import CTCLoss
-
-        x_ = np.empty(len(x), dtype=object)
-        x_[:] = list(x)
+        x_in = np.empty(len(x), dtype=object)
+        x_in[:] = list(x)
 
         # Put the model in the training mode
         self._model.train()
 
-        if self._optimizer is None:
+        if self.optimizer is None:  # pragma: no cover
             raise ValueError("An optimizer is required to train the model, but none was provided.")
 
         # Apply preprocessing
-        x_preprocessed, y_preprocessed = self._apply_preprocessing(x, y, fit=True)
+        x_preprocessed, y_preprocessed = self._apply_preprocessing(x_in, y, fit=True)
 
         # Train with batch processing
         num_batch = int(np.ceil(len(x_preprocessed) / float(batch_size)))
         ind = np.arange(len(x_preprocessed))
 
-        # Loss function
-        criterion = CTCLoss()
-
         # Start training
         for _ in range(nb_epochs):
             # Shuffle the examples
@@ -485,7 +555,7 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                 o_batch = y_preprocessed[ind[begin:end]]
 
                 # Transform data into the model input space
-                inputs, targets, input_rates, target_sizes, batch_idx = self._transform_model_input(
+                inputs, targets, input_rates, target_sizes, _ = self._transform_model_input(
                     x=i_batch, y=o_batch, compute_gradient=False
                 )
 
@@ -493,31 +563,99 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                 input_sizes = input_rates.mul_(inputs.size(-1)).int()
 
                 # Zero the parameter gradients
-                self._optimizer.zero_grad()
+                self.optimizer.zero_grad()
 
                 # Call to DeepSpeech model for prediction
                 outputs, output_sizes = self._model(inputs.to(self._device), input_sizes.to(self._device))
                 outputs = outputs.transpose(0, 1)
-                float_outputs = outputs.float()
 
-                # Form the loss
-                loss = criterion(float_outputs, targets, output_sizes, target_sizes).to(self._device)
-                loss = loss / inputs.size(0)
+                if self._version == 2:
+                    outputs = outputs.float()
+                else:
+                    outputs = outputs.log_softmax(-1)
+
+                # Compute the loss
+                loss = self.criterion(outputs, targets, output_sizes, target_sizes).to(self._device)
+
+                # Average the loss by mini batch if version 2 of DeepSpeech is used
+                if self._version == 2:
+                    loss = loss / inputs.size(0)
 
                 # Actual training
-                if self._use_amp:
-                    from apex import amp
+                if self.use_amp:  # pragma: no cover
+                    from apex import amp  # pylint: disable=E0611
 
-                    with amp.scale_loss(loss, self._optimizer) as scaled_loss:
+                    with amp.scale_loss(loss, self.optimizer) as scaled_loss:
                         scaled_loss.backward()
 
                 else:
                     loss.backward()
 
-                self._optimizer.step()
+                self.optimizer.step()
 
-    def preprocess_transform_model_input(
-        self, x: "torch.Tensor", y: np.ndarray, real_lengths: np.ndarray,
+    def compute_loss_and_decoded_output(
+        self, masked_adv_input: "torch.Tensor", original_output: np.ndarray, **kwargs
+    ) -> Tuple["torch.Tensor", np.ndarray]:
+        """
+        Compute loss function and decoded output.
+
+        :param masked_adv_input: The perturbed inputs.
+        :param original_output: Target values of shape (nb_samples). Each sample in `original_output` is a string and
+                                it may possess different lengths. A possible example of `original_output` could be:
+                                `original_output = np.array(['SIXTY ONE', 'HELLO'])`.
+        :param real_lengths: Real lengths of original sequences.
+        :return: The loss and the decoded output.
+        """
+        # This estimator needs to have real lengths for loss computation
+        real_lengths = kwargs.get("real_lengths")
+        if real_lengths is None:  # pragma: no cover
+            raise ValueError(
+                "The PyTorchDeepSpeech estimator needs information about the real lengths of input sequences to "
+                "compute loss and decoded output."
+            )
+
+        # Transform data into the model input space
+        inputs, targets, input_rates, target_sizes, batch_idx = self._preprocess_transform_model_input(
+            x=masked_adv_input.to(self.device),
+            y=original_output,
+            real_lengths=real_lengths,
+        )
+
+        # Compute real input sizes
+        input_sizes = input_rates.mul_(inputs.size()[-1]).int()
+
+        # Call to DeepSpeech model for prediction
+        outputs, output_sizes = self.model(inputs.to(self.device), input_sizes.to(self.device))
+        outputs_ = outputs.transpose(0, 1)
+
+        if self._version == 2:
+            outputs_ = outputs_.float()
+        else:
+            outputs_ = outputs_.log_softmax(-1)
+
+        # Compute the loss
+        loss = self.criterion(outputs_, targets, output_sizes, target_sizes).to(self._device)
+
+        # Average the loss by mini batch if version 2 of DeepSpeech is used
+        if self._version == 2:
+            loss = loss / inputs.size(0)
+
+        # Compute transcription
+        decoded_output, _ = self.decoder.decode(outputs, output_sizes)
+        decoded_output = [do[0] for do in decoded_output]
+        decoded_output = np.array(decoded_output)
+
+        # Rearrange to the original order
+        decoded_output_ = decoded_output.copy()
+        decoded_output[batch_idx] = decoded_output_
+
+        return loss, decoded_output
+
+    def _preprocess_transform_model_input(
+        self,
+        x: "torch.Tensor",
+        y: np.ndarray,
+        real_lengths: np.ndarray,
     ) -> Tuple["torch.Tensor", "torch.Tensor", "torch.Tensor", "torch.Tensor", List]:
         """
         Apply preprocessing and then transform the user input space into the model input space. This function is used
@@ -540,7 +678,7 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
 
         # Apply preprocessing
         x_batch = []
-        for i in range(len(x)):
+        for i, _ in enumerate(x):
             preprocessed_x_i, _ = self._apply_preprocessing(x=x[i], y=None, no_grad=False)
             x_batch.append(preprocessed_x_i)
 
@@ -548,7 +686,11 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
 
         # Transform the input space
         inputs, targets, input_rates, target_sizes, batch_idx = self._transform_model_input(
-            x=x, y=y, compute_gradient=False, tensor_input=True, real_lengths=real_lengths,
+            x=x,
+            y=y,
+            compute_gradient=False,
+            tensor_input=True,
+            real_lengths=real_lengths,
         )
 
         return inputs, targets, input_rates, target_sizes, batch_idx
@@ -582,30 +724,36 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         """
         import torch  # lgtm [py/repeated-import]
         import torchaudio
-
         from deepspeech_pytorch.loader.data_loader import _collate_fn
 
-        # These parameters are needed for the transformation
-        sample_rate = self._model.audio_conf.sample_rate
-        window_size = self._model.audio_conf.window_size
-        window_stride = self._model.audio_conf.window_stride
+        # Get parameters needed for the transformation
+        if self._version == 2:
+            window_name = self.model.audio_conf.window.value
+            sample_rate = self.model.audio_conf.sample_rate
+            window_size = self.model.audio_conf.window_size
+            window_stride = self.model.audio_conf.window_stride
+
+        else:
+            window_name = self.model.spect_cfg["window"].value
+            sample_rate = self.model.spect_cfg["sample_rate"]
+            window_size = self.model.spect_cfg["window_size"]
+            window_stride = self.model.spect_cfg["window_stride"]
 
         n_fft = int(sample_rate * window_size)
         hop_length = int(sample_rate * window_stride)
         win_length = n_fft
 
-        window = self._model.audio_conf.window.value
-
-        if window == "hamming":
-            window_fn = torch.hamming_window
-        elif window == "hann":
-            window_fn = torch.hann_window
-        elif window == "blackman":
-            window_fn = torch.blackman_window
-        elif window == "bartlett":
-            window_fn = torch.bartlett_window
-        else:
-            raise NotImplementedError("Spectrogram window %s not supported." % window)
+        # Get window for the transformation
+        if window_name == "hamming":  # pragma: no cover
+            window_fn = torch.hamming_window  # type: ignore
+        elif window_name == "hann":  # pragma: no cover
+            window_fn = torch.hann_window  # type: ignore
+        elif window_name == "blackman":  # pragma: no cover
+            window_fn = torch.blackman_window  # type: ignore
+        elif window_name == "bartlett":  # pragma: no cover
+            window_fn = torch.bartlett_window  # type: ignore
+        else:  # pragma: no cover
+            raise NotImplementedError("Spectrogram window %s not supported." % window_name)
 
         # Create a transformer to transform between the two spaces
         transformer = torchaudio.transforms.Spectrogram(
@@ -614,11 +762,11 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         transformer.to(self._device)
 
         # Create a label map
-        label_map = dict([(self._model.labels[i], i) for i in range(len(self._model.labels))])
+        label_map = {self._model.labels[i]: i for i in range(len(self._model.labels))}
 
         # We must process each sequence separately due to the diversity of their length
         batch = []
-        for i in range(len(x)):
+        for i, _ in enumerate(x):
             # First process the target
             if y is None:
                 target = []
@@ -626,7 +774,7 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
                 target = list(filter(None, [label_map.get(letter) for letter in list(y[i])]))
 
             # Push the sequence to device
-            if not tensor_input:
+            if isinstance(x, np.ndarray) and not tensor_input:
                 x[i] = x[i].astype(config.ART_NUMPY_DTYPE)
                 x[i] = torch.tensor(x[i]).to(self._device)
 
@@ -660,6 +808,26 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
 
         return inputs, targets, input_percentages, target_sizes, batch_idx
 
+    def to_training_mode(self) -> None:
+        """
+        Put the estimator in the training mode.
+        """
+        self.model.train()
+
+    @property
+    def sample_rate(self) -> int:
+        """
+        Get the sampling rate.
+
+        :return: The audio sampling rate.
+        """
+        if self._version == 2:
+            sample_rate = self.model.audio_conf.sample_rate
+        else:
+            sample_rate = self.model.spect_cfg["sample_rate"]
+
+        return sample_rate
+
     @property
     def input_shape(self) -> Tuple[int, ...]:
         """
@@ -687,10 +855,38 @@ class PyTorchDeepSpeech(SpeechRecognizerMixin, PyTorchEstimator):
         """
         return self._device
 
+    @property
+    def use_amp(self) -> bool:
+        """
+        Return a boolean indicating whether to use the automatic mixed precision tool.
+
+        :return: Whether to use the automatic mixed precision tool.
+        """
+        return self._use_amp  # type: ignore
+
+    @property
+    def optimizer(self) -> "torch.optim.Optimizer":
+        """
+        Return the optimizer.
+
+        :return: The optimizer.
+        """
+        return self._optimizer  # type: ignore
+
+    @property
+    def opt_level(self) -> str:
+        """
+        Return a string specifying a pure or mixed precision optimization level.
+
+        :return: A string specifying a pure or mixed precision optimization level. Possible
+                 values are `O0`, `O1`, `O2`, and `O3`.
+        """
+        return self._opt_level  # type: ignore
+
     def get_activations(
         self, x: np.ndarray, layer: Union[int, str], batch_size: int, framework: bool = False
     ) -> np.ndarray:
         raise NotImplementedError
 
-    def set_learning_phase(self, train: bool) -> None:
+    def compute_loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
         raise NotImplementedError
