@@ -95,12 +95,15 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         },
         "decoder": {
             "uri": (
-                "https://raw.githubusercontent.com/hesseltuinhof/lingvo/qin_patched_decoder/lingvo/tasks/asr/decoder.py"
+                "https://raw.githubusercontent.com/Trusted-AI/adversarial-robustness-toolbox/"
+                "4dabf5fcfb55502316ad48abbdc1a26033db1da5/contrib/lingvo-patched-decoder.py"
             ),
             "basename": "decoder_patched.py",
         },
     }
 
+    estimator_params = TensorFlowV2Estimator.estimator_params + ["random_seed", "sess"]
+
     def __init__(
         self,
         clip_values: Optional["CLIP_VALUES_TYPE"] = None,
@@ -140,21 +143,21 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
             preprocessing=preprocessing,
         )
         self.random_seed = random_seed
-        if self.postprocessing_defences is not None:
+        if self.postprocessing_defences is not None:  # pragma: no cover
             raise ValueError("This estimator does not support `postprocessing_defences`.")
 
         self._input_shape = None
 
         # check required TensorFlow version
-        if tf1.__version__ != "2.1.0":
+        if tf1.__version__ != "2.1.0":  # pragma: no cover
             raise AssertionError("The Lingvo estimator only supports TensorFlow 2.1.0.")
 
         # check required Python version
-        if sys.version_info[:2] != (3, 6):
+        if sys.version_info[:2] != (3, 6):  # pragma: no cover
             raise AssertionError("The Lingvo estimator only supports Python 3.6.")
 
         # check required Lingvo version
-        if pkg_resources.get_distribution("lingvo").version != "0.6.4":
+        if pkg_resources.get_distribution("lingvo").version != "0.6.4":  # pragma: no cover
             raise AssertionError("The Lingvo estimator only supports Lingvo 0.6.4")
 
         # disable eager execution as Lingvo uses tensorflow.compat.v1 API
@@ -170,9 +173,9 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         )
 
         # placeholders
-        self._x_padded = tf1.placeholder(tf1.float32, shape=[None, None], name="art_x_padded")
-        self._y_target = tf1.placeholder(tf1.string, name="art_y_target")
-        self._mask_frequency = tf1.placeholder(tf1.float32, shape=[None, None, 80], name="art_mask_frequency")
+        self._x_padded: "Tensor" = tf1.placeholder(tf1.float32, shape=[None, None], name="art_x_padded")
+        self._y_target: "Tensor" = tf1.placeholder(tf1.string, name="art_y_target")
+        self._mask_frequency: "Tensor" = tf1.placeholder(tf1.float32, shape=[None, None, 80], name="art_mask_frequency")
 
         # init Lingvo computation graph
         self._sess: "Session" = tf1.Session() if sess is None else sess
@@ -180,11 +183,13 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         self._model = model
         self._task = task
         self._cluster = cluster
-        self._metrics = None
+        self._metrics: Optional[Tuple[Union[Dict[str, "Tensor"], Dict[str, Tuple["Tensor", "Tensor"]]], ...]] = None
 
         # add prediction and loss gradient ops to graph
-        self._predict_batch_op = self._predict_batch(self._x_padded, self._y_target, self._mask_frequency)
-        self._loss_gradient_op = self._loss_gradient(self._x_padded, self._y_target, self._mask_frequency)
+        self._predict_batch_op: Dict[str, "Tensor"] = self._predict_batch(
+            self._x_padded, self._y_target, self._mask_frequency
+        )
+        self._loss_gradient_op: "Tensor" = self._loss_gradient(self._x_padded, self._y_target, self._mask_frequency)
 
     @property
     def input_shape(self) -> Tuple[int, ...]:
@@ -236,7 +241,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         import lingvo.tasks.asr.decoder as decoder
         import asr.decoder_patched as decoder_patched
 
-        decoder.AsrDecoderBase._ComputeMetrics = decoder_patched.AsrDecoderBase._ComputeMetrics
+        decoder.AsrDecoderBase._ComputeMetrics = decoder_patched.AsrDecoderBase._ComputeMetrics  # pylint: disable=W0212
 
         # check and download Lingvo ASR vocab
         # vocab_path = self._check_and_download_vocab()
@@ -250,7 +255,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         # register model params
         model_name = "asr.librispeech.Librispeech960Wpm"
         model_imports.ImportParams(model_name)
-        params = model_registry._ModelRegistryHelper.GetParams(model_name, "Test")
+        params = model_registry._ModelRegistryHelper.GetParams(model_name, "Test")  # pylint: disable=W0212
 
         # set random seed parameter
         if self.random_seed is not None:
@@ -277,9 +282,9 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
             "asr",
             "model",
         )
-        self._sess.run(tf1.global_variables_initializer())
+        self.sess.run(tf1.global_variables_initializer())
         saver = tf1.train.Saver([var for var in tf1.global_variables() if var.name.startswith("librispeech")])
-        saver.restore(self._sess, os.path.splitext(model_index_path)[0])
+        saver.restore(self.sess, os.path.splitext(model_index_path)[0])
 
         # set 'enable_asserts'-flag to False (Note: this flag ensures correct GPU support)
         tf1.flags.FLAGS.enable_asserts = False
@@ -298,7 +303,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         source_paddings = 1.0 - mask_frequency[:, :, 0]
 
         # prepare model input target, i.e. transcription target
-        target = self._task.input_generator.StringsToIds(y)
+        target = self._task.input_generator.StringsToIds(y)  # type: ignore
 
         # create decoder input
         decoder_inputs = NestedMap(
@@ -311,7 +316,8 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         decoder_inputs.tgt["weights"] = 1.0 - decoder_inputs.tgt["paddings"]
         return decoder_inputs
 
-    def _create_log_mel_features(self, x: "Tensor") -> "Tensor":
+    @staticmethod
+    def _create_log_mel_features(x: "Tensor") -> "Tensor":
         """Extract Log-Mel features from audio samples of shape (batch_size, max_length)."""
         from lingvo.core.py_utils import NestedMap
         import tensorflow.compat.v1 as tf1
@@ -348,7 +354,8 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         features = tf1.reshape(features, features_shape)
         return features
 
-    def _pad_audio_input(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    @staticmethod
+    def _pad_audio_input(x: np.ndarray) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
         """Apply padding to a batch of audio samples such that it has shape of (batch_size, max_length)."""
         max_length = max(map(len, x))
         batch_size = x.shape[0]
@@ -368,7 +375,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
             mask_frequency[i, : frequency_length[i], :] = 1
         return x_padded, x_mask, mask_frequency
 
-    def _predict_batch(self, x: "Tensor", y: "Tensor", mask_frequency: "Tensor") -> "Tensor":
+    def _predict_batch(self, x: "Tensor", y: "Tensor", mask_frequency: "Tensor") -> Dict[str, "Tensor"]:
         """Create prediction operation for a batch of padded inputs."""
         import tensorflow.compat.v1 as tf1
 
@@ -377,9 +384,9 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
 
         # call decoder
         if self._metrics is None:
-            with self._cluster, tf1.device(self._cluster.GetPlacer()):
-                self._metrics = self._task.FPropDefaultTheta(decoder_inputs)
-        predictions = self._task.Decode(decoder_inputs)
+            with self._cluster, tf1.device(self._cluster.GetPlacer()):  # type: ignore
+                self._metrics = self._task.FPropDefaultTheta(decoder_inputs)  # type: ignore
+        predictions = self._task.Decode(decoder_inputs)  # type: ignore
 
         return predictions
 
@@ -396,14 +403,14 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         :return: Array of predicted transcriptions of shape `(nb_samples)`. A possible example of a transcription
                  return is `np.array(['SIXTY ONE', 'HELLO'])`.
         """
-        if x[0].ndim != 1:
+        if x[0].ndim != 1:  # pragma: no cover
             raise ValueError(
                 "The LingvoASR estimator can only be used temporal data of type mono. Please remove any channel"
                 "dimension."
             )
         # if inputs have 32-bit floating point wav format, the preprocessing argument is required
-        is_normalized = max(map(max, np.abs(x))) <= 1.0
-        if is_normalized and self.preprocessing is None:
+        is_normalized = max(map(max, np.abs(x))) <= 1.0  # type: ignore
+        if is_normalized and self.preprocessing is None:  # pragma: no cover
             raise ValueError(
                 "The LingvoASR estimator requires input values in the range [-32768, 32767] or normalized input values"
                 " with correct preprocessing argument (mean=0, stddev=1/normalization_factor)."
@@ -429,7 +436,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
                 self._mask_frequency: mask_frequency,
             }
             # run prediction
-            y_batch = self._sess.run(self._predict_batch_op, feed_dict)
+            y_batch = self.sess.run(self._predict_batch_op, feed_dict)
 
             # extract and append transcription result
             y += y_batch["topk_decoded"][:, 0].tolist()
@@ -446,15 +453,17 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
 
         # call decoder
         if self._metrics is None:
-            with self._cluster, tf1.device(self._cluster.GetPlacer()):
-                self._metrics = self._task.FPropDefaultTheta(decoder_inputs)
+            with self._cluster, tf1.device(self._cluster.GetPlacer()):  # type: ignore
+                self._metrics = self._task.FPropDefaultTheta(decoder_inputs)  # type: ignore
 
         # compute loss gradient
         loss = tf1.get_collection("per_loss")[0]
         loss_gradient = tf1.gradients(loss, [x])[0]
         return loss_gradient
 
-    def loss_gradient(self, x: np.ndarray, y: np.ndarray, batch_mode: bool = False, **kwargs) -> np.ndarray:
+    def loss_gradient(  # pylint: disable=W0221
+        self, x: np.ndarray, y: np.ndarray, batch_mode: bool = False, **kwargs
+    ) -> np.ndarray:
         """
         Compute the gradient of the loss function w.r.t. `x`.
 
@@ -467,7 +476,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
         :return: Loss gradients of the same shape as `x`.
         """
         # if inputs have 32-bit floating point wav format, the preprocessing argument is required
-        is_normalized = max(map(max, np.abs(x))) <= 1.0
+        is_normalized = max(map(max, np.abs(x))) <= 1.0  # type: ignore
         if is_normalized and self.preprocessing is None:
             raise ValueError(
                 "The LingvoASR estimator requires input values in the range [-32768, 32767] or normalized input values"
@@ -504,7 +513,7 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
             self._y_target: y,
             self._mask_frequency: mask_frequency,
         }
-        gradients_padded = self._sess.run(self._loss_gradient_op, feed_dict)
+        gradients_padded = self.sess.run(self._loss_gradient_op, feed_dict)
 
         # undo padding, i.e. change gradients shape from (nb_samples, max_length) to (nb_samples)
         lengths = mask.sum(axis=1)
@@ -513,7 +522,9 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
             gradient = gradient_padded[:length]
             gradients.append(gradient)
 
-        return np.array(gradients, dtype=object)
+        # for ragged input, use object dtype
+        dtype = np.float32 if x.ndim != 1 else object
+        return np.array(gradients, dtype=dtype)
 
     def _loss_gradient_per_sequence(self, x: np.ndarray, y: np.ndarray) -> np.ndarray:
         """
@@ -536,15 +547,17 @@ class TensorFlowLingvoASR(SpeechRecognizerMixin, TensorFlowV2Estimator):
                 self._mask_frequency: np.expand_dims(mask_frequency_i[:frequency_length], 0),
             }
             # get loss gradient
-            gradient = self._sess.run(self._loss_gradient_op, feed_dict)
+            gradient = self.sess.run(self._loss_gradient_op, feed_dict)  # type: ignore
             gradients.append(np.squeeze(gradient))
 
-        return np.array(gradients, dtype=object)
-
-    def set_learning_phase(self, train: bool) -> None:
-        raise NotImplementedError
+        # for ragged input, use object dtype
+        dtype = np.float32 if x.ndim != 1 else object
+        return np.array(gradients, dtype=dtype)
 
     def get_activations(
         self, x: np.ndarray, layer: Union[int, str], batch_size: int, framework: bool = False
     ) -> np.ndarray:
         raise NotImplementedError
+
+    def compute_loss(self, x: np.ndarray, y: np.ndarray, **kwargs) -> np.ndarray:
+        raise NotImplementedError
