@@ -18,7 +18,7 @@
 """
 Module containing ART's exceptions.
 """
-from typing import List
+from typing import List, Tuple, Type, Union
 
 
 class EstimatorError(TypeError):
@@ -26,17 +26,25 @@ class EstimatorError(TypeError):
     Basic exception for errors raised by unexpected estimator types.
     """
 
-    def __init__(self, this_class, class_expected_list: List[str], classifier_given) -> None:
+    def __init__(self, this_class, class_expected_list: List[Union[Type, Tuple[Type]]], classifier_given) -> None:
+        super().__init__()
         self.this_class = this_class
         self.class_expected_list = class_expected_list
         self.classifier_given = classifier_given
 
         classes_expected_message = ""
         for idx, class_expected in enumerate(class_expected_list):
-            if idx == 0:
+            if idx != 0:
+                classes_expected_message += " and "
+            if isinstance(class_expected, type):
                 classes_expected_message += "{0}".format(class_expected)
             else:
-                classes_expected_message += " and {0}".format(class_expected)
+                classes_expected_message += "("
+                for or_idx, or_class in enumerate(class_expected):
+                    if or_idx != 0:
+                        classes_expected_message += " or "
+                    classes_expected_message += "{0}".format(or_class)
+                classes_expected_message += ")"
 
         self.message = (
             "{0} requires an estimator derived from {1}, "
