@@ -24,7 +24,7 @@ import logging
 from typing import Dict, List, Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
-from tqdm import trange
+from tqdm.auto import trange
 
 if TYPE_CHECKING:
     from art.estimators.classification.classifier import ClassifierDecisionTree
@@ -77,7 +77,7 @@ class Box:
                 lower_bound = max(self.intervals[key].lower_bound, value.lower_bound)
                 upper_bound = min(self.intervals[key].upper_bound, value.upper_bound)
 
-                if lower_bound >= upper_bound:
+                if lower_bound >= upper_bound:  # pragma: no cover
                     self.intervals.clear()
                     break
 
@@ -116,7 +116,12 @@ class LeafNode:
     """
 
     def __init__(
-        self, tree_id: Optional[int], class_label: int, node_id: Optional[int], box: Box, value: float,
+        self,
+        tree_id: Optional[int],
+        class_label: int,
+        node_id: Optional[int],
+        box: Box,
+        value: float,
     ) -> None:
         """
         Create a leaf node representation.
@@ -162,13 +167,15 @@ class RobustnessVerificationTreeModelsCliqueMethod:
     | Paper link: https://arxiv.org/abs/1906.03849
     """
 
-    def __init__(self, classifier: "ClassifierDecisionTree") -> None:
+    def __init__(self, classifier: "ClassifierDecisionTree", verbose: bool = True) -> None:
         """
         Create robustness verification for a decision-tree-based classifier.
 
         :param classifier: A trained decision-tree-based classifier.
+        :param verbose: Show progress bars.
         """
         self._classifier = classifier
+        self.verbose = verbose
         self._trees = self._classifier.get_trees()
 
     def verify(
@@ -203,7 +210,7 @@ class RobustnessVerificationTreeModelsCliqueMethod:
         num_samples: int = x.shape[0]
 
         # pylint: disable=R1702
-        pbar = trange(num_samples, desc="Decision tree verification")
+        pbar = trange(num_samples, desc="Decision tree verification", disable=not self.verbose)
         for i_sample in pbar:
 
             eps: float = eps_init
@@ -249,7 +256,7 @@ class RobustnessVerificationTreeModelsCliqueMethod:
                     eps /= 2.0
                 else:
                     if i_not_robust is None:
-                        if eps >= 1.0:
+                        if eps >= 1.0:  # pragma: no cover
                             logger.info("Abort binary search because eps increased above 1.0")
                             break
                         eps = min(eps * 2.0, 1.0)
@@ -261,7 +268,8 @@ class RobustnessVerificationTreeModelsCliqueMethod:
                 average_bound += clique_bound
             else:
                 logger.info(
-                    "point %s: WARNING! no robust eps found, verification bound is set as 0 !", i_sample,
+                    "point %s: WARNING! no robust eps found, verification bound is set as 0 !",
+                    i_sample,
                 )
 
         verified_error = 1.0 - num_initial_successes / num_samples
@@ -273,7 +281,10 @@ class RobustnessVerificationTreeModelsCliqueMethod:
         return average_bound, verified_error
 
     def _get_k_partite_clique(
-        self, accessible_leaves: List[List[LeafNode]], label: int, target_label: Optional[int],
+        self,
+        accessible_leaves: List[List[LeafNode]],
+        label: int,
+        target_label: Optional[int],
     ) -> Tuple[float, List]:
         """
         Find the K partite cliques among the accessible leaf nodes.
@@ -304,7 +315,10 @@ class RobustnessVerificationTreeModelsCliqueMethod:
                 cliques_old.append({"box": accessible_leaf.box, "value": new_leaf_value})
 
             # Loop over all all trees
-            for i_tree in range(start_tree + 1, min(len(accessible_leaves), start_tree + self.max_clique),):
+            for i_tree in range(
+                start_tree + 1,
+                min(len(accessible_leaves), start_tree + self.max_clique),
+            ):
                 cliques_new.clear()
                 # Loop over all existing cliques
                 for clique in cliques_old:
@@ -397,7 +411,10 @@ class RobustnessVerificationTreeModelsCliqueMethod:
             if interval.lower_bound < feature_value < interval.upper_bound:
                 distance = 0.0
             else:
-                difference = max(feature_value - interval.upper_bound, interval.lower_bound - feature_value,)
+                difference = max(
+                    feature_value - interval.upper_bound,
+                    interval.lower_bound - feature_value,
+                )
                 if norm == 0:
                     distance = 1.0
                 elif norm == np.inf:
@@ -443,7 +460,7 @@ class RobustnessVerificationTreeModelsCliqueMethod:
                     if leaf_node.box and distance <= eps:
                         leaves.append(leaf_node)
 
-                if not leaves:
+                if not leaves:  # pragma: no cover
                     raise ValueError("No accessible leaves found.")
 
                 accessible_leaves.append(leaves)
