@@ -18,98 +18,111 @@
 """
 Module providing convenience functions.
 """
+# pylint: disable=C0302
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from functools import wraps
-from inspect import signature
 import logging
 import math
 import os
 import shutil
 import sys
 import tarfile
-from typing import Callable, List, Optional, Tuple, Union, TYPE_CHECKING
 import warnings
 import zipfile
+from functools import wraps
+from inspect import signature
+from typing import TYPE_CHECKING, Callable, List, Optional, Tuple, Union
 
 import numpy as np
-from scipy.special import gammainc
 import six
-from tqdm import tqdm
+from scipy.special import gammainc  # pylint: disable=E0611
+from tqdm.auto import tqdm
 
 from art import config
 
+if TYPE_CHECKING:
+    import torch
+
 logger = logging.getLogger(__name__)
 
 
 # ------------------------------------------------------------------------------------------------- CONSTANTS AND TYPES
 
 
-DATASET_TYPE = Tuple[Tuple[np.ndarray, np.ndarray], Tuple[np.ndarray, np.ndarray], float, float]
-CLIP_VALUES_TYPE = Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]]
+DATASET_TYPE = Tuple[  # pylint: disable=C0103
+    Tuple[np.ndarray, np.ndarray], Tuple[np.ndarray, np.ndarray], float, float
+]
+CLIP_VALUES_TYPE = Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]]  # pylint: disable=C0103
 
 if TYPE_CHECKING:
-    # pylint: disable=R0401
-
+    # pylint: disable=R0401,C0412
     from art.defences.preprocessor.preprocessor import Preprocessor
 
-    PREPROCESSING_TYPE = Optional[
-        Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]], Preprocessor, Tuple[Preprocessor, ...]
+    PREPROCESSING_TYPE = Optional[  # pylint: disable=C0103
+        Union[
+            Tuple[Union[int, float, np.ndarray], Union[int, float, np.ndarray]], Preprocessor, Tuple[Preprocessor, ...]
+        ]
     ]
 
-    from art.estimators.classification.classifier import (
-        Classifier,
-        ClassifierLossGradients,
-        ClassifierClassLossGradients,
-        ClassifierNeuralNetwork,
-        ClassifierDecisionTree,
-    )
     from art.estimators.classification.blackbox import BlackBoxClassifier
     from art.estimators.classification.catboost import CatBoostARTClassifier
+    from art.estimators.classification.classifier import (
+        Classifier,
+        ClassifierClassLossGradients,
+        ClassifierDecisionTree,
+        ClassifierLossGradients,
+        ClassifierNeuralNetwork,
+    )
     from art.estimators.classification.detector_classifier import DetectorClassifier
     from art.estimators.classification.ensemble import EnsembleClassifier
     from art.estimators.classification.GPy import GPyGaussianProcessClassifier
     from art.estimators.classification.keras import KerasClassifier
+    from art.experimental.estimators.classification.jax import JaxClassifier
     from art.estimators.classification.lightgbm import LightGBMClassifier
     from art.estimators.classification.mxnet import MXClassifier
     from art.estimators.classification.pytorch import PyTorchClassifier
+    from art.estimators.classification.query_efficient_bb import QueryEfficientGradientEstimationClassifier
     from art.estimators.classification.scikitlearn import (
+        ScikitlearnAdaBoostClassifier,
+        ScikitlearnBaggingClassifier,
         ScikitlearnClassifier,
         ScikitlearnDecisionTreeClassifier,
         ScikitlearnDecisionTreeRegressor,
         ScikitlearnExtraTreeClassifier,
-        ScikitlearnAdaBoostClassifier,
-        ScikitlearnBaggingClassifier,
         ScikitlearnExtraTreesClassifier,
         ScikitlearnGradientBoostingClassifier,
-        ScikitlearnRandomForestClassifier,
         ScikitlearnLogisticRegression,
+        ScikitlearnRandomForestClassifier,
         ScikitlearnSVC,
     )
     from art.estimators.classification.tensorflow import TensorFlowClassifier, TensorFlowV2Classifier
     from art.estimators.classification.xgboost import XGBoostClassifier
-
     from art.estimators.object_detection.object_detector import ObjectDetector
+    from art.estimators.object_detection.python_object_detector import PyTorchObjectDetector
     from art.estimators.object_detection.pytorch_faster_rcnn import PyTorchFasterRCNN
     from art.estimators.object_detection.tensorflow_faster_rcnn import TensorFlowFasterRCNN
-
+    from art.estimators.pytorch import PyTorchEstimator
+    from art.estimators.regression.scikitlearn import ScikitlearnRegressor
     from art.estimators.speech_recognition.pytorch_deep_speech import PyTorchDeepSpeech
     from art.estimators.speech_recognition.tensorflow_lingvo import TensorFlowLingvoASR
+    from art.estimators.tensorflow import TensorFlowV2Estimator
 
-    CLASSIFIER_LOSS_GRADIENTS_TYPE = Union[
+    CLASSIFIER_LOSS_GRADIENTS_TYPE = Union[  # pylint: disable=C0103
         ClassifierLossGradients,
         EnsembleClassifier,
         GPyGaussianProcessClassifier,
         KerasClassifier,
+        JaxClassifier,
         MXClassifier,
         PyTorchClassifier,
         ScikitlearnLogisticRegression,
         ScikitlearnSVC,
         TensorFlowClassifier,
         TensorFlowV2Classifier,
+        QueryEfficientGradientEstimationClassifier,
     ]
 
-    CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE = Union[
+    CLASSIFIER_CLASS_LOSS_GRADIENTS_TYPE = Union[  # pylint: disable=C0103
         ClassifierClassLossGradients,
         EnsembleClassifier,
         GPyGaussianProcessClassifier,
@@ -122,7 +135,7 @@ if TYPE_CHECKING:
         TensorFlowV2Classifier,
     ]
 
-    CLASSIFIER_NEURALNETWORK_TYPE = Union[
+    CLASSIFIER_NEURALNETWORK_TYPE = Union[  # pylint: disable=C0103
         ClassifierNeuralNetwork,
         DetectorClassifier,
         EnsembleClassifier,
@@ -133,18 +146,17 @@ if TYPE_CHECKING:
         TensorFlowV2Classifier,
     ]
 
-    CLASSIFIER_DECISION_TREE_TYPE = Union[
+    CLASSIFIER_DECISION_TREE_TYPE = Union[  # pylint: disable=C0103
         ClassifierDecisionTree,
         LightGBMClassifier,
         ScikitlearnDecisionTreeClassifier,
-        ScikitlearnDecisionTreeRegressor,
         ScikitlearnExtraTreesClassifier,
         ScikitlearnGradientBoostingClassifier,
         ScikitlearnRandomForestClassifier,
         XGBoostClassifier,
     ]
 
-    CLASSIFIER_TYPE = Union[
+    CLASSIFIER_TYPE = Union[  # pylint: disable=C0103
         Classifier,
         BlackBoxClassifier,
         CatBoostARTClassifier,
@@ -152,12 +164,12 @@ if TYPE_CHECKING:
         EnsembleClassifier,
         GPyGaussianProcessClassifier,
         KerasClassifier,
+        JaxClassifier,
         LightGBMClassifier,
         MXClassifier,
         PyTorchClassifier,
         ScikitlearnClassifier,
         ScikitlearnDecisionTreeClassifier,
-        ScikitlearnDecisionTreeRegressor,
         ScikitlearnExtraTreeClassifier,
         ScikitlearnAdaBoostClassifier,
         ScikitlearnBaggingClassifier,
@@ -172,12 +184,35 @@ if TYPE_CHECKING:
         CLASSIFIER_NEURALNETWORK_TYPE,
     ]
 
-    OBJECT_DETECTOR_TYPE = Union[
-        ObjectDetector, PyTorchFasterRCNN, TensorFlowFasterRCNN,
+    REGRESSOR_TYPE = Union[ScikitlearnRegressor, ScikitlearnDecisionTreeRegressor]  # pylint: disable=C0103
+
+    OBJECT_DETECTOR_TYPE = Union[  # pylint: disable=C0103
+        ObjectDetector,
+        PyTorchObjectDetector,
+        PyTorchFasterRCNN,
+        TensorFlowFasterRCNN,
     ]
 
-    SPEECH_RECOGNIZER_TYPE = Union[
-        PyTorchDeepSpeech, TensorFlowLingvoASR,
+    SPEECH_RECOGNIZER_TYPE = Union[  # pylint: disable=C0103
+        PyTorchDeepSpeech,
+        TensorFlowLingvoASR,
+    ]
+
+    PYTORCH_ESTIMATOR_TYPE = Union[  # pylint: disable=C0103
+        PyTorchClassifier,
+        PyTorchDeepSpeech,
+        PyTorchEstimator,
+        PyTorchObjectDetector,
+        PyTorchFasterRCNN,
+    ]
+
+    TENSORFLOWV2_ESTIMATOR_TYPE = Union[  # pylint: disable=C0103
+        TensorFlowV2Classifier,
+        TensorFlowV2Estimator,
+    ]
+
+    ESTIMATOR_TYPE = Union[  # pylint: disable=C0103
+        CLASSIFIER_TYPE, REGRESSOR_TYPE, OBJECT_DETECTOR_TYPE, SPEECH_RECOGNIZER_TYPE
     ]
 
 # --------------------------------------------------------------------------------------------------------- DEPRECATION
@@ -229,7 +264,9 @@ def deprecated(end_version: str, *, reason: str = "", replaced_by: str = "") ->
         def wrapper(*args, **kwargs):
             warnings.simplefilter("always", category=DeprecationWarning)
             warnings.warn(
-                deprecated_msg + replaced_msg + reason_msg, category=DeprecationWarning, stacklevel=2,
+                deprecated_msg + replaced_msg + reason_msg,
+                category=DeprecationWarning,
+                stacklevel=2,
             )
             warnings.simplefilter("default", category=DeprecationWarning)
             return function(*args, **kwargs)
@@ -323,7 +360,8 @@ def projection(values: np.ndarray, eps: Union[int, float, np.ndarray], norm_p: U
             raise NotImplementedError("The parameter `eps` of type `np.ndarray` is not supported to use with norm 1.")
 
         values_tmp = values_tmp * np.expand_dims(
-            np.minimum(1.0, eps / (np.linalg.norm(values_tmp, axis=1, ord=1) + tol)), axis=1,
+            np.minimum(1.0, eps / (np.linalg.norm(values_tmp, axis=1, ord=1) + tol)),
+            axis=1,
         )
 
     elif norm_p in [np.inf, "inf"]:
@@ -344,7 +382,10 @@ def projection(values: np.ndarray, eps: Union[int, float, np.ndarray], norm_p: U
 
 
 def random_sphere(
-    nb_points: int, nb_dims: int, radius: Union[int, float, np.ndarray], norm: Union[int, float, str],
+    nb_points: int,
+    nb_dims: int,
+    radius: Union[int, float, np.ndarray],
+    norm: Union[int, float, str],
 ) -> np.ndarray:
     """
     Generate randomly `m x n`-dimension points with radius `radius` and centered around 0.
@@ -414,7 +455,9 @@ def original_to_tanh(
 
 
 def tanh_to_original(
-    x_tanh: np.ndarray, clip_min: Union[float, np.ndarray], clip_max: Union[float, np.ndarray],
+    x_tanh: np.ndarray,
+    clip_min: Union[float, np.ndarray],
+    clip_max: Union[float, np.ndarray],
 ) -> np.ndarray:
     """
     Transform input from tanh to original space.
@@ -438,7 +481,7 @@ def to_categorical(labels: Union[np.ndarray, List[float]], nb_classes: Optional[
     :param nb_classes: The number of classes (possible labels).
     :return: A binary matrix representation of `y` in the shape `(nb_samples, nb_classes)`.
     """
-    labels = np.array(labels, dtype=np.int32)
+    labels = np.array(labels, dtype=int)
     if nb_classes is None:
         nb_classes = np.max(labels) + 1
     categorical = np.zeros((labels.shape[0], nb_classes), dtype=np.float32)
@@ -446,15 +489,13 @@ def to_categorical(labels: Union[np.ndarray, List[float]], nb_classes: Optional[
     return categorical
 
 
-def float_to_categorical(labels, nb_classes=None):
+def float_to_categorical(labels: np.ndarray, nb_classes: Optional[int] = None):
     """
     Convert an array of floating point labels to binary class matrix.
 
     :param labels: An array of integer labels of shape `(nb_samples,)`
-    :type labels: `np.ndarray`
     :param nb_classes: The number of classes (possible labels)
-    :type nb_classes: `int`
-    :return: A binary matrix representation of `y` in the shape `(nb_samples, nb_classes)`
+    :return: A binary matrix representation of `labels` in the shape `(nb_samples, nb_classes)`
     :rtype: `np.ndarray`
     """
     labels = np.array(labels)
@@ -462,12 +503,29 @@ def float_to_categorical(labels, nb_classes=None):
     unique.sort()
     indexes = [np.where(unique == value)[0] for value in labels]
     if nb_classes is None:
-        nb_classes = len(unique) + 1
+        nb_classes = len(unique)
     categorical = np.zeros((labels.shape[0], nb_classes), dtype=np.float32)
     categorical[np.arange(labels.shape[0]), np.squeeze(indexes)] = 1
     return categorical
 
 
+def floats_to_one_hot(labels: np.ndarray):
+    """
+    Convert a 2D-array of floating point labels to binary class matrix.
+
+    :param labels: A 2D-array of floating point labels of shape `(nb_samples, nb_classes)`
+    :return: A binary matrix representation of `labels` in the shape `(nb_samples, nb_classes)`
+    :rtype: `np.ndarray`
+    """
+    labels = np.array(labels)
+    for feature in labels.T:  # pylint: disable=E1133
+        unique = np.unique(feature)
+        unique.sort()
+        for index, value in enumerate(unique):
+            feature[feature == value] = index
+    return labels.astype(np.float32)
+
+
 def check_and_transform_label_format(
     labels: np.ndarray, nb_classes: Optional[int] = None, return_one_hot: bool = True
 ) -> np.ndarray:
@@ -480,16 +538,27 @@ def check_and_transform_label_format(
     :return: Labels with shape `(nb_samples, nb_classes)` (one-hot) or `(nb_samples,)` (index).
     """
     if labels is not None:
-        if len(labels.shape) == 2 and labels.shape[1] > 1:
+        if len(labels.shape) == 2 and labels.shape[1] > 1:  # multi-class, one-hot encoded
             if not return_one_hot:
                 labels = np.argmax(labels, axis=1)
-        elif len(labels.shape) == 2 and labels.shape[1] == 1:
-            labels = np.squeeze(labels)
+                labels = np.expand_dims(labels, axis=1)
+        elif (
+            len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes > 2
+        ):  # multi-class, index labels
             if return_one_hot:
                 labels = to_categorical(labels, nb_classes)
-        elif len(labels.shape) == 1:
+            else:
+                labels = np.expand_dims(labels, axis=1)
+        elif (
+            len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes == 2
+        ):  # binary, index labels
             if return_one_hot:
                 labels = to_categorical(labels, nb_classes)
+        elif len(labels.shape) == 1:  # index labels
+            if return_one_hot:
+                labels = to_categorical(labels, nb_classes)
+            else:
+                labels = np.expand_dims(labels, axis=1)
         else:
             raise ValueError(
                 "Shape of labels not recognised."
@@ -545,7 +614,10 @@ def second_most_likely_class(x: np.ndarray, classifier: "CLASSIFIER_TYPE") -> np
     :param classifier: The classifier used for computing predictions.
     :return: Second most likely class predicted by `classifier` for sample `x` in one-hot encoding.
     """
-    return to_categorical(np.argpartition(classifier.predict(x), -2, axis=1)[:, -2], nb_classes=classifier.nb_classes,)
+    return to_categorical(
+        np.argpartition(classifier.predict(x), -2, axis=1)[:, -2],
+        nb_classes=classifier.nb_classes,
+    )
 
 
 def get_label_conf(y_vec: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
@@ -568,7 +640,10 @@ def get_labels_np_array(preds: np.ndarray) -> np.ndarray:
     :param preds: Array of class confidences, nb of instances as first dimension.
     :return: Labels.
     """
-    preds_max = np.amax(preds, axis=1, keepdims=True)
+    if len(preds.shape) >= 2:
+        preds_max = np.amax(preds, axis=1, keepdims=True)
+    else:
+        preds_max = np.round(preds)
     y = preds == preds_max
     y = y.astype(np.uint8)
     return y
@@ -594,11 +669,19 @@ def compute_success_array(
     :param batch_size: Batch size.
     :return: Percentage of successful adversarial samples.
     """
-    adv_preds = np.argmax(classifier.predict(x_adv, batch_size=batch_size), axis=1)
+    adv_preds = classifier.predict(x_adv, batch_size=batch_size)
+    if len(adv_preds.shape) >= 2:
+        adv_preds = np.argmax(adv_preds, axis=1)
+    else:
+        adv_preds = np.round(adv_preds)
     if targeted:
         attack_success = adv_preds == np.argmax(labels, axis=1)
     else:
-        preds = np.argmax(classifier.predict(x_clean, batch_size=batch_size), axis=1)
+        preds = classifier.predict(x_clean, batch_size=batch_size)
+        if len(preds.shape) >= 2:
+            preds = np.argmax(preds, axis=1)
+        else:
+            preds = np.round(preds)
         attack_success = adv_preds != preds
 
     return attack_success
@@ -655,7 +738,9 @@ def compute_accuracy(preds: np.ndarray, labels: np.ndarray, abstain: bool = True
 # -------------------------------------------------------------------------------------------------- DATASET OPERATIONS
 
 
-def load_cifar10(raw: bool = False,) -> DATASET_TYPE:
+def load_cifar10(
+    raw: bool = False,
+) -> DATASET_TYPE:
     """
     Loads CIFAR10 dataset from config.CIFAR10_PATH or downloads it if necessary.
 
@@ -689,7 +774,7 @@ def load_cifar10(raw: bool = False,) -> DATASET_TYPE:
         "cifar-10-batches-py",
         extract=True,
         path=config.ART_DATA_PATH,
-        url="http://www.cs.toronto.edu/~kriz/cifar-10-python.tar.gz",
+        url="https://www.cs.toronto.edu/~kriz/cifar-10-python.tar.gz",
     )
 
     num_train_samples = 50000
@@ -721,14 +806,20 @@ def load_cifar10(raw: bool = False,) -> DATASET_TYPE:
     return (x_train, y_train), (x_test, y_test), min_, max_
 
 
-def load_mnist(raw: bool = False,) -> DATASET_TYPE:
+def load_mnist(
+    raw: bool = False,
+) -> DATASET_TYPE:
     """
     Loads MNIST dataset from `config.ART_DATA_PATH` or downloads it if necessary.
 
     :param raw: `True` if no preprocessing should be applied to the data. Otherwise, data is normalized to 1.
     :return: `(x_train, y_train), (x_test, y_test), min, max`.
     """
-    path = get_file("mnist.npz", path=config.ART_DATA_PATH, url="https://s3.amazonaws.com/img-datasets/mnist.npz",)
+    path = get_file(
+        "mnist.npz",
+        path=config.ART_DATA_PATH,
+        url="https://s3.amazonaws.com/img-datasets/mnist.npz",
+    )
 
     dict_mnist = np.load(path)
     x_train = dict_mnist["x_train"]
@@ -800,35 +891,45 @@ def load_iris(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
     :param test_set: Proportion of the data to use as validation split. The value should be between 0 and 1.
     :return: Entire dataset and labels.
     """
-    # Download data if needed
-    path = get_file(
-        "iris.data",
-        path=config.ART_DATA_PATH,
-        extract=False,
-        url="https://archive.ics.uci.edu/ml/machine-learning-databases/iris/iris.data",
-    )
+    from sklearn.datasets import load_iris as load_iris_sk
 
-    data = np.loadtxt(path, delimiter=",", usecols=(0, 1, 2, 3), dtype=config.ART_NUMPY_DTYPE)
-    labels = np.loadtxt(path, delimiter=",", usecols=4, dtype=str)
-
-    # Preprocess
+    iris = load_iris_sk()
+    data = iris.data
     if not raw:
-        label_map = {"Iris-setosa": 0, "Iris-versicolor": 1, "Iris-virginica": 2}
-        labels = np.array([label_map[labels[i]] for i in range(labels.size)], dtype=np.int32)
-        data, labels = preprocess(data, labels, nb_classes=3)
+        data /= np.amax(data)
+    labels = to_categorical(iris.target, 3)
+
     min_, max_ = np.amin(data), np.amax(data)
 
     # Split training and test sets
     split_index = int((1 - test_set) * len(data) / 3)
     x_train = np.vstack((data[:split_index], data[50 : 50 + split_index], data[100 : 100 + split_index]))
-    y_train = np.vstack((labels[:split_index], labels[50 : 50 + split_index], labels[100 : 100 + split_index],))
+    y_train = np.vstack(
+        (
+            labels[:split_index],
+            labels[50 : 50 + split_index],
+            labels[100 : 100 + split_index],
+        )
+    )
 
     if split_index >= 49:
         x_test, y_test = None, None
     else:
 
-        x_test = np.vstack((data[split_index:50], data[50 + split_index : 100], data[100 + split_index :],))
-        y_test = np.vstack((labels[split_index:50], labels[50 + split_index : 100], labels[100 + split_index :],))
+        x_test = np.vstack(
+            (
+                data[split_index:50],
+                data[50 + split_index : 100],
+                data[100 + split_index :],
+            )
+        ).astype(np.float32)
+        y_test = np.vstack(
+            (
+                labels[split_index:50],
+                labels[50 + split_index : 100],
+                labels[100 + split_index :],
+            )
+        )
         assert len(x_train) + len(x_test) == 150
 
         # Shuffle test set
@@ -837,25 +938,59 @@ def load_iris(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
 
     # Shuffle training set
     random_indices = np.random.permutation(len(y_train))
-    x_train, y_train = x_train[random_indices], y_train[random_indices]
+    x_train, y_train = x_train[random_indices].astype(np.float32), y_train[random_indices]
 
     return (x_train, y_train), (x_test, y_test), min_, max_
 
 
-def load_nursery(raw: bool = False, test_set: float = 0.2, transform_social: bool = False) -> DATASET_TYPE:
+def load_diabetes(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
+    """
+    Loads the Diabetes Regression dataset from sklearn.
+
+    :param raw: `True` if no preprocessing should be applied to the data. Otherwise, data is normalized to 1.
+    :param test_set: Proportion of the data to use as validation split. The value should be between 0 and 1.
+    :return: Entire dataset and labels.
+    """
+    from sklearn.datasets import load_diabetes as load_diabetes_sk
+
+    diabetes = load_diabetes_sk()
+    data = diabetes.data
+    if not raw:
+        data /= np.amax(data, axis=0)
+    targets = diabetes.target
+
+    min_, max_ = np.amin(data), np.amax(data)
+
+    # Shuffle data set
+    random_indices = np.random.permutation(len(data))
+    data, targets = data[random_indices], targets[random_indices]
+
+    # Split training and test sets
+    split_index = int((1 - test_set) * len(data))
+    x_train = data[:split_index]
+    y_train = targets[:split_index]
+    x_test = data[split_index:]
+    y_test = targets[split_index:]
+
+    return (x_train, y_train), (x_test, y_test), min_, max_
+
+
+def load_nursery(
+    raw: bool = False, scaled: bool = True, test_set: float = 0.2, transform_social: bool = False
+) -> DATASET_TYPE:
     """
     Loads the UCI Nursery dataset from `config.ART_DATA_PATH` or downloads it if necessary.
 
     :param raw: `True` if no preprocessing should be applied to the data. Otherwise, categorical data is one-hot
-                encoded and data is scaled using sklearn's StandardScaler.
+                encoded and data is scaled using sklearn's StandardScaler according to the value of `scaled`.
+    :param scaled: `True` if data should be scaled.
     :param test_set: Proportion of the data to use as validation split. The value should be between 0 and 1.
     :param transform_social: If `True`, transforms the social feature to be binary for the purpose of attribute
                              inference. This is done by assigning the original value 'problematic' the new value 1, and
                              the other original values are assigned the new value 0.
-    :return: Entire dataset and labels.
+    :return: Entire dataset and labels as numpy array.
     """
     import pandas as pd
-    import sklearn.model_selection
     import sklearn.preprocessing
 
     # Download data if needed
@@ -867,9 +1002,9 @@ def load_nursery(raw: bool = False, test_set: float = 0.2, transform_social: boo
     )
 
     # load data
-    features = ["parents", "has_nurs", "form", "children", "housing", "finance", "social", "health", "label"]
+    features_names = ["parents", "has_nurs", "form", "children", "housing", "finance", "social", "health", "label"]
     categorical_features = ["parents", "has_nurs", "form", "housing", "finance", "social", "health"]
-    data = pd.read_csv(path, sep=",", names=features, engine="python")
+    data = pd.read_csv(path, sep=",", names=features_names, engine="python")
     # remove rows with missing label or too sparse label
     data = data.dropna(subset=["label"])
     data.drop(data.loc[data["label"] == "recommend"].index, axis=0, inplace=True)
@@ -884,14 +1019,13 @@ def load_nursery(raw: bool = False, test_set: float = 0.2, transform_social: boo
     def modify_label(value):  # 5 classes
         if value == "not_recom":
             return 0
-        elif value == "very_recom":
+        if value == "very_recom":
             return 1
-        elif value == "priority":
+        if value == "priority":
             return 2
-        elif value == "spec_prior":
+        if value == "spec_prior":
             return 3
-        else:
-            raise Exception("Bad label value: %s" % value)
+        raise Exception("Bad label value: %s" % value)
 
     data["label"] = data["label"].apply(modify_label)
     data["children"] = data["children"].apply(lambda x: 4 if x == "more" else x)
@@ -901,8 +1035,7 @@ def load_nursery(raw: bool = False, test_set: float = 0.2, transform_social: boo
         def modify_social(value):
             if value == "problematic":
                 return 1
-            else:
-                return 0
+            return 0
 
         data["social"] = data["social"].apply(modify_social)
         categorical_features.remove("social")
@@ -920,34 +1053,44 @@ def load_nursery(raw: bool = False, test_set: float = 0.2, transform_social: boo
         data = data.drop(features_to_remove, axis=1)
 
         # normalize data
-        label = data.loc[:, "label"]
-        features = data.drop(["label"], axis=1)
-        scaler = sklearn.preprocessing.StandardScaler()
-        scaler.fit(features)
-        scaled_features = pd.DataFrame(scaler.transform(features), columns=features.columns)
-
-        data = pd.concat([label, scaled_features], axis=1, join="inner")
+        if scaled:
+            label = data.loc[:, "label"]
+            features = data.drop(["label"], axis=1)
+            scaler = sklearn.preprocessing.StandardScaler()
+            scaler.fit(features)
+            scaled_features = pd.DataFrame(scaler.transform(features), columns=features.columns)
+            data = pd.concat([label, scaled_features], axis=1, join="inner")
 
     features = data.drop(["label"], axis=1)
-    # print(features.columns)
-    min_, max_ = np.amin(features.to_numpy()), np.amax(features.to_numpy())
+    if raw:
+        numeric_features = features.drop(categorical_features, axis=1).to_numpy().astype(np.int32)
+        min_, max_ = np.amin(numeric_features), np.amax(numeric_features)
+    else:
+        min_, max_ = np.amin(features.to_numpy().astype(np.float64)), np.amax(features.to_numpy().astype(np.float64))
 
     # Split training and test sets
     stratified = sklearn.model_selection.StratifiedShuffleSplit(n_splits=1, test_size=test_set, random_state=18)
-    for train_set, test_set in stratified.split(data, data["label"]):
-        train = data.iloc[train_set]
-        test = data.iloc[test_set]
+    for train_set_i, test_set_i in stratified.split(data, data["label"]):
+        train = data.iloc[train_set_i]
+        test = data.iloc[test_set_i]
     x_train = train.drop(["label"], axis=1).to_numpy()
     y_train = train.loc[:, "label"].to_numpy()
     x_test = test.drop(["label"], axis=1).to_numpy()
     y_test = test.loc[:, "label"].to_numpy()
 
+    if not raw and not scaled:
+        x_train = x_train.astype(np.float64)
+        x_test = x_test.astype(np.float64)
+
     return (x_train, y_train), (x_test, y_test), min_, max_
 
 
-def load_dataset(name: str,) -> DATASET_TYPE:
+def load_dataset(
+    name: str,
+) -> DATASET_TYPE:
     """
-    Loads or downloads the dataset corresponding to `name`. Options are: `mnist`, `cifar10` and `stl10`.
+    Loads or downloads the dataset corresponding to `name`. Options are: `mnist`, `cifar10`, `stl10`, `iris`, `nursery`
+    and `diabetes`.
 
     :param name: Name of the dataset.
     :return: The dataset separated in training and test sets as `(x_train, y_train), (x_test, y_test), min, max`.
@@ -963,19 +1106,21 @@ def load_dataset(name: str,) -> DATASET_TYPE:
         return load_iris()
     if "nursery" in name:
         return load_nursery()
+    if "diabetes" in name:
+        return load_diabetes()
 
     raise NotImplementedError("There is no loader for dataset '{}'.".format(name))
 
 
 def _extract(full_path: str, path: str) -> bool:
     archive: Union[zipfile.ZipFile, tarfile.TarFile]
-    if full_path.endswith("tar"):
+    if full_path.endswith("tar"):  # pragma: no cover
         if tarfile.is_tarfile(full_path):
             archive = tarfile.open(full_path, "r:")
-    elif full_path.endswith("tar.gz"):
+    elif full_path.endswith("tar.gz"):  # pragma: no cover
         if tarfile.is_tarfile(full_path):
             archive = tarfile.open(full_path, "r:gz")
-    elif full_path.endswith("zip"):
+    elif full_path.endswith("zip"):  # pragma: no cover
         if zipfile.is_zipfile(full_path):
             archive = zipfile.ZipFile(full_path)
         else:
@@ -985,7 +1130,7 @@ def _extract(full_path: str, path: str) -> bool:
 
     try:
         archive.extractall(path)
-    except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
+    except (tarfile.TarError, RuntimeError, KeyboardInterrupt):  # pragma: no cover
         if os.path.exists(path):
             if os.path.isfile(path):
                 os.remove(path)
@@ -1009,8 +1154,6 @@ def get_file(filename: str, url: str, path: Optional[str] = None, extract: bool
     :return: Path to the downloaded file.
     """
     if path is None:
-        from art import config
-
         path_ = os.path.expanduser(config.ART_DATA_PATH)
     else:
         path_ = os.path.expanduser(path)
@@ -1041,10 +1184,10 @@ def get_file(filename: str, url: str, path: Optional[str] = None, extract: bool
                 # [SSL: CERTIFICATE_VERIFY_FAILED] certificate verify failed (_ssl.c:847)
                 import ssl
 
-                ssl._create_default_https_context = ssl._create_unverified_context
+                ssl._create_default_https_context = ssl._create_unverified_context  # pylint: disable=W0212
 
                 if verbose:
-                    with tqdm() as t:
+                    with tqdm() as t_bar:
                         last_block = [0]
 
                         def progress_bar(blocks: int = 1, block_size: int = 1, total_size: Optional[int] = None):
@@ -1054,8 +1197,8 @@ def get_file(filename: str, url: str, path: Optional[str] = None, extract: bool
                             :param total_size: Total size (in tqdm units). If [default: None] or -1, remains unchanged.
                             """
                             if total_size not in (None, -1):
-                                t.total = total_size
-                            displayed = t.update((blocks - last_block[0]) * block_size)
+                                t_bar.total = total_size
+                            displayed = t_bar.update((blocks - last_block[0]) * block_size)
                             last_block[0] = blocks
                             return displayed
 
@@ -1063,11 +1206,11 @@ def get_file(filename: str, url: str, path: Optional[str] = None, extract: bool
                 else:
                     urlretrieve(url, full_path)
 
-            except HTTPError as exception:
+            except HTTPError as exception:  # pragma: no cover
                 raise Exception(error_msg.format(url, exception.code, exception.msg)) from HTTPError  # type: ignore
-            except URLError as exception:
+            except URLError as exception:  # pragma: no cover
                 raise Exception(error_msg.format(url, exception.errno, exception.reason)) from HTTPError
-        except (Exception, KeyboardInterrupt):
+        except (Exception, KeyboardInterrupt):  # pragma: no cover
             if os.path.exists(full_path):
                 os.remove(full_path)
             raise
@@ -1112,7 +1255,10 @@ def clip_and_round(x: np.ndarray, clip_values: Optional["CLIP_VALUES_TYPE"], rou
 
 
 def preprocess(
-    x: np.ndarray, y: np.ndarray, nb_classes: int = 10, clip_values: Optional["CLIP_VALUES_TYPE"] = None,
+    x: np.ndarray,
+    y: np.ndarray,
+    nb_classes: int = 10,
+    clip_values: Optional["CLIP_VALUES_TYPE"] = None,
 ) -> Tuple[np.ndarray, np.ndarray]:
     """
     Scales `x` to [0, 1] and converts `y` to class categorical confidences.
@@ -1147,7 +1293,7 @@ def segment_by_class(data: Union[np.ndarray, List[int]], classes: np.ndarray, nu
     """
     by_class: List[List[int]] = [[] for _ in range(num_classes)]
     for indx, feature in enumerate(classes):
-        if num_classes > 2:
+        if len(classes.shape) == 2 and classes.shape[1] > 1:
             assigned = np.argmax(feature)
         else:
             assigned = int(feature)
@@ -1179,8 +1325,7 @@ def performance_diff(
     :return: The difference in performance performance(model1) - performance(model2).
     :raises `ValueError`: If an unsupported performance function is requested.
     """
-    from sklearn.metrics import accuracy_score
-    from sklearn.metrics import f1_score
+    from sklearn.metrics import accuracy_score, f1_score
 
     model1_labels = model1.predict(test_data)
     model2_labels = model2.predict(test_data)
@@ -1228,10 +1373,44 @@ def pad_sequence_input(x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
     max_length = max(map(len, x))
     batch_size = x.shape[0]
 
-    x_padded = np.zeros((batch_size, max_length))
+    # note: use dtype of inner elements
+    x_padded = np.zeros((batch_size, max_length), dtype=x[0].dtype)
     x_mask = np.zeros((batch_size, max_length), dtype=bool)
 
     for i, x_i in enumerate(x):
         x_padded[i, : len(x_i)] = x_i
         x_mask[i, : len(x_i)] = 1
     return x_padded, x_mask
+
+
+# -------------------------------------------------------------------------------------------------------- CUDA SUPPORT
+
+
+def to_cuda(x: "torch.Tensor") -> "torch.Tensor":
+    """
+    Move the tensor from the CPU to the GPU if a GPU is available.
+
+    :param x: CPU Tensor to move to GPU if available.
+    :return: The CPU Tensor moved to a GPU Tensor.
+    """
+    from torch.cuda import is_available
+
+    use_cuda = is_available()
+    if use_cuda:  # pragma: no cover
+        x = x.cuda()
+    return x
+
+
+def from_cuda(x: "torch.Tensor") -> "torch.Tensor":
+    """
+    Move the tensor from the GPU to the CPU if a GPU is available.
+
+    :param x: GPU Tensor to move to CPU if available.
+    :return: The GPU Tensor moved to a CPU Tensor.
+    """
+    from torch.cuda import is_available
+
+    use_cuda = is_available()
+    if use_cuda:  # pragma: no cover
+        x = x.cpu()
+    return x
