@@ -41,13 +41,19 @@ def fix_get_mnist_subset(get_mnist_dataset):
     yield x_train_mnist[:n_train], y_train_mnist[:n_train], x_test_mnist[:n_test], y_test_mnist[:n_test]
 
 
-@pytest.mark.skipMlFramework("tensorflow1", "keras", "pytorch", "non_dl_frameworks", "mxnet", "kerastf")
+@pytest.mark.skip_framework("tensorflow1", "keras", "pytorch", "non_dl_frameworks", "mxnet", "kerastf")
 def test_generate_default(art_warning, fix_get_mnist_subset, image_dl_estimator):
     try:
         classifier, _ = image_dl_estimator(from_logits=True)
 
         attack = AutoAttack(
-            estimator=classifier, norm=np.inf, eps=0.3, eps_step=0.1, attacks=None, batch_size=32, estimator_orig=None,
+            estimator=classifier,
+            norm=np.inf,
+            eps=0.3,
+            eps_step=0.1,
+            attacks=None,
+            batch_size=32,
+            estimator_orig=None,
         )
 
         (x_train_mnist, y_train_mnist, x_test_mnist, y_test_mnist) = fix_get_mnist_subset
@@ -60,7 +66,7 @@ def test_generate_default(art_warning, fix_get_mnist_subset, image_dl_estimator)
         art_warning(e)
 
 
-@pytest.mark.skipMlFramework("tensorflow1", "keras", "pytorch", "non_dl_frameworks", "mxnet", "kerastf")
+@pytest.mark.skip_framework("tensorflow1", "keras", "pytorch", "non_dl_frameworks", "mxnet", "kerastf")
 def test_generate_attacks_and_targeted(art_warning, fix_get_mnist_subset, image_dl_estimator):
     try:
         classifier, _ = image_dl_estimator(from_logits=True)
@@ -139,6 +145,37 @@ def test_generate_attacks_and_targeted(art_warning, fix_get_mnist_subset, image_
         art_warning(e)
 
 
+@pytest.mark.skip_framework("tensorflow1", "keras", "pytorch", "non_dl_frameworks", "mxnet", "kerastf")
+def test_check_params(art_warning, image_dl_estimator_for_attack):
+    try:
+        from art.attacks.evasion import FastGradientMethod
+
+        classifier = image_dl_estimator_for_attack(AutoAttack)
+
+        attacks = [FastGradientMethod(estimator=classifier)]
+
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, norm=0)
+
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, eps="1")
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, eps=-1.0)
+
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, eps_step="1")
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, eps_step=-1.0)
+
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, batch_size=1.0)
+        with pytest.raises(ValueError):
+            _ = AutoAttack(classifier, attacks=attacks, batch_size=-1)
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
 @pytest.mark.framework_agnostic
 def test_classifier_type_check_fail(art_warning):
     try:
