@@ -43,9 +43,10 @@ class AudioInput:
 
     def get_data(self):
         if self.channels_first:
-            return np.zeros((self.batch_size, self.channels, self.sample_rate), dtype=np.int16)
+            shape = (self.batch_size, self.channels, self.sample_rate)
         else:
-            return np.zeros((self.batch_size, self.sample_rate, self.channels), dtype=np.int16)
+            shape = (self.batch_size, self.sample_rate, self.channels)
+        return np.zeros(shape, dtype=np.int16)
 
 
 @pytest.fixture(params=[1, 2], ids=["mono", "stereo"])
@@ -82,7 +83,7 @@ def test_non_temporal_data_error(art_warning, image_batch_small):
 
 
 @pytest.mark.parametrize("channels_first", [True, False])
-@pytest.mark.skipMlFramework("keras", "pytorch", "scikitlearn", "mxnet")
+@pytest.mark.skip_framework("keras", "pytorch", "scikitlearn", "mxnet")
 def test_mp3_compresssion(art_warning, audio_batch, channels_first):
     try:
         test_input, test_output, sample_rate = audio_batch
@@ -91,3 +92,32 @@ def test_mp3_compresssion(art_warning, audio_batch, channels_first):
         assert_array_equal(mp3compression(test_input)[0], test_output)
     except ARTTestException as e:
         art_warning(e)
+
+
+@pytest.mark.parametrize("channels_first", [True, False])
+@pytest.mark.skip_framework("keras", "pytorch", "scikitlearn", "mxnet")
+def test_mp3_compresssion_object(art_warning, audio_batch, channels_first):
+    try:
+        test_input, test_output, sample_rate = audio_batch
+        test_input_object = np.array([x for x in test_input], dtype=object)
+        mp3compression = Mp3Compression(sample_rate=sample_rate, channels_first=channels_first)
+
+        assert_array_equal(mp3compression(test_input_object)[0], test_output)
+
+        grad = mp3compression.estimate_gradient(x=test_input_object, grad=np.ones_like(test_input_object))
+
+        assert grad.dtype == object
+        assert grad.shape == (2, test_input.shape[1], 44100) if channels_first else (2, 44100, test_input.shape[2])
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
+@pytest.mark.skip_framework("keras", "pytorch", "scikitlearn", "mxnet")
+def test_check_params(art_warning):
+    try:
+        with pytest.raises(ValueError):
+            _ = Mp3Compression(sample_rate=1000, verbose="False")
+
+    except ARTTestException as e:
+        art_warning(e)
