@@ -233,6 +233,13 @@ class TestReverseSigmoid(unittest.TestCase):
         np.testing.assert_array_almost_equal(preds, classifier_prediction_expected, decimal=4)
         np.testing.assert_array_almost_equal(post_preds, post_classifier_prediction_expected, decimal=4)
 
+    def test_check_params(self):
+        with self.assertRaises(ValueError):
+            _ = ReverseSigmoid(beta=-1.0, gamma=0.5)
+
+        with self.assertRaises(ValueError):
+            _ = ReverseSigmoid(beta=-1.0, gamma=-0.5)
+
 
 if __name__ == "__main__":
     unittest.main()
