@@ -37,7 +37,7 @@ from art.utils import load_dataset
 logger = logging.getLogger(__name__)
 
 # ----------------------------------------------------------------------------------------------------- TEST BASE CLASS
-art_supported_frameworks = ["keras", "tensorflow", "pytorch", "scikitlearn"]
+art_supported_frameworks = ["keras", "tensorflow", "tensorflow2v1", "pytorch", "scikitlearn"]
 
 
 class TestBase(unittest.TestCase):
@@ -67,8 +67,6 @@ class TestBase(unittest.TestCase):
         cls._x_test_iris_original = cls.x_test_iris.copy()
         cls._y_test_iris_original = cls.y_test_iris.copy()
 
-        import warnings
-
         # Filter warning for scipy, removed with scipy 1.4
         warnings.filterwarnings("ignore", ".*the output shape of zoom.*")
 
@@ -135,7 +133,7 @@ def check_adverse_example_x(x_adv, x_original, max=1.0, min=0.0, bounded=True):
         assert np.amax(x_adv) <= max, "x_test_adv values should have all been below {0}".format(max)
         assert np.amin(x_adv) >= min, "x_test_adv values should have all been above {0}".format(min)
     else:
-        assert (x_adv > max).any(), "some x_test_adv values should have been above 1".format(max)
+        assert (x_adv > max).any(), "some x_test_adv values should have been above {0}".format(max)
         assert (x_adv < min).any(), " some x_test_adv values should have all been below {0}".format(min)
 
 
@@ -146,7 +144,7 @@ def check_adverse_predicted_sample_y(y_pred_adv, y_non_adv):
 def is_valid_framework(framework):
     if framework not in art_supported_frameworks:
         raise Exception(
-            "mlFramework value {0} is unsupported. Please use one of these valid values: {1}".format(
+            "Framework value {0} is unsupported. Please use one of these valid values: {1}".format(
                 framework, " ".join(art_supported_frameworks)
             )
         )
@@ -331,9 +329,9 @@ def get_image_classifier_tf_v2(from_logits=False):
     """
     # pylint: disable=E0401
     import tensorflow as tf
-    from tensorflow.keras import Model
+    from tensorflow.keras.layers import Conv2D, Dense, Flatten, MaxPool2D
     from tensorflow.keras.models import Sequential
-    from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPool2D
+
     from art.estimators.classification.tensorflow import TensorFlowV2Classifier
 
     if tf.__version__[0] != "2":
@@ -426,8 +424,8 @@ def get_image_classifier_kr(
         is_tf23_keras24 = True
         tf.compat.v1.disable_eager_execution()
         from tensorflow import keras
+        from tensorflow.keras.layers import Conv2D, Dense, Flatten, MaxPooling2D
         from tensorflow.keras.models import Sequential
-        from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
     else:
         is_tf23_keras24 = False
         import keras
@@ -599,11 +597,12 @@ def get_image_classifier_kr(
 
 
 def get_image_classifier_kr_functional(input_layer=1, output_layer=1):
-    from art.estimators.classification.keras import KerasClassifier
     import keras
     from keras.layers import Conv2D, Dense, Dropout, Flatten, Input, MaxPooling2D
     from keras.models import Model
 
+    from art.estimators.classification.keras import KerasClassifier
+
     def _functional_model():
         in_layer = Input(shape=(28, 28, 1), name="input0")
         layer = Conv2D(32, kernel_size=(3, 3), activation="relu")(in_layer)
@@ -643,16 +642,17 @@ def get_image_classifier_kr_functional(input_layer=1, output_layer=1):
 
 def get_image_classifier_kr_tf_functional(input_layer=1, output_layer=1):
     """
-       Standard Keras_tf classifier for unit testing built with a functional model
+    Standard Keras_tf classifier for unit testing built with a functional model
 
-       :return: KerasClassifier
-       """
+    :return: KerasClassifier
+    """
     import tensorflow as tf
 
     if tf.__version__[0] == "2":
         tf.compat.v1.disable_eager_execution()
     from tensorflow.keras.layers import Conv2D, Dense, Dropout, Flatten, Input, MaxPooling2D
     from tensorflow.keras.models import Model
+
     from art.estimators.classification.keras import KerasClassifier
 
     def functional_model():
@@ -712,8 +712,8 @@ def get_image_classifier_kr_tf(loss_name="categorical_crossentropy", loss_type="
 
     if tf.__version__[0] == "2":
         tf.compat.v1.disable_eager_execution()
+    from tensorflow.keras.layers import Conv2D, Dense, Flatten, MaxPooling2D
     from tensorflow.keras.models import Sequential
-    from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
 
     from art.estimators.classification.keras import KerasClassifier
 
@@ -863,8 +863,8 @@ def get_image_classifier_kr_tf_binary():
 
     if tf.__version__[0] == "2":
         tf.compat.v1.disable_eager_execution()
+    from tensorflow.keras.layers import Conv2D, Dense, Flatten, MaxPooling2D
     from tensorflow.keras.models import Sequential
-    from tensorflow.keras.layers import Dense, Flatten, Conv2D, MaxPooling2D
 
     from art.estimators.classification.keras import KerasClassifier
 
@@ -901,8 +901,8 @@ def get_image_classifier_kr_tf_with_wildcard():
 
     if tf.__version__[0] == "2":
         tf.compat.v1.disable_eager_execution()
+    from tensorflow.keras.layers import LSTM, Conv1D, Dense
     from tensorflow.keras.models import Sequential
-    from tensorflow.keras.layers import Dense, Conv1D, LSTM
 
     from art.estimators.classification.keras import KerasClassifier
 
@@ -930,8 +930,7 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
     :return: PyTorchClassifier
     """
     import torch
-    import torch.nn as nn
-    import torch.optim as optim
+
     from art.estimators.classification.pytorch import PyTorchClassifier
 
     class Model(torch.nn.Module):
@@ -1010,6 +1009,40 @@ def get_image_classifier_pt(from_logits=False, load_init=True):
     return ptc
 
 
+def get_image_classifier_pt_functional():
+    """
+    Simple PyTorch functional classifier for unit testing.
+    """
+    import torch.nn as nn
+    import torch.optim as optim
+
+    from art.estimators.classification import PyTorchClassifier
+
+    model = nn.Sequential(
+        nn.Conv2d(1, 4, 5),
+        nn.ReLU(),
+        nn.MaxPool2d(2, 2),
+        nn.Conv2d(4, 10, 5),
+        nn.ReLU(),
+        nn.MaxPool2d(2, 2),
+        nn.Flatten(),
+        nn.Linear(4 * 4 * 10, 100),
+        nn.Linear(100, 10),
+    )
+    criterion = nn.CrossEntropyLoss()
+    optimizer = optim.Adam(model.parameters(), lr=0.01)
+
+    classifier = PyTorchClassifier(
+        model=model,
+        clip_values=(0, 1),
+        loss=criterion,
+        optimizer=optimizer,
+        input_shape=(1, 28, 28),
+        nb_classes=10,
+    )
+    return classifier
+
+
 def get_classifier_bb(defences=None):
     """
     Standard BlackBox classifier for unit testing
@@ -1031,6 +1064,29 @@ def get_classifier_bb(defences=None):
     return bbc
 
 
+def get_classifier_bb_nn(defences=None):
+    """
+    Standard BlackBox Neural Network classifier for unit testing.
+
+    :return: BlackBoxClassifierNeuralNetwork
+    """
+    from art.estimators.classification.blackbox import BlackBoxClassifierNeuralNetwork
+    from art.utils import to_categorical
+
+    # define black-box classifier
+    def predict(x):
+        with open(
+            os.path.join(os.path.dirname(os.path.dirname(__file__)), "utils/data/mnist", "api_output.txt")
+        ) as json_file:
+            predictions = json.load(json_file)
+        return to_categorical(predictions["values"][: len(x)], nb_classes=10)
+
+    bbc = BlackBoxClassifierNeuralNetwork(
+        predict, (28, 28, 1), 10, clip_values=(0, 255), preprocessing_defences=defences
+    )
+    return bbc
+
+
 def get_image_classifier_mxnet_custom_ini():
     import mxnet
 
@@ -1073,6 +1129,7 @@ def get_image_classifier_mxnet_custom_ini():
 
 def get_gan_inverse_gan_ft():
     import tensorflow as tf
+
     from utils.resources.create_inverse_gan_models import build_gan_graph, build_inverse_gan_graph
 
     if tf.__version__[0] == "2":
@@ -1089,9 +1146,17 @@ def get_gan_inverse_gan_ft():
         sess = tf.Session()
         sess.run(tf.global_variables_initializer())
 
-        gan = TensorFlowGenerator(input_ph=z_ph, model=gen_tf, sess=sess,)
+        gan = TensorFlowGenerator(
+            input_ph=z_ph,
+            model=gen_tf,
+            sess=sess,
+        )
 
-        inverse_gan = TensorFlowEncoder(input_ph=image_to_enc_ph, model=enc_tf, sess=sess,)
+        inverse_gan = TensorFlowEncoder(
+            input_ph=image_to_enc_ph,
+            model=enc_tf,
+            sess=sess,
+        )
         return gan, inverse_gan, sess
 
 
@@ -1216,6 +1281,7 @@ def get_tabular_classifier_tf_v2():
     import tensorflow as tf
     from tensorflow.keras import Model
     from tensorflow.keras.layers import Dense
+
     from art.estimators.classification.tensorflow import TensorFlowV2Classifier
 
     if tf.__version__[0] != "2":
@@ -1280,15 +1346,14 @@ def get_tabular_classifier_tf_v2():
 
 
 def get_tabular_classifier_scikit_list(clipped=False, model_list_names=None):
-    from art.estimators.classification.scikitlearn import (
-        ScikitlearnDecisionTreeClassifier,
-        # ScikitlearnExtraTreeClassifier,
+    from art.estimators.classification.scikitlearn import (  # ScikitlearnExtraTreeClassifier,
         ScikitlearnAdaBoostClassifier,
         ScikitlearnBaggingClassifier,
+        ScikitlearnDecisionTreeClassifier,
         ScikitlearnExtraTreesClassifier,
         ScikitlearnGradientBoostingClassifier,
-        ScikitlearnRandomForestClassifier,
         ScikitlearnLogisticRegression,
+        ScikitlearnRandomForestClassifier,
         ScikitlearnSVC,
     )
 
@@ -1361,8 +1426,8 @@ def get_tabular_classifier_kr(load_init=True):
         is_tf23_keras24 = True
         tf.compat.v1.disable_eager_execution()
         from tensorflow import keras
-        from tensorflow.keras.models import Sequential
         from tensorflow.keras.layers import Dense
+        from tensorflow.keras.models import Sequential
     else:
         is_tf23_keras24 = False
         import keras
@@ -1560,6 +1625,7 @@ def get_attack_classifier_pt(num_features):
     """
     import torch.nn as nn
     import torch.optim as optim
+
     from art.estimators.classification.pytorch import PyTorchClassifier
 
     class AttackModel(nn.Module):
@@ -1578,7 +1644,7 @@ def get_attack_classifier_pt(num_features):
     loss_fn = nn.BCELoss()
     optimizer = optim.Adam(model.parameters(), lr=0.0001)
     attack_model = PyTorchClassifier(
-        model=model, loss=loss_fn, optimizer=optimizer, input_shape=(num_features,), nb_classes=1
+        model=model, loss=loss_fn, optimizer=optimizer, input_shape=(num_features,), nb_classes=2
     )
 
     return attack_model
